
/*C*

________________________________________________________________

        biff2matlab
        $Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
        Copyright 1994, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $, Blab, UiO";

#include <mat.h>

#include <xite/includes.h>
#include <xite/matlab.h>
#ifndef mxMAXNAM
# define mxMAXNAM 20
#endif
#include <xite/message.h>
#include <xite/debug.h>
#include XITE_STRING_H

#define OK_IND           0
#define CREATE_IND       1
#define CONVERT_IND      2
#define OPEN_MAT_IND     3
#define WRITE_MAT_IND    4
#define INCOMPATIBLE_IND 5
#define NOT_NUMERIC_IND  6
#define ARRAY_CLASS_IND  7
#define CLOSE_MAT_IND    8

static char *biff2matlab_error_s[] = {
  "OK\n",
  "Failure in creating MATLAB array.\n",
  "Failure in converting BIFF band to MATLAB array.\n",
  "Failure in opening MATLAB MAT file.\n",
  "Failure in writing MATLAB array to file.\n",
  "Data type of MATLAB array and pixel type of BIFF band are incompatible.\n",
  "MATLAB array is not numeric.\n",
  "Could not determine array class.\n",
  "Failure in closing MATLAB MAT file.\n"
};

#ifndef MAIN

static mxComplexity mxComplexityFromBand (const IBAND band) {
  IPIXTYP pixtyp       = Ipixtyp(band);
  mxComplexity complex = mxREAL;

  switch ((int) pixtyp) {
  case Icomplex_typ: /* Fall-through */
  case Id_complex_typ:
    complex = mxCOMPLEX;
    break;
  }

  return complex;

} /* mxComplexityFromBand */

static mxClassID mxClassIDFromBand (const IBAND band) {
  IPIXTYP pixtyp  = Ipixtyp(band);
  mxClassID class = mxUNKNOWN_CLASS;

  switch ((int) pixtyp) {
  case Iu_byte_typ:
    class = mxUINT8_CLASS;
    break;
  case Is_byte_typ:
    class = mxINT8_CLASS;
    break;
  case Iu_short_typ:
    class = mxUINT16_CLASS;
    break;
  case Is_short_typ:
    class = mxINT16_CLASS;
    break;
  case Integer_typ:
    class = mxINT32_CLASS;
    break;
  case Ireal_typ:
    class = mxSINGLE_CLASS;
    break;
  case Idouble_typ:
    class = mxDOUBLE_CLASS;
    break;
  case Icomplex_typ:
    class = mxSINGLE_CLASS;
    break;
  case Id_complex_typ:
    class = mxDOUBLE_CLASS;
    break;
  }

  return class;

} /* mxClassIDFromBand */

/*F:band2matlab*

________________________________________________________________

		band2matlab
________________________________________________________________

Name:		band2matlab - Convert from XITE BIFF band to MATLAB array.

Syntax:         | #include <xite/matlab.h>
                |
                | int band2matlab(const IBAND band, mxArray* array );

Description:    Converts the pixel data in the XITE BIFF band 'band'
                into the MATLAB array 'array'. 'band' may consist of
                any of the following pixel types:
		UNS_BYTE, SIGN_BYTE, UNS_SHORT, SIGN_SHORT, INTEGER, REAL,
		COMPLEX, DOUBLE, DCOMPLEX. 'array' must be of a compatible
                data type.

Return value:   | 0 => ok
                | 5 => Incompatible data types between array and band.
                | 6 => Array is not numeric.

Restrictions:   Storage for the MATLAB array must be allocated prior to
                calling this routine.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id: 		$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int band2matlab(const IBAND band, mxArray *array) {
/*
    MATLAB arrays are column-ordered, while BIFF bands are row-ordered.
*/

    long i, j, xsize, ysize;
    IPIXTYP pixTyp;

    ENTER_FUNCTION_DEBUG("band2matlab");

    if (!mxIsNumeric(array))
      return(Error(NOT_NUMERIC_IND, biff2matlab_error_s[NOT_NUMERIC_IND]));

    pixTyp = Ipixtyp(band);
    xsize  = Ixsize(band);
    ysize  = Iysize(band);

#   define CONVERT(INN_BAND_TYPE, MX_ELEMENT_TYPE) \
    { \
        MX_ELEMENT_TYPE *pr   = (MX_ELEMENT_TYPE *) mxGetData(array); \
	INN_BAND_TYPE in_band = (INN_BAND_TYPE) band; \
        \
        for (i = 1; i <= ysize; i++) \
          for (j = 1; j <= xsize; j++) \
      	    pr[i + (j - 1) * ysize - 1] = in_band[i][j]; \
    }

#   define C_CONVERT(INN_BAND_TYPE, MX_ELEMENT_TYPE) \
    { \
        MX_ELEMENT_TYPE *pr   = (MX_ELEMENT_TYPE *) mxGetData(array); \
        MX_ELEMENT_TYPE *pi   = (MX_ELEMENT_TYPE *) mxGetImagData(array); \
        INN_BAND_TYPE in_band = (INN_BAND_TYPE) band; \
        \
        for (i = 1; i <= ysize; i++) \
        for (j = 1; j <= xsize; j++) { \
  	  pr[i + (j - 1) * ysize - 1] = in_band[i][j].re; \
	  pi[i + (j - 1) * ysize - 1] = in_band[i][j].im; \
         } \
    }

    switch((int) pixTyp) {
    case Iu_byte_typ:

      if (!mxIsUint8(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
	
	
      CONVERT(IUB_BAND, uint8_T);

      break;
    case Is_byte_typ:

      if (!mxIsInt8(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ISB_BAND, int8_T);

      break;
    case Iu_short_typ:
      if (!mxIsUint16(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(IUS_BAND, uint16_T);

      break;
    case Is_short_typ:
      if (!mxIsInt16(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ISS_BAND, int16_T);

      break;
    case Integer_typ:
      if (!mxIsInt32(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(II_BAND, int32_T);

      break;
    case Ireal_typ:
      if (!mxIsSingle(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(IR_BAND, real32_T);

      break;
    case Icomplex_typ:
      if (!mxIsSingle(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      if (!mxIsComplex(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      C_CONVERT(IC_BAND, real32_T);

      break;
    case Idouble_typ:
      if (!mxIsDouble(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ID_BAND, real64_T);

      break;
    case Id_complex_typ:
      if (!mxIsDouble(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      if (!mxIsComplex(array))
	return(Error(INCOMPATIBLE_IND,
		     biff2matlab_error_s[INCOMPATIBLE_IND]));
      
      C_CONVERT(IDC_BAND, real64_T);

      break;
    }

    LEAVE_FUNCTION_DEBUG("band2matlab");

    return(Iok);

} /* band2matlab */

/*L:arrayName*

________________________________________________________________

		arrayName
________________________________________________________________

Name:		arrayName - Generate name of MATLAB array from name of
                XITE BIFF image or default name

Syntax:		| static char *arrayName( char *name, IMAGE img,
                |    char *defaultName, long bandNo );

Description:    Generate name of MATLAB array from name of BIFF image or
                default name.

		If the BIFF image has a non-empty title, the name is generated
		from it (with an extension for multiband images). If the BIFF
		image has no non-empty title, the name is created from
		'defaultName' (and band number). If 'defaultName' is also
		NULL or empty, the name becomes "no_name" (with a possible band
		extension).

		A MATLAB array name must start with a letter and only consist
		of letters, digits or underscores (a total of maximum mxMAXNAM
		characters, typically 64).  An illegal initial character is
		replaced by the letter "X", any other character is replaced by
		an underscore.

Restrictions:   Storage must be allocated for 'name' prior to calling this
                routine. It should be able to hold mxMAXNAM characters (and an
                additional string terminator).

Return value:   Pointer to MATLAB array name
Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id: 		$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

static char *arrayName(char *name, IMAGE img, char *defaultName, long bandNo) {
    char *imgName, extension[80], *t, *legalChars;
    int len;

    ENTER_FUNCTION_DEBUG("arrayName");

    imgName = Ititle(img);

    /* Try to avoid an an empty string. */
    if (imgName == NULL || strlen(imgName) == 0) imgName = defaultName;
    if (imgName == NULL || strlen(imgName) == 0) {
      imgName    = mxCalloc((size_t) mxMAXNAM + 1, sizeof(char));
      /* imgName now contains all zeros, i.e. already terminated length zero */
      strcpy(imgName, "no_name");
      /* imgName now has length greater than zero */
    }

    /* imgName now has length at least equal to one. */

    /* Make sure the first character is a letter, and substitute anything but
       letters, digits and underscores with an underscore */

    legalChars =
      "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    if (strspn(imgName, legalChars) < 1) imgName[0] = 'X';

    legalChars =
      "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
    t = imgName;
    while (1) {
      len = strspn(t, legalChars);
      if (len == strlen(t)) break; /* The rest of the name is legal. */
      else {
        t[len] = '_';
	t += len;
      }
    }

    (void) strncpy(name, imgName, mxMAXNAM + 1);

    if (Inbands(img) > 1) {
      /* Add band-extension */
      if (strlen(name) < mxMAXNAM) (void) strcat(name, "_");
      (void) sprintf(extension, "%ld", bandNo);
      if (strlen(name) + strlen(extension) <= mxMAXNAM)
	(void) strcat(name, extension);
    }

    /* Limit length to maximum MATLAB name-string length */
    if (strlen(name) > mxMAXNAM) name[mxMAXNAM] = '\0';

    LEAVE_FUNCTION_DEBUG("arrayName");

    return(name);

} /* arrayName */

/*F:biff2matlab*

________________________________________________________________

		biff2matlab
________________________________________________________________

Name:		biff2matlab - Convert all bands of XITE BIFF image to MATLAB
                arrays

Syntax:         | #include <xite/matlab.h>
                |
                | int biff2matlab( const IMAGE img, long* bandNo,
                |    long nbands, mxArray** arrays, char *defaultName,
		|    char **matNames );

Description:    Convert the 'nbands' bands (listed in 'bandNo') of a BIFF image
                'img' to MATLAB arrays. 'bandNo' determines the order in which
		the bands are taken from 'img', i.e. band number 'bandNo'[i]
		(i=0, ..., nbands-1) will be stored in 'arrays'[i]. Remember
		that 'bandNo'[i] must be greater than zero because bands are
		numbered from one. If 'bandNo' is NULL, store band number
		i in arrays[i-1].

		The data type of the MATLAB arrays will be compatible with the
		data type of the corresponding BIFF image band. Be aware that
		this routine may require lots of memory. It may be better to
		convert one band at a time if you simply want to store them in
		a file (see 'band2matlab(3)' and 'saveBiffAsMatlab(3)').

		'matNames' will return names for the corresponding arrays
		in 'arrays', unless 'matNames' is NULL. If the BIFF image
		has a non-empty title, the array name(s) is (are)
		generated from it (with an extension for multiband
		images). If the BIFF image has no non-empty title, the
		array name(s) is (are) created from 'defaultName' (and
		band number). If also 'defaultName' is NULL or empty, the name
		becomes "no_name" (with a possible band extension).

                A MATLAB array name must start with a letter and only
                consist of letters, digits and underscores (a total of
                maximum mxMAXNAM characters, typically 64). An illegal
		initial character is replaced by the letter "X", any other
		character is replaced by an underscore.

		For a BIFF image with more than one band, all the arrays will
		be returned in a MATLAB struct of length one, with each
		field name equal to the array name.

Restrictions:   Memory for the arrays will be allocated by the routine, but
                the pointer array 'arrays' must be allocated prior to the
		call. Memory for the 'matNames' pointer array must be allocated
		prior to the call (unless it is NULL), while the individual
		name elements will be allocated by the routine.

Return value:   | 0 => OK
                | 1 => Failure in creating MATLAB array.
		| 2 => Failure in converting BIFF image to MATLAB array.
                | 5 => Incompatible data types between array and band.
                | 6 => Array is not numeric.
		| 7 => Could not determine array class.

See also:	'band2matlab(3)', 'saveBiffAsMatlab(3), 'matlab2biff(3)'
Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id: 		$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int biff2matlab(const IMAGE img, long *bandNo, long nbands, mxArray **arrays, char *defaultName, char **matNames) {
    long ind, bandNum;
    IBAND band;
    mxArray *array;
    char *name;
    mxClassID class;
    mxComplexity complexity;
    int free_bandNo = FALSE;
    int status;

    ENTER_FUNCTION_DEBUG("biff2matlab");
    
    if (bandNo == NULL) {
      bandNo      = (long *) mxCalloc((size_t) nbands, sizeof(long));
      free_bandNo = TRUE;

      for (ind = 0; ind < nbands; ind++)
        bandNo[ind] = ind + 1;
    }

    for (ind = 0; ind < nbands; ind++) {
      bandNum    = bandNo[ind];
      band       = img[bandNum];
      class      = mxClassIDFromBand(band);
      if (class == mxUNKNOWN_CLASS) {
	LEAVE_FUNCTION_DEBUG("biff2matlab");
	return(Error(ARRAY_CLASS_IND, biff2matlab_error_s[ARRAY_CLASS_IND]));
      }

      complexity = mxComplexityFromBand(band);
      array      = mxCreateNumericMatrix(Iysize(band), Ixsize(band),
					 class, complexity);
      if (array == NULL) {
	LEAVE_FUNCTION_DEBUG("biff2matlab");
	return(Error(CREATE_IND, biff2matlab_error_s[CREATE_IND]));
      }

      name = (char *) mxMalloc(mxMAXNAM + 1);
      (void) arrayName(name, img, defaultName, bandNum);

      if (matNames != NULL) {
	if (strlen(name) > 0) matNames[ind] = name;
	else mxFree((void *) name);
      }

      if ((status = band2matlab(band, array)) != Iok) {
	LEAVE_FUNCTION_DEBUG("biff2matlab");
	return(Error(status, biff2matlab_error_s[status]));
      }

      arrays[ind] = array;
    }

    if (nbands > 1) {
      array = mxCreateStructMatrix(1, 1, nbands, (const char **) matNames);

      for (ind = 0; ind < nbands; ind++) {
	mxSetFieldByNumber(array, 0, ind, arrays[ind]);
      }

      arrays[0] = array;
    }

    if (free_bandNo) mxFree(bandNo);

    LEAVE_FUNCTION_DEBUG("biff2matlab");

    return(Iok);

} /* biff2matlab */

/*F:saveBiffAsMatlab*

________________________________________________________________

		saveBiffAsMatlab
________________________________________________________________

Name:		saveBiffAsMatlab - Save XITE BIFF image in MATLAB MAT file.

Syntax:         | #include <xite/matlab.h>
                |
                | int saveBiffAsMatlab( IMAGE img, char* matFileName );

Description:    Convert the XITE BIFF image 'img' to MATLAB arrays and
                save to MAT file for MATLAB version 5. The data type of
		each array will be compatible with the data type of the
		corresponding BIFF band.

		If the BIFF image has a title, the MATLAB array (or arrays) is
		(are) named after it (with an extension for multiband
		images). If the BIFF image has no title, the MATLAB array
		name(s) is (are) created from the MAT filename (and band
		number). With also filename NULL or "", the name becomes
		"no_name" (with a possible band extension).

		A MATLAB array name must start with a letter and only
		consist of letters, digits and underscores (a total of
		maximum mxMAXNAM characters, typically 64). An illegal
		initial character is replaced by the letter "X", any other
		character is replaced by an underscore.

		If the MATLAB MAT file exists, the array is appended to
		the file (unless an array with the same name already exists,
		in which case the new array replaces the old).

Return value:   | 0 => OK
		| 1 => Failure in creating MATLAB array.
		| 2 => Failure in converting BIFF band to MATLAB array.
                | 3 => Failure in opening MATLAB MAT file.
		| 4 => Failure in writing MATLAB array to file.
                | 5 => Incompatible data types between array and band.
                | 6 => Array is not numeric.
		| 7 => Could not determine array class.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.

Id: 		$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int saveBiffAsMatlab(IMAGE img, char *matFileName) {
    mxArray *array;
    MATFile *matFile;
    long nbands, bandNum;
    IBAND band;
    mxClassID class;
    mxComplexity complexity;
    char matName[mxMAXNAM + 1];
    int status;

    ENTER_FUNCTION_DEBUG("saveBiffAsMatlab");

    nbands = Inbands(img);

    if ((matFile = matOpen(matFileName, "w")) == NULL) {
      LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
      return(Error(OPEN_MAT_IND, biff2matlab_error_s[OPEN_MAT_IND]));
    }

    /* Process each BIFF band separately instead of calling biff2matlab.
       This saves memory.
    */

    for (bandNum = 1; bandNum <= nbands; bandNum++) {
	band       = img[bandNum];
	class      = mxClassIDFromBand(band);
	if (class == mxUNKNOWN_CLASS) {
	  LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
	  return(Error(ARRAY_CLASS_IND, biff2matlab_error_s[ARRAY_CLASS_IND]));
	}

	complexity = mxComplexityFromBand(band);
	array      = mxCreateNumericMatrix(Iysize(band), Ixsize(band),
					   class, complexity);
	if (array == NULL) {
	  LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
	  return(Error(CREATE_IND, biff2matlab_error_s[CREATE_IND]));
	}

	(void) arrayName(matName, img, matFileName, bandNum);

	if((status = band2matlab(band, array)) != Iok) {
	  LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
	  return(Error(status, biff2matlab_error_s[status]));
	}

	if (matPutVariable(matFile, matName, array) != Iok) {
	  LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
	  return(Error(WRITE_MAT_IND, biff2matlab_error_s[WRITE_MAT_IND]));
	}

	mxDestroyArray(array);
    }

    if (matClose(matFile) != Iok) {
      LEAVE_FUNCTION_DEBUG("saveBiffAsMatlab");
      return(Error(CLOSE_MAT_IND, biff2matlab_error_s[CLOSE_MAT_IND]));
    }

    return(Iok);

} /* saveBiffAsMatlab */

#endif /* not MAIN */

/*P:biff2matlab*

________________________________________________________________

		biff2matlab
________________________________________________________________

Name:		biff2matlab - Convert from XITE BIFF image file format to
                MATLAB array file format.

Syntax:		biff2matlab <biffFile> <matFile>

Description:    Convert from XITE BIFF image file format to MATLAB version 5
                MAT-file format. Each band may consist of any of the
		following pixel	types: UNS_BYTE, SIGN_BYTE, UNS_SHORT,
		SIGN_SHORT, INTEGER, REAL, COMPLEX, DOUBLE, DCOMPLEX.

		"-", "" or "-0" can be used for 'stdin'. The input filename
		may be followed by band specifications, as in file:3 for the
		third band (see 'Iread_image(3)').
		
		If the BIFF image has a non-empty title, the name (names)
		is (are) generated from it (with a filename extension for
		multiband images). If the BIFF image has no non-empty title,
		the name(s) is (are) created from the MATLAB MAT filename
		(and band number). If the filename is empty too, the name 
		becomes "no_name" (with a possible band extension).

		A MATLAB array name must start with a letter and only
		consist of letters, digits or underscores (a total of
		maximum mxMAXNAM characters, typically 64). An illegal
		initial character is replaced by the letter "a", any other
		character is replaced by an underscore.

		If the MATLAB array file exists, the array is appended to
		the file (unless an array with the same name already exists,
		in which case the new array replaces the old).

See also:	'matlab2biff(1)', 'Iread_image(5)'

Return value:   | 0 => OK
                | 1 => Usage message
                | 2 => Illegal command line
		| 3 => Failure in creating MATLAB array
		| 4 => Failure in converting BIFF band to MATLAB array
                | 5 => Failure in opening MATLAB array file
		| 6 => Failure in writing MATLAB array to file

Author:		Svein Bøe, BLAB, Ifi, UiO

Id: 		$Id: biff2matlab.c,v 1.32 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

#ifdef MAIN

int main(int argc, char **argv) {
    IMAGE img;
    int status;

    /* Initialize XITE program */

    Iset_message(1);
    Iset_abort(1);
    InitMessage(&argc, argv, xite_app_std_usage_text(
      "Usage: %s <biffFile> <matlabFile>\n\
        '-' can be used for stdin.\n"));

    /* stdin if name is "-", "" or "-0" (XITE standard) */

    if (argc == 1) Usage(1, NULL);
    if (argc != 3) Usage(2, "Illegal number of arguments.\n");

    img = Iread_image(argv[1]);

    if ((status = saveBiffAsMatlab(img, argv[2])) != Iok)
      Error(status, biff2matlab_error_s[status]);

    return(0);
}

#endif /* MAIN */

