
/*C*

________________________________________________________________

        loadBiff
        $Id: loadBiff.c,v 1.20 2014/01/31 12:52:07 svein Exp $
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: loadBiff.c,v 1.20 2014/01/31 12:52:07 svein Exp $, Blab, UiO";

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/message.h>
#include <xite/debug.h>
#include <mex.h>
#include <xite/mex.h>

#define FILENAME prhs[0]
#define BANDS    prhs[1]

static void setBandNo(long **bandNo, long start, long nMat) {
  long ind;

  *bandNo = (long *) mxCalloc((size_t) nMat, sizeof(long));
  for (ind = 1; ind <= nMat; ind++)
    (*bandNo)[ind - 1] = ind + start - 1;
}

/*L:mexFunction*

________________________________________________________________

		mexFunction
________________________________________________________________

Name:		mexFunction
Syntax:		| #include <xite/matlab.h>
                |
		| void mexFunction( int nlhs, mxArray *plhs[],
		|    int nrhs, const mxArray *prhs[] );
		|

Description:    Load image from XITE BIFF image file into MATLAB.
                prhs (input arguments) are
                |   - BIFF image filename
                |   - band numbers to load (starting at one). Non-existent
		|     or (-1) defaults to all bands. This argument must be
		|     a vector or a scalar, assuming type double. Remember
		|     that bands are numbered from one.

		Without output arguments (nlhs = 0), the BIFF image bands
		appear as variables in the callers workspace. With nlhs = 1,
		the contents of the BIFF file are returned in a struct with
		fields matching the variables. See MATLAB's own load-function.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id:             $Id: loadBiff.c,v 1.20 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

/*M*

%loadBiff  Load image from XITE BIFF file
%
% SYNOPSIS:
%   loadBiff(file, bands)
%   s = loadBiff(file, bands);
% OUTPUT:
%   s : A struct with fields matching the bands retrieved from the file.
% INPUT:
%   file        : String with name of XITE BIFF image file.
%   bands       : Scalar or vector containing requested band numbers
%                 (default, if missing or -1, is all bands in the file). Must
%                 be MATLAB default type double. Remember that bands are
%                 numbered from one.
% DESCRIPTION:
%   Load XITE BIFF image from file.
%   Without output arguments the BIFF image bands appear as variables in
%   the callers workspace. With one output argument, the contents of
%   the BIFF file are returned in a struct with fields matching
%   the variables. See MATLAB's own load-function.
%
%   The names of the array variables are generated from the image title,
%   with possible band extensions (if no title, then from the filename).

% AUTHOR:
%   Svein Bøe, Ifi Uio, 1994, 1997, 2002, 2003.
%

*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  IMAGE img;
  long nbands, n_dim, ind, *bandNo = NULL;
  mxArray **matrices;
  int argc = 1, level;
  char *biffFileName, **matNames;
  char *argv[] = {"loadBiff"};

  ENTER_FUNCTION_DEBUG("mexFunction for loadBiff");

  Iset_message(1);
  InitMessage(&argc, argv, NULL);
  level = PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab,
		      ExitOnError());
  FPRINTF2("level=%d\n", level);

  if (nrhs < 1) errorMsgToMatlab(2, "Missing file name argument.");
  if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.");
  if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.");

  if (!mxIsChar(FILENAME))
    errorMsgToMatlab(2, "File name argument must be string.");

  n_dim = (long) mxGetN(FILENAME);
  FPRINTF2("n_dim=%ld (length of filename)\n", n_dim);

  biffFileName = (char *) mxCalloc((size_t) (n_dim + 1), sizeof(char));
  mxGetString(FILENAME, biffFileName, (int) (n_dim + 1));
  FPRINTF2("biffFileName=%s\n", biffFileName);

  if (Iopen_image(&img, biffFileName, Ireadonly) != Iok)
    errorMsgToMatlab(2, "Could not open BIFF image file.");
    
  nbands = Inbands(img);
  FPRINTF2("nbands=%ld\n", nbands);

  /*** Check right-hand side (input) arguments ***/

  if (nrhs != 2) {
    /* Set default - all bands. */

    FPRINTF1("Default - all bands.\n");
    setBandNo(&bandNo, 1, nbands);
  } else { /* nrhs must equal two. */
    /* Get input band number argument and check validity */
    long m_dim;

    if (!mxIsDouble(BANDS)) {
      errorMsgToMatlab(2, "Band specification must be MATLAB default type double.");
    }

    /* Validate dimension */
    m_dim = (long) mxGetM(BANDS);
    n_dim = (long) mxGetN(BANDS);
    FPRINTF3("m=%ld, n=%ld\n", m_dim, n_dim);
    if (m_dim > 1 && n_dim > 1)
      errorMsgToMatlab(2, "Band number argument must be vector.");
    if (m_dim > n_dim) n_dim = m_dim;

    if (m_dim == 1 && n_dim == 1 && (int) (mxGetPr(BANDS)[0]) < 0) {
      /* Set default - all bands. */
      FPRINTF1("Default - all bands, because of negative argument.\n");
      setBandNo(&bandNo, 1, nbands);
    } else {
      /* Check validity and set the required bands. */
      FPRINTF1("Bands specified.\n");

      setBandNo(&bandNo, 1, n_dim); /* Just for allocation. */

      for (ind = 1; ind <= n_dim; ind++) {
	if ((int) (mxGetPr(BANDS)[ind - 1]) > nbands ||
	    (int) (mxGetPr(BANDS)[ind - 1]) <  0)
	  errorMsgToMatlab(2, "Band number does not exist in file.");
	else bandNo[ind - 1] = (long) (mxGetPr(BANDS)[ind - 1]);
      }
      nbands = n_dim;
    }
  }

  FPRINTF1("Ready to read bands.\n");

  /* Get the required BIFF bands */
  for (ind = 1; ind <= nbands; ind++) {
    if (Iread_band(img, (int) bandNo[ind - 1]) != Iok)
      errorMsgToMatlab(2, "Failure in reading band.");
  }

  Iclose_image(img);

  matNames = (char **) mxCalloc((size_t) nbands, sizeof(char *));
  matrices = (mxArray **) mxCalloc((size_t) nbands, sizeof(mxArray *));

  if (biff2matlab(img, bandNo, nbands, matrices, biffFileName, matNames)
      != Iok)
    errorMsgToMatlab(2, "Failure in converting BIFF image to MATLAB arrays.");

  if (nlhs == 0) {
    /* Return in callers workspace. */

    if (nbands > 1) {
      /* biff2matlab returned arrays in struct. */

      for (ind = 0; ind < nbands; ind++) {
	mexPutVariable("caller", matNames[ind],
		       mxGetFieldByNumber(matrices[0], 0, ind));
      }
    } else {
      for (ind = 0; ind < nbands; ind++) {
	mexPutVariable("caller", matNames[ind], matrices[ind]);
      }
    }
  } else {
    /* Return in struct. */

    if (nbands > 1) {
      /* Already struct. */

      plhs[0] = matrices[0];
    } else {
      plhs[0] = mxCreateStructMatrix(1, 1, nbands, (const char **) matNames);
      for (ind = 0; ind < nbands; ind++) {
	mxSetFieldByNumber(plhs[0], 0, ind, matrices[ind]);
      }
    }
  }

  mxFree(biffFileName);
  mxFree(matNames);
  mxFree(matrices);
  if (bandNo != NULL) mxFree(bandNo);

  level = PopMessage();
  FPRINTF2("level=%d\n", level);
}
