
/*C*

________________________________________________________________

        matlab2biff
        $Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
        Copyright 1990-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $, Blab, UiO";

#include <mat.h>

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/message.h>
#include <xite/readarg.h>
#include XITE_MALLOC_H
#include XITE_STRING_H
#include <xite/debug.h>

#define biffMAXLEN 32

#define OK_IND               0
#define USAGE_IND            1
#define ILLEGAL_CMD_IND      2
#define OPEN_MAT_IND         3
#define READ_MAT_IND         4
#define NOT_FOUND_IND        5
#define INIT_BIFF_IND        6
#define CREATE_IND           7
#define CONVERT_IND          8
#define OPEN_IND             9
#define WRITE_IND           10
#define INCOMPATIBLE_IND    11
#define NOT_NUMERIC_IND     12

static char *matlab2biff_error_s[] = {
  "OK\n",
  "Error\n",
  "Illegal command line.\n",
  "Failure in opening the MAT-file.\n",
  "Failure in reading the MAT-file directory.\n",
  "Did not find all requested arrays in file.\n",
  "Failure in initializing BIFF image.\n",
  "Failure in creating BIFF image band.\n",
  "Failure in converting MATLAB array to XITE BIFF band.\n",
  "Failure in opening BIFF image file.\n",
  "Failure in writing BIFF image band to file.\n",
  "Data type of MATLAB array and pixel type of BIFF band are incompatible.\n",
  "MATLAB array is not numeric.\n"
};

#ifndef MAIN

static IPIXTYP pixtypFromMxArray (const mxArray *array){
  mxClassID class = mxGetClassID(array);
  IPIXTYP pixtyp;

  ENTER_FUNCTION_DEBUG("pixtypFromMxArray");

  if (!mxIsNumeric(array)) {
    pixtyp = Iunknown_typ;
  } else
  if (class == mxUINT8_CLASS)  pixtyp = Iu_byte_typ;  else
  if (class == mxINT8_CLASS)   pixtyp = Is_byte_typ;  else
  if (class == mxUINT16_CLASS) pixtyp = Iu_short_typ; else
  if (class == mxINT16_CLASS)  pixtyp = Is_short_typ; else
  if (class == mxINT32_CLASS)  pixtyp = Integer_typ;  else
  if (class == mxSINGLE_CLASS) {
    if (mxIsComplex(array)) pixtyp = Icomplex_typ;
    else pixtyp = Ireal_typ;
  } else
  if (class == mxDOUBLE_CLASS) {
    if (mxIsComplex(array)) pixtyp = Id_complex_typ;
    else pixtyp = Idouble_typ;
  } else 
  if (class == mxUINT32_CLASS) {
    Warning(1, "Unsigned 32-bit integer not in XITE, using signed.\n");
    pixtyp = Integer_typ;
  } else
  if (class == mxUINT64_CLASS) {
    Warning(1, "64-bit integer not in XITE, using signed 32-bit.\n");
    pixtyp = Integer_typ;
  } else
  if (class == mxINT64_CLASS) {
    Warning(1, "64-bit integer not in XITE, using signed 32-bit.\n");
    pixtyp = Integer_typ;
  } else {
    pixtyp = Iunknown_typ;
  }

  LEAVE_FUNCTION_DEBUG("pixtypFromMxArray");

  return pixtyp;

} /* pixtypFromMxArray */

/*L:biffName*
  
________________________________________________________________
 
		biffName
________________________________________________________________

Name:		biffName - Generate title for XITE BIFF image from MATLAB
                array names or default name

Syntax:		| static char *biffName( char *name,
                |    char *matNames[],
                |    char *defaultName, long nMat );

Description:    Generate title of XITE BIFF image from MATLAB array names
                in 'matNames' ('nMat' entries) or a default name.

		The first non-empty array name is used, or else 'defaultName'
		(if non-empty), or else "no_name".  'matNames' may be NULL.

		New title is returned in 'name'.

Restrictions:   Storage must be allocated for 'name' prior to the call,
                at least enough for a valid BIFF image title (typically
                32 characters).

Return value:   Pointer to BIFF image name

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.

Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

static char *biffName(char *name, char *matNames[], char *defaultName, long nMat) {
  char *arrayName;
  long i;

  ENTER_FUNCTION_DEBUG("biffName");

  arrayName = "\0";

  if (matNames != NULL) {
    i = 1;
    while (strlen(arrayName) == 0 && i <= nMat) {
      arrayName = matNames[i-1];
      i++;
    }
  }
  if (arrayName == NULL) arrayName = "\0";

  /* Remove any trailing blanks */
  if (strlen(arrayName) > strcspn(arrayName, " ")) {
      arrayName[strcspn(arrayName, " ")] = '\0';
  }

  /* Check for an empty string and a decent output filename */
  if (strlen(arrayName) == 0    &&
      defaultName       != NULL &&
      strlen(defaultName) > 0) {
      arrayName = defaultName;
  }

  if (strlen(arrayName) == 0) arrayName = "no_name";
  (void) strcpy(name, arrayName);

  LEAVE_FUNCTION_DEBUG("biffName");

  return(name);

} /* biffName */

/*L:matlabStruct2Arrays*

________________________________________________________________

		matlabStruct2Arrays
________________________________________________________________

Name:		matlabStruct2Arrays - Unpack MATLAB struct array and return
                the struct fields arrays

Syntax:         | static int matlabStruct2Arrays( const mxArray *array,
                |    mxArray **arrays, char ***names, long *nArrays );

Description:    If 'array' refers to a MATLAB struct, return the individual
                struct fields in 'arrays' and the corresponding names in
		'names'. 'nArrays' returns the number of returned arrays.

		Memory for 'arrays' and 'names' will be allocated by the
		routine.

Return value:   | 0 => ok

Author:		Svein Bøe, BLAB, Ifi, UiO, 2003.

Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

static int matlabStruct2Arrays (const mxArray *array, mxArray ***arrays, char ***names, long *nArrays) {
  long ind;

  if (!mxIsStruct(array)) {
    *nArrays = 0;
    return(Iok);
  }

  *nArrays = mxGetNumberOfFields(array);
  *arrays  = (mxArray **) mxCalloc((size_t) *nArrays, sizeof(mxArray *));
  *names   = (char **) mxCalloc((size_t) *nArrays, sizeof(char *));

  for (ind = 0; ind < *nArrays; ind++) {
    *arrays[ind] = mxGetFieldByNumber(array, 0, ind);
    *names[ind]  = (char *) mxGetFieldNameByNumber(array, ind);
  }

  return(Iok);

} /* matlabStruct2Arrays */

/*F:matlab2band*

________________________________________________________________

		matlab2band
________________________________________________________________

Name:		matlab2band - Convert from MATLAB array to XITE BIFF band

Syntax:         | #include <xite/matlab.h>
                |
		| int matlab2band( const mxArray *array, IBAND band );

Description:    Converts the numeric data in the MATLAB array 'array' into
                the XITE BIFF band 'band'. 'band' must have pixel type
                compatible with the type of 'array'. 'array' types of
                unsigned 32-bit, signed and unsigned 64-bit integers
                are accepted by 'band' type of integer.

Return value:   | 0 => ok
                | 11 => Incompatible data types between array and band.
		| 12 => Array is not numeric.

Restrictions:   Storage for 'band' must be allocated prior to calling this
                routine.

See also:	'matlab2biff(3)', 'saveMatlabAsBiff(3)', 'band2matlab(3)'.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.

Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int matlab2band(const mxArray *array, IBAND band) {
/*
    MATLAB arrays are column-ordered, BIFF bands are row-ordered.
*/

    long i, j, xsize, ysize;
    IPIXTYP pixTyp;

    ENTER_FUNCTION_DEBUG("matlab2band");

    if (!mxIsNumeric(array))
      return(Error(NOT_NUMERIC_IND, matlab2biff_error_s[NOT_NUMERIC_IND]));

    pixTyp = Ipixtyp(band);
    xsize  = (long) mxGetN(array);
    ysize  = (long) mxGetM(array);

#   define CONVERT(OUT_BAND_TYPE, PIXEL_TYPE, MX_ELEMENT_TYPE) \
    { \
        MX_ELEMENT_TYPE *pr    = (MX_ELEMENT_TYPE *) mxGetData(array); \
        OUT_BAND_TYPE out_band = (OUT_BAND_TYPE) band; \
        \
        for (i = 1; i <= ysize; i++) \
          for (j = 1; j <= xsize; j++) \
      	    out_band[i][j] = (PIXEL_TYPE) pr[i + (j - 1) * ysize - 1]; \
    }

#   define C_CONVERT(OUT_BAND_TYPE, MX_ELEMENT_TYPE) \
    { \
        MX_ELEMENT_TYPE *pr    = (MX_ELEMENT_TYPE *) mxGetData(array); \
        MX_ELEMENT_TYPE *pi    = (MX_ELEMENT_TYPE *) mxGetImagData(array); \
        OUT_BAND_TYPE out_band = (OUT_BAND_TYPE) band; \
        \
        for (i = 1; i <= ysize; i++) \
        for (j = 1; j <= xsize; j++) { \
  	  out_band[i][j].re = pr[i + (j - 1) * ysize - 1]; \
	  out_band[i][j].im = pi[i + (j - 1) * ysize - 1]; \
         } \
    }

    switch((int) pixTyp) {
    case Iu_byte_typ:

	if (!mxIsUint8(array))
	  return(Error(INCOMPATIBLE_IND,
		       matlab2biff_error_s[INCOMPATIBLE_IND]));

	CONVERT(IUB_BAND, UNS_BYTE, uint8_T);
	
	break;
    case Is_byte_typ:
      
      if (!mxIsInt8(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ISB_BAND, SIGN_BYTE, int8_T);
      
      break;
    case Iu_short_typ:
      
      if (!mxIsUint16(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(IUS_BAND, UNS_SHORT, uint16_T);
      
      break;
    case Is_short_typ:
      
      if (!mxIsInt16(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ISS_BAND, SIGN_SHORT, int16_T);
      
      break;
    case Integer_typ:
      
      if (mxIsInt32(array)) {
	CONVERT(II_BAND, INTEGER, int32_T);
      }
      else if (mxIsUint32(array)) {
	CONVERT(II_BAND, INTEGER, uint32_T);
      }
      else if (mxIsInt64(array)) {
	CONVERT(II_BAND, INTEGER, int64_T);
      }
      else if (mxIsUint64(array)) {
	CONVERT(II_BAND, INTEGER, uint64_T);
      }
      else
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      break;
    case Ireal_typ:
      
      if (!mxIsSingle(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(IR_BAND, REAL, real32_T);
      
      break;
    case Icomplex_typ:
      
      if (!mxIsSingle(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      if (!mxIsComplex(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      C_CONVERT(IC_BAND, real32_T);
      
      break;
    case Idouble_typ:
      
      if (!mxIsDouble(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      CONVERT(ID_BAND, DOUBLE, real64_T);
      
      break;
    case Id_complex_typ:
      
      if (!mxIsDouble(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      if (!mxIsComplex(array))
	return(Error(INCOMPATIBLE_IND,
		     matlab2biff_error_s[INCOMPATIBLE_IND]));
      
      C_CONVERT(IDC_BAND, real64_T);
      
      break;
    }

    LEAVE_FUNCTION_DEBUG("matlab2band");
    
    return(Iok);
    
} /* matlab2band */

/*F:matlab2biff*

________________________________________________________________

		matlab2biff
________________________________________________________________

Name:		matlab2biff - Convert several MATLAB arrays to a multi-band
                XITE BIFF image

Syntax:         | #include <xite/matlab.h>
                |
                | int matlab2biff( IMAGE img,
                |    long* bandNo, long nbands,
                |    const mxArray** arrays, char* defaultName,
		|    char *matNames[]);

Description:    Convert the 'nbands' 'arrays' to a BIFF image with 'nbands'
                bands or if 'arrays'[0] is an array struct, convert the struct
		fields in 'arrays'[0] to a BIFF image. If 'arrays'[0] is not a
		struct array, 'bandNo' determines what band each array will be
		stored in, i.e. 'arrays'[i] (i=0,...,nbands-1) will be stored
		in band number 'bandNo'[i]. Remember that 'bandNo'[i] must be
		greater than zero because bands are numbered from one.
                If bandNo is NULL, store 'arrays[i]' in band number (i+1).

		Memory for the bands will be allocated by the routine (but not
		for the image info and band pointers - use e.g. Init_image
		prior to calling this routine).

                The pixel type of the BIFF bands will be compatible with the
                type of each corresponding array (which may be a struct field).
		'array' types of unsigned 32-bit, signed and unsigned 64-bit
		integers will be stored in 'band' type of integer.

		The BIFF image title will be the same as the first non-empty
		MATLAB array name in 'matNames' (or the first field name if
		'arrays'[0] is a struct), or else the 'defaultName' (if
		non-empty), or else the empty string.

Return value:   | 0  => OK
                | 7  => Failure in creating BIFF band.
		| 8  => Failure in converting MATLAB array to BIFF band.
                | 11 => Incompatible data types between array and image.
		| 12 => Array (which may be struct field) is not numeric.

See also:       'biff2matlab(3)', 'saveBiffAsMatlab(3)'.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.

Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int matlab2biff(IMAGE img, long *bandNo, long nbands, const mxArray **arrays, char *defaultName, char *matNames[]) {
    long ind;
    IPIXTYP pixTyp;
    char imageName[biffMAXLEN + 1];
    IBAND band;
    int free_bandNo = FALSE;
    int status;
    mxArray **unpacked = NULL;
    char **unpacked_names = NULL;

    ENTER_FUNCTION_DEBUG("matlab2biff");

    if (mxIsStruct(arrays[0])) {
      (void) matlabStruct2Arrays(arrays[0], &unpacked, &unpacked_names,
				 &nbands);
      bandNo = NULL;
    }

    if (bandNo == NULL) {
      bandNo      = (long *) mxCalloc((size_t) nbands, sizeof(long));
      free_bandNo = TRUE;

      for (ind = 0; ind < nbands; ind++)
	bandNo[ind] = ind + 1;
    }

    for (ind = 0; ind < nbands; ind++) {
      const mxArray *array;

      if (unpacked != NULL) array = unpacked[ind];
      else array = arrays[ind];

      pixTyp = pixtypFromMxArray(array);
      if (pixTyp == Iunknown_typ) {
	if (!mxIsNumeric(array))
	  return(Error(NOT_NUMERIC_IND,
		       matlab2biff_error_s[NOT_NUMERIC_IND]));
	else
	  return(Error(INCOMPATIBLE_IND,
		       matlab2biff_error_s[INCOMPATIBLE_IND]));
      }

      band = Imake_band(pixTyp, mxGetN(array), mxGetM(array));

      if (band == NULL)
	return(Error(CREATE_IND, matlab2biff_error_s[CREATE_IND]));
      
      if ((status = matlab2band(array, band)) != Iok)
	return(Error(status, matlab2biff_error_s[status]));

      img[bandNo[ind]] = band;
    }

    if (unpacked_names != NULL)
      (void) biffName(imageName, unpacked_names, defaultName, nbands);
    else
      (void) biffName(imageName, matNames, defaultName, nbands);

    if (strlen(imageName) > 0) Iset_title(img, imageName);

    if (free_bandNo) mxFree(bandNo);

    LEAVE_FUNCTION_DEBUG("matlab2biff");

    return(Iok);

} /* matlab2biff */

/*F:saveMatlabAsBiff*

________________________________________________________________

		saveMatlabAsBiff
________________________________________________________________

Name:		saveMatlabAsBiff - Save MATLAB arrays in XITE BIFF image file

Syntax:         | #include <xite/matlab.h>
                |
                | int saveMatlabAsBiff( const mxArray** arrays,
		|    char *matNames[],
                |    long nMat, char* biffFileName,
                |    char* historyText );

Description:    Convert the 'nMat' MATLAB arrays 'arrays' to XITE BIFF
                image and save to file. The pixeltype of each BIFF band will
		be compatible with the data type of the corresponding array.
		'array' types of unsigned 32-bit, signed and unsigned 64-bit
                integers will be stored in 'band' type of integer.

		The BIFF image title will be the same as the name of the first
		MATLAB array with a non-empty name string (names given
		by 'matNames'), or else the BIFF filename (if non-empty),
		or else the empty string.

		The historyText argument is used as the second argument in a
		call to 'Ihistory(5)'.

Return value:   | 0  => OK
                | 6  => Failure in initializing BIFF image.
		| 7  => Failure in creating BIFF image band.
		| 8  => Failure in converting MATLAB array to XITE
                |       BIFF band.
                | 11 => Incompatible data types between array and image.
                | 12 => Array is not numeric.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.

Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

int saveMatlabAsBiff(const mxArray **arrays, char *matNames[], long nMat, char *biffFileName, char *historyText) {
  IMAGE img;
  int status;

  ENTER_FUNCTION_DEBUG("saveMatlabAsBiff");

  if ((img = Init_image(nMat, "")) == NULL)
    return(Error(INIT_BIFF_IND, matlab2biff_error_s[INIT_BIFF_IND]));

  status = matlab2biff(img, NULL, nMat, arrays, biffFileName, matNames);

  if (status != Iok) {
    return(Error(status, matlab2biff_error_s[status]));
  }

  Ihistory(img, historyText, "");
  Iwrite_image(img, biffFileName);

  LEAVE_FUNCTION_DEBUG("saveMatlabAsBiff");

  return(Iok);

} /* saveMatlabAsBiff */

/*F:readMatrices*

________________________________________________________________

		readMatrices
________________________________________________________________

Name:		readMatrices - Read MATLAB arrays from file

Syntax:         | #include <xite/matlab.h>
                |
                | int readMatrices( mxArray*** ptrArrays,
                |    long* nMat, char*** pMatNames, long nNames,
                |    char* fileName );

Description:    Read from file the 'nNames' numeric or struct arrays with names
                given by 'pMatNames' (or all if 'nNames' == 0). Return
		the arrays in 'ptrArrays' and the number of successfully read
		arrays in 'nMat'. 'pMatNames' will upon return contain only the
		names of the read arrays, corresponding to 'ptrArrays'. These
		names are copies of the names in 'pMatNames' prior to the call,
		they don't use the same memory.

		Will read both MATLAB version 4, version 5 and version 6.5
		MAT-files.

Restrictions:	Will not read sparse arrays. Memory will be allocated by the
                mx allocation routines for the pointer array 'ptrArrays'
		as well as for the arrays and 'pMatNames'. The caller is
		responsible for deallocating the memory which was used by
		'pMatNames' prior to the call.

Return value:   0 => OK
                3 => Failure in opening the MAT file.
		4 => Failure in reading the MAT file directory.
		5 => Did not find all requested arrays in file.

Author:		Svein Bøe, Ifi, UiO, 1994-2003.
________________________________________________________________

*/

int readMatrices(mxArray ***ptrArrays, long *nMat, char ***pMatNames, long nNames, char *fileName) {
  MATFile *matFile;
  mxArray **arrays;
  long i, nameInd;
  char **matNames;

  ENTER_FUNCTION_DEBUG("readMatrices");

  if ((matFile = matOpen(fileName, "r")) == NULL)
    return(Error(OPEN_MAT_IND, matlab2biff_error_s[OPEN_MAT_IND]));

  if (nNames == 0) {
    /* Read all numeric and struct arrays */

    if ((*pMatNames = matGetDir(matFile, (int *) &nNames)) == NULL)
      return(Error(READ_MAT_IND, matlab2biff_error_s[READ_MAT_IND]));
  }
  matNames = (char **) mxMalloc(nNames * sizeof(char *));

  arrays = *ptrArrays =
    (mxArray **) mxCalloc((size_t) nNames, sizeof(mxArray *));

  *nMat   = 0;
  nameInd = 0;
  while (nameInd < nNames &&
	 (arrays[*nMat] = matGetVariable(matFile, (*pMatNames)[nameInd]))
	     != NULL) {
    if ((!mxIsNumeric(arrays[*nMat]) && !mxIsStruct(arrays[*nMat]))
	|| mxIsSparse(arrays[*nMat]))
      mxDestroyArray(arrays[*nMat]);
    else {
      matNames[*nMat] = (char *) mxMalloc(strlen((*pMatNames)[nameInd]) + 1);
      (void) strcpy(matNames[*nMat], (*pMatNames)[nameInd]);
      (*nMat)++;
    }
    nameInd++;
  }

  /* (*nMat) gives the number of arrays read successfully */

  matClose(matFile);

  /* Repack the returned names so as not to allocate too much space (for
     unused pointers). */

  *pMatNames = (char **) mxMalloc((*nMat) * sizeof(char *));
  for (i = 0; i < *nMat; i++) {
    (*pMatNames)[i] = (char *) mxMalloc(strlen(matNames[i]) + 1);
    (void) strcpy((*pMatNames)[i], matNames[i]);
    mxFree(matNames[i]);
  }
  mxFree(matNames);

  /* Some of the array pointers in 'arrays' may not be freed.
     Repack the returned array pointers so as not to leak unused
     pointer space. */

  *ptrArrays = (mxArray **) mxCalloc((size_t) (*nMat), sizeof(mxArray *));
  for (i = 0; i < *nMat; i++) {
    (*ptrArrays)[i] = arrays[i];
  }
  mxFree(arrays);

  if (nNames > 0 && *nMat < nNames)
    return(Error(NOT_FOUND_IND, matlab2biff_error_s[NOT_FOUND_IND]));
  else return(Iok);

  LEAVE_FUNCTION_DEBUG("readMatrices");

} /* readMatrices */

#endif /* not MAIN */

/*P:matlab2biff*

________________________________________________________________

		matlab2biff
________________________________________________________________

Name:		matlab2biff - Convert from MATLAB MAT-file format to
                XITE BIFF image file format.

Syntax:		matlab2biff <MAT-file> <BIFF-file> [-s <arrayName>] ...

Description:    Convert from MATLAB MAT-file format to XITE BIFF image file
                format. Only numeric arrays are converted. If more than
		one array, they are all converted into the same BIFF file.
		Default is to convert all numeric arrays. The XITE BIFF image
		bands will get pixel type compatible with the array data type.
		Array types of unsigned 32-bit, signed and unsigned 64-bit
                integers will be stored in band type of integer.

		"-", "" or "-1" can be used for 'stdout'. The output filename
		may be followed by band specifications, as in "file:3" for the
		third band (see 'Iwrite_image(3)').

		The BIFF image title will be the same as the name of the first
		MATLAB array with a non-empty name string, or else the BIFF
		filename (if not "-" or "" or "-1" for 'stdout'), or the
		empty string.

		Will read MATLAB version 4, version 5 and version 6.5
		MAT files.

Restrictions:   Will not read sparse arrays.

See also:	'biff2matlab(1)', 'Iwrite_image(5)'

Return value:   | 0 => OK
                | 1 => Usage message
                | 2 => Illegal command line
                | 3 => Failure in opening the MAT-file.
		| 4 => Failure in reading the MAT-file directory.
		| 5 => Did not find all requested arrays in file.
                | 6 => Failure in initializing BIFF image.
		| 7 => Failure in creating BIFF image band.
		| 8 => Failure in converting MATLAB array to
		|      XITE BIFF band.
		| 9 => Failure in opening BIFF image file.
		| 10 => Failure in writing BIFF image band to file.
		| 11 => Incompatible data types between array and band.
		| 12 => Array is not numeric.

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id: 		$Id: matlab2biff.c,v 1.36 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

#ifdef MAIN

int main(int argc, char **argv) {
    mxArray **arrays;
    char **matNames, *name;
    long nMat, nNames;
    int status;

    /* Initialize XITE program */

    Iset_message(1);
    Iset_abort(1);
    InitMessage(&argc, argv, xite_app_std_usage_text(
      "Usage: %s <matFile> <biffFile> [-s <arrayName>] ... \n\
        '-' can be used for stdout.\n"));

    /* stdout if name is "-", "" or "-1" (XITE standard) */

    if (argc == 1) Usage(USAGE_IND, NULL);
    if ((argc % 2) != 1) Usage(USAGE_IND, "Illegal number of arguments.\n");

    matNames = (char **) mxMalloc(sizeof(char *) * 100);
    nNames = 0;
    while (argc > 3) {
      if (strlen(name = read_switch(&argc, argv, "-s", 1, "")) > 0)
	matNames[nNames++] = strcpy((char *) mxMalloc(strlen(name)), name);
      else break;
    }

    if ((argc % 2) != 1) Usage(USAGE_IND, "Illegal number of arguments.\n");

    /* nNames == 0: Read all numeric arrays (default) */
    if (( status = readMatrices(&arrays, &nMat, &matNames, nNames, argv[1]))
	!= Iok)
      Error(status, matlab2biff_error_s[status]);

    if ((status = saveMatlabAsBiff((const mxArray **) arrays, matNames, nMat,
				   argv[2], argv[0]))
	!= Iok)
      Error(status, matlab2biff_error_s[status]);

    return(0);
}

#endif /* MAIN */
