
/*C*

________________________________________________________________

        matlab2region
        $Id: matlab2region.c,v 1.8 2005/08/12 08:36:04 svein Exp $
        Copyright 1990-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: matlab2region.c,v 1.8 2005/08/12 08:36:04 svein Exp $, Blab, UiO";

#include <mat.h>

#include <xite/includes.h>
#include <xite/matlab.h>
#include "matlab_region.h"
#include <xite/message.h>
#include <xite/readarg.h>
#include XITE_MALLOC_H
#include XITE_STRING_H
#include <xite/debug.h>

#ifndef MAIN

#define OK_IND              0
#define NOT_STRUCT_IND      1
#define NOT_NFIELDS_IND     2
#define NOT_FIELD_NAME_IND  3
#define NOT_FIELD_CLASS_IND 4

static char *error_s[] = {
  "OK\n",
  "MATLAB array is not struct.\n",
  "Not the correct number of fields in struct.\n",
  "Not the correct field name in struct.\n",
  "Not the correct class for field in struct.\n"
};

static void region_behaviour_init(region_behaviour *beh, int header,
				  int nc, int sort, int split, int cursorx,
				  int cursory, int bg) {
  beh->header  = header;
  beh->nc      = nc;
  beh->sort    = sort;
  beh->split   = split;
  beh->cursorx = cursorx;
  beh->cursory = cursory;
  beh->bg      = bg;
  beh->ifline  = NULL;
  beh->outfile = NULL;

  return;

} /* region_behaviour_init */

static void region_properties_init(region_properties *prop, int state) {
  prop->objectnr   = state;
  prop->regionnr   = state;
  prop->area       = state;
  prop->perimeter  = state;
  prop->xmin       = state;
  prop->ymin       = state;
  prop->xmax       = state;
  prop->ymax       = state;
  prop->height     = state;
  prop->width      = state;
  prop->xpoint     = state;
  prop->ypoint     = state;
  prop->minpix     = state;
  prop->maxpix     = state;
  prop->xmean      = (double) state;
  prop->ymean      = (double) state;
  prop->moment     = (momentData *) mxCalloc(1, sizeof(momentData));
  prop->moment->p  = state;
  prop->moment->q  = state;
  prop->cmoment    = (momentData *) mxCalloc(1, sizeof(momentData));
  prop->cmoment->p = state;
  prop->cmoment->q = state;
  prop->imoment    = (double *) mxCalloc(8, sizeof(double));
  prop->volume     = state;
  prop->mean       = (double) state;
  prop->median     = state;
  prop->stdev      = (double) state;

  return;

} /* region_properties_init */

int matlab2region_behaviour (const mxArray *array, region_behaviour **beh) {
  int field_num;
  region_behaviour *r_beh;

  if (mxIsEmpty(array)) {
    r_beh = (region_behaviour *) mxCalloc(1, sizeof(region_behaviour));
    region_behaviour_init(r_beh, 1, 4, 1, 1, 0, 0, 0);
    *beh = r_beh;

    return(Iok);
  }

  if (!mxIsStruct(array)) {
    *beh = NULL;
    return(Error(NOT_STRUCT_IND, error_s[NOT_STRUCT_IND]));
  }

  if (mxGetNumberOfFields(array) != num_behaviour_fields_s) {
    return(Error(NOT_NFIELDS_IND, error_s[NOT_NFIELDS_IND]));
  }

  r_beh = (region_behaviour *) mxCalloc(1, sizeof(region_behaviour));

  for (field_num = 0; field_num < num_behaviour_fields_s; field_num++) {
    const char *name;
    mxArray *field;

    name = mxGetFieldNameByNumber(array, field_num);
    if (strcmp(name, behaviour_field_names_s[field_num])) {
      return(Error(NOT_FIELD_NAME_IND,
		   error_s[NOT_FIELD_NAME_IND]));
    }
    field = mxGetFieldByNumber(array, 0, field_num);

    if (mxGetClassID(field) != behaviour_field_classes_s[field_num]) {
      return(Error(NOT_FIELD_CLASS_IND,
		   error_s[NOT_FIELD_CLASS_IND]));
    }

    if        (!strcmp(name, BEHAVIOUR_FIELD_HEADER)) {
      r_beh->header = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_NC)) {
      r_beh->nc = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_SORT)) {
      r_beh->sort = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_SPLIT)) {
      r_beh->split = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_CURSORX)) {
      r_beh->cursorx = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_CURSORY)) {
      r_beh->cursory = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_BG)) {
      r_beh->bg = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, BEHAVIOUR_FIELD_IFLINE)) {
      r_beh->ifline = mxArrayToString(field);
    } else if (!strcmp(name, BEHAVIOUR_FIELD_OUTFILE)) {
      r_beh->outfile = mxArrayToString(field);
    }
  } /* for */

  *beh = r_beh;

  return(Iok);

} /* matlab2region_behaviour */

int matlab2region_properties (const mxArray *array, region_properties **prop) {
  int field_num;
  region_properties *r_prop;

  if (mxIsEmpty(array)) {
    r_prop = (region_properties *) mxCalloc(1, sizeof(region_properties));
    region_properties_init(r_prop, 1);
    *prop = r_prop;

    return(Iok);
  }

  if (!mxIsStruct(array)) {
    *prop = NULL;
    return(Error(NOT_STRUCT_IND, error_s[NOT_STRUCT_IND]));
  }

  if (mxGetNumberOfFields(array) != num_properties_fields_s) {
    return(Error(NOT_NFIELDS_IND, error_s[NOT_NFIELDS_IND]));
  }

  r_prop = (region_properties *) mxCalloc(1, sizeof(region_properties));

  for (field_num = 0; field_num < num_properties_fields_s; field_num++) {
    const char *name;
    mxArray *field;

    name = mxGetFieldNameByNumber(array, field_num);
    if (strcmp(name, properties_field_names_s[field_num])) {
      return(Error(NOT_FIELD_NAME_IND,
		   error_s[NOT_FIELD_NAME_IND]));
    }
    field = mxGetFieldByNumber(array, 0, field_num);

    if (mxGetClassID(field) != properties_field_classes_s[field_num]) {
      return(Error(NOT_FIELD_CLASS_IND,
		   error_s[NOT_FIELD_CLASS_IND]));
    }

    if        (!strcmp(name, PROPERTIES_FIELD_OBJECTNR)) {
      r_prop->objectnr = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_REGIONNR)) {
      r_prop->regionnr = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_AREA)) {
      r_prop->area = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_PERIMETER)) {
      r_prop->perimeter = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_XMIN)) {
      r_prop->xmin = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_YMIN)) {
      r_prop->ymin = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_XMAX)) {
      r_prop->xmax = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_YMAX)) {
      r_prop->ymax = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_HEIGHT)) {
      r_prop->height = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_WIDTH)) {
      r_prop->width = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_XPOINT)) {
      r_prop->xpoint = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_YPOINT)) {
      r_prop->ypoint = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MINPIX)) {
      r_prop->minpix = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MAXPIX)) {
      r_prop->maxpix = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_XMEAN)) {
      r_prop->xmean = ((double *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_YMEAN)) {
      r_prop->ymean = ((double *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MOMENT)) {
      if (r_prop->moment == NULL) {
	r_prop->moment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
    } else if (!strcmp(name, PROPERTIES_FIELD_MOMENTP)) {
      if (r_prop->moment == NULL) {
	r_prop->moment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
      r_prop->moment->p = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MOMENTQ)) {
      if (r_prop->moment == NULL) {
	r_prop->moment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
      r_prop->moment->q = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_CMOMENTP)) {
      if (r_prop->cmoment == NULL) {
	r_prop->cmoment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
    } else if (!strcmp(name, PROPERTIES_FIELD_CMOMENTP)) {
      if (r_prop->cmoment == NULL) {
	r_prop->cmoment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
      r_prop->cmoment->p = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_CMOMENTQ)) {
      if (r_prop->cmoment == NULL) {
	r_prop->cmoment = (momentData *) mxCalloc(1, sizeof(momentData));
      }
      r_prop->cmoment->q = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_IMOMENT)) {
      r_prop->imoment = (double *) mxCalloc(8, sizeof(double));
      memcpy(r_prop->imoment, (double *) mxGetData(field), 8 * sizeof(double));
    } else if (!strcmp(name, PROPERTIES_FIELD_VOLUME)) {
      r_prop->volume = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MEAN)) {
      r_prop->mean = ((double *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_MEDIAN)) {
      r_prop->median = ((int *) mxGetData(field))[0];
    } else if (!strcmp(name, PROPERTIES_FIELD_STDEV)) {
      r_prop->stdev = ((double *) mxGetData(field))[0];
    }
  } /* for */

  *prop = r_prop;

  return(Iok);

} /* matlab2region_properties */

#endif /* not MAIN */
