
/*

________________________________________________________________

        saveBiff
        $Id: saveBiff.c,v 1.19 2014/01/31 12:52:07 svein Exp $
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: saveBiff.c,v 1.19 2014/01/31 12:52:07 svein Exp $, Blab, UiO";

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/message.h>
#include <xite/debug.h>
#include <mex.h>
#include <xite/mex.h>

static void setBandNo(long **bandNo, long start, long nMat) {
  long ind;

  *bandNo = (long *) mxCalloc((size_t) nMat, sizeof(long));
  for (ind = 1; ind <= nMat; ind++)
    (*bandNo)[ind - 1] = ind + start - 1;
}

/*L:mexFunction*

________________________________________________________________

		mexFunction
________________________________________________________________

Name:		mexFunction
Syntax:		| #include <xite/matlab.h>
                |
		| void mexFunction( int nlhs, mxArray *plhs[],
		|    int nrhs, const mxArray *prhs[] );
		|
Description:    | Save MATLAB arrays in XITE BIFF image file.
                | prhs (input arguments) are
		|   - Arrays (numeric, non-sparse)
                |   - BIFF filename (string)

Author:		Svein Bøe, BLAB, Ifi, UiO, 1994-2003.
Id:             $Id: saveBiff.c,v 1.19 2014/01/31 12:52:07 svein Exp $
________________________________________________________________

*/

/*M*

%saveBiff  Save arrays in XITE BIFF image file.
%
% SYNOPSIS:
%   saveBiff(m1, m2, ..., file);
% OUTPUT:
%
% INPUT:
%   m1, m2, ... : Arrays (numeric, non-sparse).
%   file        : String with name of XITE BIFF image file.
% DESCRIPTION:
%   Save arrays in XITE BIFF image file.
%   

% AUTHOR:
%   Svein Bøe, Ifi Uio, 1994, 1997, 2003.
%

*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  IMAGE img;
  long nMat, len, *bandNo = NULL;
  int argc = 1, level;
  char *biffFileName;
  char *argv[] = {"saveBiff"};

  ENTER_FUNCTION_DEBUG("mexFunction for saveBiff");

  Iset_message(1);
  InitMessage(&argc, argv, NULL);
  level = PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab,
		      ExitOnError());
  FPRINTF2("level=%d\n", level);

  if (nrhs < 1) errorMsgToMatlab(2, "Missing file name argument.");
  if (nlhs > 0) errorMsgToMatlab(2, "Too many output arguments.");

  nMat = 0;
  while (nMat < nrhs && !mxIsChar(prhs[nMat])) nMat++;

  if (nMat == 0)    errorMsgToMatlab(2, "Missing matrix arguments.");
  if (nMat == nrhs) errorMsgToMatlab(2, "Missing filename argument.");
  if (!mxIsChar(prhs[nMat])) errorMsgToMatlab(2, "Missing filename argument.");

  /* Extract filename argument */
  len = (long) mxGetN(prhs[nMat]);
  biffFileName = (char *) mxCalloc((size_t) (len + 1), sizeof(char));
  mxGetString(prhs[nMat], biffFileName, (int) (len + 1));

  /* Set default */
  setBandNo(&bandNo, 1, nMat);

  if ((img = Init_image(nMat, "")) == NULL)
    errorMsgToMatlab(2, "Failure in initializing BIFF image.");

  if (matlab2biff(img, bandNo, nMat, prhs, "", NULL) != Iok)
    errorMsgToMatlab(2, "Failure in converting matlab matrices to BIFF image.");

  if (bandNo != NULL) mxFree(bandNo);

  Ihistory(img, "saveBiff", "Saved from MATLAB");

  if (Iwrite_image(img, biffFileName) != Iok)
    errorMsgToMatlab(2, "Could not write BIFF image to file.");

  mxFree(biffFileName);
  level = PopMessage();
  FPRINTF2("level=%d\n", level);

  LEAVE_FUNCTION_DEBUG("mexFunction for saveBiff");
}
