
/*C*

________________________________________________________________

        xite_absDiff.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int absDiff(IBAND input1, IBAND input2, IBAND output) */

/*M*
function [output] = xite_absDiff(input1, input2)
%xite_absDiff Absolute difference between two bands.
%   Calls the XITE C function absDiff.
%   function [output] = xite_absDiff(input1, input2)
%
%   The C function has the following prototype:
%   int absDiff(IBAND input1, IBAND input2, IBAND output)
%
%   and the following manual page:
%
%   NAME
%          absDiff - absolute difference between two bands
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int absDiff( IBAND input1, IBAND input2,
%              IBAND output );
%
%   DESCRIPTION
%          absDiff calculates, pixel by pixel, the absolute value of the grey
%          level difference between the two input images.
%
%          If the result is outside the pixeltype range, the result is limited.
%
%   RESTRICTIONS
%          Accepts all pixeltypes, as long as all three bands have the same pixel-
%          type.
%
%   RETURN VALUE
%           0 => ok
%           1 => bands have different pixel types
%           2 => unknown pixel type
%
%   SEE ALSO
%          absDiff(1), signDiff(3), addw(3), multiply(3), divide(3)
%
%   AUTHOR
%          Tor L|nnestad, BLAB, ifi, UiO
%
%   ID
%          $Id: absDiff.c,v 1.35 2009/02/17 11:59:48 svein Exp $
%
%                                     27 Feb 2014 ABSDIFF(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"absDiff"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input1;
    IBAND input2;
    IBAND output;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input1_img;
    IMAGE input2_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input1_img = Init_image(1, "");
    
    convert_status = matlab2biff(input1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input1 = (IBAND) (input1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input1));
    input2_img = Init_image(1, "");
    
    convert_status = matlab2biff(input2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    input2 = (IBAND) (input2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input2));

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = absDiff(input1, input2, output);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C absDiff.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
