
/*C*

________________________________________________________________

        xite_average.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int average(IMAGE inimg, ID_BAND outband, double scale) */

/*M*
function [outband] = xite_average(inimg, scale)
%xite_average Computes the average of several bands in an input image.
%   Calls the XITE C function average.
%   function [outband] = xite_average(inimg, scale)
%
%   The C function has the following prototype:
%   int average(IMAGE inimg, ID_BAND outband, double scale)
%
%   and the following manual page:
%
%   NAME
%          average - computes the average of several bands in an input image
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int average( IMAGE inimg, ID_BAND outband,
%              int Scale );
%
%   DESCRIPTION
%          Computes the average image of two or several bands of an image. Grey
%          levels of the output average image will be multiplied by a Scale
%          (default Scale is 1.0).
%
%           (inimg[1] + inimg[2] +........+ inimg[n]) / n * Scale
%
%          The output image will be of pixeltype double.
%
%   RESTRICTIONS
%          Input image should not have pixels of type "complex".
%
%   RETURN VALUE
%           0 => OK
%           1 => Unknown pixeltype of input band
%
%   AUTHOR
%          Yogesan, BLAB, Ifi, UiO
%
%   ID
%          $Id: average.c,v 1.35 2009/02/17 12:02:39 svein Exp $
%
%                                     27 Feb 2014 AVERAGE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"average"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IMAGE inimg;
    ID_BAND outband;
    double scale;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    if (mxIsStruct(prhs[0])) {
      inimg = Init_image(mxGetNumberOfFields(prhs[0]), "");
    } else {
      inimg = Init_image(1, "");
    }
    convert_status = matlab2biff(inimg, NULL, 1,
                                 &prhs[0], NULL, NULL);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inimg[1]));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inimg[1]));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inimg[1]));
    scale = ((double *) mxGetData(prhs[1]))[0];

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (ID_BAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = average(inimg, outband, scale);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C average.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
