
/*C*

________________________________________________________________

        xite_colMaxArrBand.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int colMaxArrBand(IBAND band, double **vector, long *size, double *min, double *max) */

/*M*
function [vector, size, min, max] = xite_colMaxArrBand(band)
%xite_colMaxArrBand Horizontal and vertical cross-sections, row or column of
%   BIFF band.
%   Calls the XITE C function colMaxArrBand.
%   function [vector, size, min, max] = xite_colMaxArrBand(band)
%
%   The C function has the following prototype:
%   int colMaxArrBand(IBAND band, double **vector, long *size, double *min, double *max)
%
%   and the following manual page:
%
%   NAME
%          crossSection, crossSectionBand, colMaxArrBand, rowMaxArrBand, colMinAr-
%          rBand, rowMinArrBand, rowBand, columnBand, colMaxArr, rowMaxArr, colMi-
%          nArr, rowMinArr, row, column - Horizontal and vertical cross-sections,
%          row or column of BIFF band
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int colMaxArrBand( IBAND band,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int rowMaxArrBand( IBAND band,,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int colMinArrBand( IBAND band,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int rowMinArrBand( IBAND band,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int columnBand( IBAND band, int col,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int rowBand( IBAND band, int rowno,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int colMaxArr( IMAGE img, int bandno,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int rowMaxArr( IMAGE img, int bandno,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int colMinArr( IMAGE img, int bandno,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int rowMinArr( IMAGE img, int bandno,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int column( IMAGE img, int bandno, int col,
%              double** vector, long* size, double* min,
%              double* max );
%
%           int row( IMAGE img, int bandno, int rowno,
%              double** vector, long* size, double* min,
%              double* max );
%
%   DESCRIPTION
%          colMaxArrBand finds the horizontal cross-section of band, based on col-
%          umn maxima.  The cross-section vector/array is returned in a vector
%          which has size elements and peak values min and max.
%
%          columnBand finds column number col from band.  The remaining parameters
%          have the same meaning as for colMaxArrBand.
%
%          The image versions of these functions behave like the band versions,
%          except that they operate on band number bandno in image img.
%
%          The image versions of these functions are macros calling the band ver-
%          sions.  The image versions are retained for compatibility with previous
%          XITE versions.
%
%          All pixel types are accepted. If a band is of a complex type, absolute
%          values are returned.
%
%   RETURN VALUE
%           0 => OK
%           1 => Band reference is NULL
%           2 => Illegal row or column number (for ’column’ and
%                ’row’)
%           3 => Illegal pixeltype (for ’column’ and ’row’)
%
%   SEE ALSO
%          vectorPeak(3), crossSection(1), profile(3)
%
%   AUTHOR
%          Svein Bøe, BLAB, Ifi, UiO
%
%                                     27 Feb 2014 CROSSSECTION(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"colMaxArrBand"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    double *vector;
    long size;
    double min;
    double max;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 4) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));

    /* Prepare C result variable. */
                    
    /* Execute C routine. */
    return_value = colMaxArrBand(band, &vector, &size, &min, &max);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C colMaxArrBand.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(size, 1, mxDOUBLE_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), vector, (size) * sizeof(double));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }
    if (nlhs > 1) {
        tmp_plhs[1] =
            mxCreateNumericMatrix(1, 1, mxINT64_CLASS, mxREAL);
        ((long *) mxGetData(tmp_plhs[1]))[0] = size;
        plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =
            mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
        ((double *) mxGetData(tmp_plhs[2]))[0] = min;
        plhs[2] = tmp_plhs[2];
    }
    if (nlhs > 3) {
        tmp_plhs[3] =
            mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
        ((double *) mxGetData(tmp_plhs[3]))[0] = max;
        plhs[3] = tmp_plhs[3];
    }

    PopMessage();

    return;
}
