
/*C*

________________________________________________________________

        xite_curveToBand.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int curveToBand(IBAND band, double *curve, double minval, double maxval, double scale, double cur, double fg, double bg) */

/*M*
function [band] = xite_curveToBand(curve, minval, maxval, scale, cur, fg, bg)
%xite_curveToBand Make BIFF image band visualize 1-D curve.
%   Calls the XITE C function curveToBand.
%   function [band] = xite_curveToBand(curve, minval, maxval, scale, cur, fg, bg)
%
%   The C function has the following prototype:
%   int curveToBand(IBAND band, double *curve, double minval, double maxval, double scale, double cur, double fg, double bg)
%
%   and the following manual page:
%
%   NAME
%          curveToBand - Make BIFF image band visualize 1-D curve
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int curveToBand(IBAND band, double *curve,
%              double minval, double maxval, double scale,
%              double cur, double fg, double bg);
%
%   DESCRIPTION
%          Let band visualize the curve in curve, with pixel-values below the
%          curve given by fg, pixel-values on the curve given by cur and pixel-
%          values above the curve given by bg.
%
%          band can be any non-complex pixeltype.  minval and maxval hold the
%          extreme values of curve or values where the curve should be clipped
%          (values outside this range will be displayed as minval or maxval).
%          scale determines the visual peak of curve in band, relative to the
%          height of band.
%
%   SEE ALSO
%          vectorPeak(3), crossSection(3), crossSection(1)
%
%   RESTRICTIONS
%          band can not be complex or double complex.
%
%   RETURN VALUE
%           0 : ok
%           1 : Band reference is NULL
%           2 : Illegal pixeltype
%
%   AUTHOR
%          Svein Bøe, BLAB, Ifi, UiO
%
%   ID
%          $Id: crossSection.c,v 1.48 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 CURVETOBAND(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"curveToBand"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    double *curve;
    double minval;
    double maxval;
    double scale;
    double cur;
    double fg;
    double bg;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    curve = ((double **) mxGetData(prhs[0]))[0];
    minval = ((double *) mxGetData(prhs[1]))[0];
    maxval = ((double *) mxGetData(prhs[2]))[0];
    scale = ((double *) mxGetData(prhs[3]))[0];
    cur = ((double *) mxGetData(prhs[4]))[0];
    fg = ((double *) mxGetData(prhs[5]))[0];
    bg = ((double *) mxGetData(prhs[6]))[0];

    /* Prepare C result variable. */
    band_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    band = (IBAND) (band_img[1]);
    
    /* Execute C routine. */
    return_value = curveToBand(band, curve, minval, maxval, scale, cur, fg, bg);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C curveToBand.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
