
/*C*

________________________________________________________________

        xite_logarithm.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus logarithm(IBAND in_band, IBAND out_band, double eps, double offset, double scale, double minimum) */

/*M*
function [out_band] = xite_logarithm(in_band, eps, offset, scale, minimum)
%xite_logarithm Take logarithm of BIFF band (compress dynamic range).
%   Calls the XITE C function logarithm.
%   function [out_band] = xite_logarithm(in_band, eps, offset, scale, minimum)
%
%   The C function has the following prototype:
%   BiffStatus logarithm(IBAND in_band, IBAND out_band, double eps, double offset, double scale, double minimum)
%
%   and the following manual page:
%
%   NAME
%          logarithm, fftDisplay - Take logarithm of BIFF band (compress dynamic
%          range)
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int logarithm( IBAND in_band, IBAND out_band,
%              double eps, double offset, double scale,
%              double minimum );
%
%           #include <xite/fft.h>
%
%           int fftDisplay( IBAND in_band, IBAND out_band,
%              double scale );
%
%   DESCRIPTION
%          Takes the logarithm of the magnitude of the pixelvalues of a real or
%          complex band according to the algorithm
%
%           out_pix = (log10(eps + |in_pix|) + offset) * scale
%           out_pix = (out_pix >= minimum) ? out_pix : minimum
%
%          This mapping will compress the dynamic range of an image and make small
%          pixelvalues better visible (e.g. used to display Fourier transform mag-
%          nitude).
%
%          eps equal to DOUBLE_MIN gives the default value
%           eps = 1 / (10 * xsize * ysize)
%
%          offset equal to DOUBLE_MAX gives the default value
%           offset = -log10(eps)
%
%          scale equal to 0.0 gives the default value 20.0.
%
%          minimum equal to DOUBLE_MAX gives the default value -10.0 * scale.
%
%          For an integral output pixeltype, the resulting pixel-values are
%          rounded. The pixelvalues are limited to fit within the range of the
%          output pixeltype.
%
%          fftDisplay is retained for compatibility with previous XITE versions.
%          It is a macro call to logarithm with eps equal to DOUBLE_MIN, offset
%          equal to DOUBLE_MAX and minimum equal to -DOUBLE_MAX. This will give
%          the default values for eps and offset mentioned above. The call to log-
%          arithm is followed by a call to shift_band.
%
%   SEE ALSO
%          logarithm(1), power(3), phase(3), real(3), imag(3), absValue(3)
%
%   RESTRICTIONS
%          Input band may be any pixeltype, output may be double, real or unsigned
%          byte.
%
%   RETURN VALUE
%           1 - bands have different xsize
%           2 - bands have different ysize
%           3 - out_band has wrong pixeltype
%           4 - in_band has unknown pixeltype
%
%   AUTHOR
%          Trond Sørensen (wrote fftDisplay).
%
%   MODIFIED
%          Svein Bøe (added output pixeltype real and double, fixed bug for input
%          pixeltype real, added lots of more input pixeltypes) when converting
%          from fftDisplay to logarithm.
%
%   ID
%          $Id: logarithm.c,v 1.49 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 LOGARITHM(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"logarithm"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND in_band;
    IBAND out_band;
    double eps;
    double offset;
    double scale;
    double minimum;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE in_band_img;
    IMAGE out_band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 5) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 5) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    in_band_img = Init_image(1, "");
    
    convert_status = matlab2biff(in_band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    in_band = (IBAND) (in_band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) in_band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) in_band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) in_band));
    eps = ((double *) mxGetData(prhs[1]))[0];
    offset = ((double *) mxGetData(prhs[2]))[0];
    scale = ((double *) mxGetData(prhs[3]))[0];
    minimum = ((double *) mxGetData(prhs[4]))[0];

    /* Prepare C result variable. */
    out_band_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    out_band = (IBAND) (out_band_img[1]);
    
    /* Execute C routine. */
    return_value = logarithm(in_band, out_band, eps, offset, scale, minimum);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C logarithm.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(out_band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
