
/*C*

________________________________________________________________

        xite_logical_bnot.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int logical_bnot(IBAND input, IBAND output) */

/*M*
function [output] = xite_logical_bnot(input)
%xite_logical_bnot Logical operators on bands.
%   Calls the XITE C function logical_bnot.
%   function [output] = xite_logical_bnot(input)
%
%   The C function has the following prototype:
%   int logical_bnot(IBAND input, IBAND output)
%
%   and the following manual page:
%
%   NAME
%          logical, logical_and, logical_or, logical_xor, logical_not, logi-
%          cal_band, logical_bor, logical_bxor, logical_bnot, logical_lshift, log-
%          ical_rshift, logical_eq, logical_ne, logical_lt, logical_le, logi-
%          cal_gt, logical_ge, logical_mask, logical_bandc, logical_borc, logi-
%          cal_bxorc, logical_eqc, logical_nec, logical_ltc, logical_lec, logi-
%          cal_gtc, logical_gec - logical operators on bands
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int logical_and( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_or( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_xor( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_not( IBAND input, IBAND output );
%
%           int logical_band( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_bor( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_bxor( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_bnot( IBAND input, IBAND output );
%
%           int logical_lshift( IBAND input, IBAND output,
%              int n );
%
%           int logical_rshift( IBAND input, IBAND output,
%              int n );
%
%           int logical_eq( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_ne( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_lt( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_le( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_gt( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_ge( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_mask( IBAND input1, IBAND input2,
%              IBAND output );
%
%           int logical_bandc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_borc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_bxorc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_eqc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_nec( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_ltc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_lec( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_gtc( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%           int logical_gec( IBAND input, IBAND output,
%              UNS_BYTE c );
%
%   DESCRIPTION
%          logical is a collection of logical operators on bands.  For every pixel
%          position, the unary or binary operator is performed on the input
%          band(s), and the result is assigned to the output band.
%
%           logical_and - logical and (MIN operator)
%           logical_or - logical or (MAX operator)
%           logical_xor - logical xor
%           logical_not - logical not
%           logical_band - bitwise and
%           logical_bor - bitwise or
%           logical_bxor - bitwise xor
%           logical_bnot - bitwise not
%           logical_lshift - left shift
%           logical_rshift - right shift
%           logical_eq - equality
%           logical_ne - not equality
%           logical_lt - less than
%           logical_le - less equal
%           logical_gt - greater than
%           logical_ge - greater equal
%           logical_mask - (input2) ? input1 : 0
%           logical_bandc - bitwise and with constant c
%           logical_borc - bitwise or with constant c
%           logical_bxorc - bitwise xor with constant c
%           logical_eqc - equal to constant
%           logical_nec - not equal to constant
%           logical_ltc - less than constant
%           logical_lec - less equal constant
%           logical_gtc - greater than constant
%           logical_gec - greater equal constant
%
%          Only the largest rectangle common for all bands is processed. Identical
%          actual band arguments are allowed.
%
%   RESTRICTIONS
%          Only pixeltype unsigned byte is allowed.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel value input band 1
%           2 => bad pixel value input band 2
%           3 => bad pixel value output band
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: logical.c,v 1.34 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 LOGICAL(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"logical_bnot"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = logical_bnot(input, output);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C logical_bnot.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
