
/*C*

________________________________________________________________

        xite_multiply.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int multiply(IBAND band1, IBAND band2, IBAND output, double scale) */

/*M*
function [output] = xite_multiply(band1, band2, scale)
%xite_multiply Multiply two bands pixel by pixel.
%   Calls the XITE C function multiply.
%   function [output] = xite_multiply(band1, band2, scale)
%
%   The C function has the following prototype:
%   int multiply(IBAND band1, IBAND band2, IBAND output, double scale)
%
%   and the following manual page:
%
%   NAME
%          multiply, fftMult - multiply two bands pixel by pixel
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int multiply( IBAND band1, IBAND band2,
%              IBAND output, double scale );
%
%           #include <xite/fft.h>
%
%           int fftMult( IC_BAND in_band1,
%              IC_BAND in_band2, IC_BAND out_band );
%
%   DESCRIPTION
%          multiply calculates, pixel by pixel, the product of band1, band2, and
%          scale, and assigns the result to output.
%
%           output(x,y) = band1(x,y) * band2(x,y) * scale
%
%          Resulting values outside the domain of the actual pixel type of output
%          is truncated to the nearest legal value.
%
%          fftMult is retained for compatibility with previous XITE versions. It
%          is a macro call to multiply with scale equal to 1.0.
%
%   RESTRICTIONS
%          All pixel types are accepted, as long as all three bands have the same
%          pixel type.  There are two exceptions: band2 may be real if band1 and
%          band3 are complex, and band2 may be double if band1 and band3 are dou-
%          ble complex.
%
%          Only the largest rectangle common to all three bands is multiplied.
%
%   RETURN VALUE
%           Status, 0 => ok
%           1 => Illegal combination of pixel types
%           2 => Unknown pixel type
%
%   SEE ALSO
%          multiply(1), divide(3), absDiff(3), signDiff(3), addw(3), average(3),
%          scale(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   MODIFIED
%          Svein Bøe, BLAB, Ifi, UiO (allowed combinations of complex times real
%          and double complex times double - this takes care of the capabilities
%          of fftMult).
%
%   ID
%          $Id: multiply.c,v 1.40 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 MULTIPLY(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"multiply"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band1;
    IBAND band2;
    IBAND output;
    double scale;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band1_img;
    IMAGE band2_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band1_img = Init_image(1, "");
    
    convert_status = matlab2biff(band1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band1 = (IBAND) (band1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band1));
    band2_img = Init_image(1, "");
    
    convert_status = matlab2biff(band2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    band2 = (IBAND) (band2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band2));
    scale = ((double *) mxGetData(prhs[2]))[0];

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = multiply(band1, band2, output, scale);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C multiply.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
