
/*C*

________________________________________________________________

        xite_power.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int power(IBAND input, IBAND output, double scale) */

/*M*
function [output] = xite_power(input, scale)
%xite_power Take the power of each pixel value.
%   Calls the XITE C function power.
%   function [output] = xite_power(input, scale)
%
%   The C function has the following prototype:
%   int power(IBAND input, IBAND output, double scale)
%
%   and the following manual page:
%
%   NAME
%          power, fftPower - take the power of each pixel value
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int power( IBAND input, IBAND output,
%              double scale );
%
%           #include <xite/fft.h>
%
%           int fftPower( IC_BAND in_band,
%              IR_BAND out_band );
%
%   DESCRIPTION
%          power finds the absolute magnitude squared of each pixel in input and
%          inserts this value in output.
%
%           output(x,y) = |input(x,y)| * |input(x,y)| * scale
%
%          Only the largest rectangle common to the two bands is powerd.
%
%          Accepts all pixel types. Both bands must have the same pixel type,
%          except that if input is complex, then output may be either complex or
%          real, and if input is double complex, then output may be either double
%          complex or double.
%
%          If the result is outside the domain of the actual pixel type, the
%          result is rounded.
%
%          Identical arguments for input and output are allowed.
%
%          power is identical to square for non-complex bands.
%
%          fftPower is retained for compatibility with previous XITE versions.  It
%          is a macro call to power with scale equal to 1.0
%
%   RETURN VALUE
%           0 => ok
%           1 => Illegal combination of pixel types
%           2 => Unknown pixel type
%
%   SEE ALSO
%          power(1), square(3), squareRoot(3), phase(3), absValue(3), imag(3),
%          real(3), multiply(3), divide(3), scale(3), convertBand(3)
%
%   AUTHOR
%          Svein Bøe, BLAB, Ifi, UiO
%
%   ID
%          $Id: power.c,v 1.13 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 POWER(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"power"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    double scale;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    scale = ((double *) mxGetData(prhs[1]))[0];

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = power(input, output, scale);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C power.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
