
/*C*

________________________________________________________________

        xite_profile.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int profile(IBAND band, double x, double y, double direction, double step, double dist, int ip, double *arr, int maxlen) */

/*M*
function [return_value, arr] = xite_profile(band, x, y, direction, step, dist, ip, maxlen)
%xite_profile Calculate pixel values along a line.
%   Calls the XITE C function profile.
%   function [return_value, arr] = xite_profile(band, x, y, direction, step, dist, ip, maxlen)
%
%   The C function has the following prototype:
%   int profile(IBAND band, double x, double y, double direction, double step, double dist, int ip, double *arr, int maxlen)
%
%   and the following manual page:
%
%   NAME
%          profile - Calculate pixel values along a line.
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int profile( IBAND band, double x, double y,
%              double direction, double step, double dist,
%              int ip, double* arr, int maxlen );
%
%   DESCRIPTION
%          profile calculates the pixel values along a line starting at (x,y) in
%          the direction direction.
%
%          band is a BIFF image band of any type.  x and y make up the start point
%          of the profile.  direction is the direction of the profile in radians.
%          step is the distance between two points in the profile.   dist is the
%          length of the profile.  ip is the interpolation degree (only ip = 1 is
%          implemented).  arr is an array of return values, starting on array
%          index 1 where the profile is stored.  maxlen is the maximum length of
%          the profile (length of arr)
%
%   RETURN VALUE
%          Length of profile, or negative value on error.
%
%   SEE ALSO
%          crossSection(3)
%
%   AUTHOR
%          Otto Milvang
%
%                                     27 Feb 2014 PROFILE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"profile"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    double x;
    double y;
    double direction;
    double step;
    double dist;
    int ip;
    double *arr;
    int maxlen;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 8) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 8) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    x = ((double *) mxGetData(prhs[1]))[0];
    y = ((double *) mxGetData(prhs[2]))[0];
    direction = ((double *) mxGetData(prhs[3]))[0];
    step = ((double *) mxGetData(prhs[4]))[0];
    dist = ((double *) mxGetData(prhs[5]))[0];
    
    /* If scalar double is passed in for "ip" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        ip = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        ip = ((int *) mxGetData(prhs[6]))[0];
    }
    
    /* If scalar double is passed in for "maxlen" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[7]) && mxGetM(prhs[7]) && mxGetN(prhs[7])) {
        maxlen = (int) (((double *) mxGetData(prhs[7]))[0]);
    } else {
        maxlen = ((int *) mxGetData(prhs[7]))[0];
    }

    /* Prepare C result variable. */
    arr = (double *) mxCalloc(maxlen, sizeof(double));
    
    /* Execute C routine. */
    return_value = profile(band, x, y, direction, step, dist, ip, arr, maxlen);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    /* Convert to MATLAB arrays. */
    if (nlhs > 1) {
        tmp_plhs[1] =    mxCreateNumericMatrix(maxlen, 1, mxDOUBLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[1]), arr, (maxlen) * sizeof(double));
        plhs[1] = tmp_plhs[1];
    }

    PopMessage();

    return;
}
