
/*C*

________________________________________________________________

        xite_rms.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* double rms(IBAND band1, IBAND band2, int edge) */

/*M*
function [return_value] = xite_rms(band1, band2, edge)
%xite_rms Root mean square difference between two bands.
%   Calls the XITE C function rms.
%   function [return_value] = xite_rms(band1, band2, edge)
%
%   The C function has the following prototype:
%   double rms(IBAND band1, IBAND band2, int edge)
%
%   and the following manual page:
%
%   NAME
%          rms - root mean square difference between two bands
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           double rms( IBAND band1, IBAND band2,
%              int edge );
%
%   DESCRIPTION
%          The root mean square difference between the two input bands is calcu-
%          lated and written to stdout.
%
%           rms = SQRT{ SUM [ |band1[y][x] - band2[y][x]| **2 ] / N }
%
%          where SUM is over all N pixels in the band minus an edge of width edge
%          which is not included in the calculaton. The purpose of this is to
%          exclude differences due to different processing of the edge by local
%          filters.
%
%          All pixeltypes are allowed, but band1 and band2 must be of the same
%          pixeltype.
%
%   RETURN VALUE
%           >= 0 : The rms difference
%           -1 : Different pixeltypes in the two bands
%           -2 : Unknown pixeltype
%
%   SEE ALSO
%          rms(1), absDiff(3), signDiff(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO.
%
%   MODIFIED
%          Svein Bøe, allowing all pixeltypes.
%
%   ID
%          $Id: rms.c,v 1.35 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 RMS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"rms"};
    double return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band1;
    IBAND band2;
    int edge;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band1_img;
    IMAGE band2_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band1_img = Init_image(1, "");
    
    convert_status = matlab2biff(band1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band1 = (IBAND) (band1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band1));
    band2_img = Init_image(1, "");
    
    convert_status = matlab2biff(band2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    band2 = (IBAND) (band2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band2));
    
    /* If scalar double is passed in for "edge" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        edge = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        edge = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* Execute C routine. */
    return_value = rms(band1, band2, edge);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
    ((double *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
