
/*C*

________________________________________________________________

        xite_rowMaxBand.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* double rowMaxBand(IBAND band, int y) */

/*M*
function [return_value] = xite_rowMaxBand(band, y)
%xite_rowMaxBand Maximum and minimum of BIFF band row or column.
%   Calls the XITE C function rowMaxBand.
%   function [return_value] = xite_rowMaxBand(band, y)
%
%   The C function has the following prototype:
%   double rowMaxBand(IBAND band, int y)
%
%   and the following manual page:
%
%   NAME
%          vectorPeak, rowMaxBand, colMaxBand, rowMinBand, colMinBand, rowMax,
%          colMax, rowMin, colMin - Maximum and minimum of BIFF band row or column
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           double rowMaxBand( IBAND band, int y );
%           double colMaxBand( IBAND band, int x );
%           double rowMinBand( IBAND band, int y );
%           double colMinBand( IBAND band, int x );
%
%           double rowMax( IMAGE img, int bandno, int y );
%           double colMax( IMAGE img, int bandno, int x );
%           double rowMin( IMAGE img, int bandno, int y );
%           double colMin( IMAGE img, int bandno, int x );
%
%   DESCRIPTION
%          rowMaxBand returns the maximum value among the pixels in row number y
%          of band.
%
%          rowMax behaves like rowMaxBand, except on band number bandno in image
%          img.
%
%          The image versions of these functions are macros calling the band ver-
%          sions. The image versions are retained for compatibility with previous
%          XITE versions.
%
%          All pixeltypes are accepted.  If a band is of type Icomplex_typ or
%          Id_complex_typ, the absolute values are used.
%
%   RETURN VALUE
%          The returning value is a double.
%
%   SEE ALSO
%          crossSection(3), crossSection(1)
%
%   AUTHOR
%          Svein Bøe, BLAB, Ifi, UiO
%
%                                     27 Feb 2014 VECTORPEAK(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"rowMaxBand"};
    double return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    int y;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    
    /* If scalar double is passed in for "y" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        y = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        y = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* Execute C routine. */
    return_value = rowMaxBand(band, y);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
    ((double *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
