
/*C*

________________________________________________________________

        xite_signDiff.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/arithmetic.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int signDiff(IBAND input1, IBAND input2, IBAND output, double scale, double offset) */

/*M*
function [output] = xite_signDiff(input1, input2, scale, offset)
%xite_signDiff Signed difference between two bands.
%   Calls the XITE C function signDiff.
%   function [output] = xite_signDiff(input1, input2, scale, offset)
%
%   The C function has the following prototype:
%   int signDiff(IBAND input1, IBAND input2, IBAND output, double scale, double offset)
%
%   and the following manual page:
%
%   NAME
%          signDiff - signed difference between two bands
%
%   SYNTAX
%           #include <xite/arithmetic.h>
%
%           int signDiff( IBAND input1, IBAND input2,
%              IBAND output, double scale, double offset );
%
%   DESCRIPTION
%          signDiff calculates, pixel by pixel, the signed difference between the
%          two input images:
%
%           output(x,y) =
%               (input1(x,y)-input2(x,y))*scale + offset
%
%          For complex and double-complex bands, the result is
%
%           output(x,y).re =
%               (input1(x,y).re-input2(x,y).re)*scale + offset
%           output(x,y).im =
%               (input1(x,y).im-input2(x,y).im)*scale + offset
%
%          If the result is outside the pixeltype range, the result is truncated.
%          Within the pixeltype range, the result is rounded to the nearest possi-
%          ble pixelvalue.
%
%   RESTRICTIONS
%          All bands must be of the same type. Only the largest rectangle common
%          to alle three bands is processed.
%
%   RETURN VALUE
%           0 => ok
%           1 => bands have different pixel types
%           2 => unknown pixel types
%
%   SEE ALSO
%          signDiff(1), absDiff(3), addw(3), multiply(3), divide(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   REVISED
%          Svein Bøe, Ifi, UiO. Implemented for all pixel types, not only unsigned
%          byte.
%
%   ID
%          $Id: signDiff.c,v 1.37 2014/01/31 12:33:52 svein Exp $
%
%                                     27 Feb 2014 SIGNDIFF(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"signDiff"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input1;
    IBAND input2;
    IBAND output;
    double scale;
    double offset;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input1_img;
    IMAGE input2_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input1_img = Init_image(1, "");
    
    convert_status = matlab2biff(input1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input1 = (IBAND) (input1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input1));
    input2_img = Init_image(1, "");
    
    convert_status = matlab2biff(input2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    input2 = (IBAND) (input2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input2));
    scale = ((double *) mxGetData(prhs[2]))[0];
    offset = ((double *) mxGetData(prhs[3]))[0];

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = signDiff(input1, input2, output, scale, offset);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C signDiff.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
