
/*C*

________________________________________________________________

        xite_bin_be.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_be(int region_size, int contrast_limit, int set_doubt_to_low, IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_bin_be(region_size, contrast_limit, set_doubt_to_low, inband)
%xite_bin_be Binarize a gray-scale band using Bernsen‘s method.
%   Calls the XITE C function bin_be.
%   function [outband] = xite_bin_be(region_size, contrast_limit, set_doubt_to_low, inband)
%
%   The C function has the following prototype:
%   int bin_be(int region_size, int contrast_limit, int set_doubt_to_low, IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          bin_be - binarize a gray-scale band using Bernsen‘s method
%
%   SYNTAX
%           #include <xite/binarize.h>
%           int bin_be( int region_size,
%              int contrast_limit, int set_doubt_to_low,
%              IBAND inband, IBAND outband );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Bernsen, the
%          result returned in outband.
%
%          For each pixel, the threshold
%
%          T(x,y) = (Z_low + Z_high)/2
%
%          is used, where Z_low and Z_high are the lowest and highest gray level
%          pixel values in a square neighborhood of size (  region_size x
%          region_size) about the pixel (x,y).  However, if the contrast measure
%
%          C(x,y) = Z_high - Z_low < contrast_limit
%
%          the pixel is set to 255 if set_doubt_to_low ==  0, and to 0 if
%          set_doubt_to_low >= 1.
%
%   FILES
%          src/binarize/bin_be.c
%
%   SEE ALSO
%          binarize(1), thresBernsen(1), thresBernsen(3)
%
%   REFERENCES
%          ..I John Bernsen
%                 "Dynamic thresholding of grey-level images", Proc. 8th Interna-
%                 tional Conference on Pattern Recognition (ICPR8), pp 1251-1255,
%                 Paris, France, October 1986.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   AUTHOR
%          Oivind Due Trier
%
%   EXAMPLES
%   ID
%          $Id: bin_be.c,v 1.15 1997/01/14 15:46:50 svein Exp $
%
%                                                                   3(27 Feb 2014)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_be"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    int region_size;
    int contrast_limit;
    int set_doubt_to_low;
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "region_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        region_size = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        region_size = ((int *) mxGetData(prhs[0]))[0];
    }
    
    /* If scalar double is passed in for "contrast_limit" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        contrast_limit = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        contrast_limit = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "set_doubt_to_low" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        set_doubt_to_low = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        set_doubt_to_low = ((int *) mxGetData(prhs[2]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[3], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_be(region_size, contrast_limit, set_doubt_to_low, inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_be.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
