
/*C*

________________________________________________________________

        xite_bin_etm.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_etm(int region_size, int step_size, double dist_limit, double weight, int glob_min, int glob_max, int bin_method, IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_bin_etm(region_size, step_size, dist_limit, weight, glob_min, glob_max, bin_method, inband)
%xite_bin_etm Binarize a gray-scale band using Eikvil, Taxt and Moen‘s method.
%   Calls the XITE C function bin_etm.
%   function [outband] = xite_bin_etm(region_size, step_size, dist_limit, weight, glob_min, glob_max, bin_method, inband)
%
%   The C function has the following prototype:
%   int bin_etm(int region_size, int step_size, double dist_limit, double weight, int glob_min, int glob_max, int bin_method, IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          bin_etm -  binarize a gray-scale band using Eikvil, Taxt and Moen‘s
%          method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_etm( int region_size, int step_size,
%              double dist_limit, double weight,
%              int glob_min, int glob_max, int bin_method,
%              IBAND inband, IBAND outband );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Eikvil, Taxt and
%          Moen, the result returned in outband.
%
%          The pixels inside a small window S of size ( small_window_size x
%          small_window_size) are thresholded on basis of the clustering of the
%          pixels inside a larger concentric window L of size ( region_size x
%          region_size) S and L slide across the image in steps equal to the side
%          of S.   For all pixels inside L, a threshold T is calculated to divide
%          the pixels into two clusters. If the two estimated cluster means mu1
%          and mu2 are further apart than a limit, mu2 - mu1 > mean_diff_limit the
%          pixels inside S are binarized using the threshold value T.  When mu2 -
%          mu1 <  mean_diff_limit, the pixels inside S are all assigned to the
%          class with the closest updated mean value, being it print or back-
%          ground.
%
%          bin_method is used to select the global binarization method which finds
%          T.  Use bin_method = OTSU or bin_method = KITTLER_ILLINGWORTH.
%
%          weight is used to calculate the updated mean values MU1 and MU2.  For
%          each small window S, the new values are caluculated as
%
%           MU1 = ’weight’ * MU1 + (1.0-’weight’) * mu1
%           MU2 = ’weight’ * MU2 + (1.0-’weight’) * mu2
%
%          weight = 1.0 means that the initial estimates for MU1 and MU2 are kept
%          forever, while weight = 0.0 means that the mean values mu1 and mu2 of
%          the two clusters in L are used in the next step.
%
%          For extremely difficult images, a preprocessing step can be used on the
%          image. Replace all pixel values higher than the value glob_max by
%          glob_max itself, and similarly, replace all pixel values lower than
%          glob_min by glob_min.
%
%   FILES
%          src/binarize/bin_etm.c
%
%   SEE ALSO
%          binarize(1), bin_ot(3), bin_ki(3)
%
%   REFERENCES
%          ..I L.. Eikvil,, T.. Taxt && K.. Moen
%                 "An Adaptive Method for Binarization of Grey Level Images",
%                 NOBIM National Conference on Image Processing and Pattern Recog-
%                 nition, pp 123-131, June 1991
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_etm.c,v 1.18 2014/02/04 14:37:47 svein Exp $
%
%                                                                   3(27 Feb 2014)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_etm"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    int region_size;
    int step_size;
    double dist_limit;
    double weight;
    int glob_min;
    int glob_max;
    int bin_method;
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 8) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 8) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "region_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        region_size = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        region_size = ((int *) mxGetData(prhs[0]))[0];
    }
    
    /* If scalar double is passed in for "step_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        step_size = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        step_size = ((int *) mxGetData(prhs[1]))[0];
    }
    dist_limit = ((double *) mxGetData(prhs[2]))[0];
    weight = ((double *) mxGetData(prhs[3]))[0];
    
    /* If scalar double is passed in for "glob_min" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        glob_min = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        glob_min = ((int *) mxGetData(prhs[4]))[0];
    }
    
    /* If scalar double is passed in for "glob_max" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        glob_max = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        glob_max = ((int *) mxGetData(prhs[5]))[0];
    }
    
    /* If scalar double is passed in for "bin_method" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        bin_method = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        bin_method = ((int *) mxGetData(prhs[6]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[7], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_etm(region_size, step_size, dist_limit, weight, glob_min, glob_max, bin_method, inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_etm.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
