
/*C*

________________________________________________________________

        xite_bin_mh.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_mh(IBAND inband, IBAND outband, int use_sem_var, double convergence_limit) */

/*M*
function [outband] = xite_bin_mh(inband, use_sem_var, convergence_limit)
%xite_bin_mh Binarize a gray-scale band using Mardia and Hainsworth‘s method.
%   Calls the XITE C function bin_mh.
%   function [outband] = xite_bin_mh(inband, use_sem_var, convergence_limit)
%
%   The C function has the following prototype:
%   int bin_mh(IBAND inband, IBAND outband, int use_sem_var, double convergence_limit)
%
%   and the following manual page:
%
%   NAME
%          bin_mh -  binarize a gray-scale band using Mardia and Hainsworth‘s
%          method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_mh( IBAND inband, IBAND outband,
%              int use_sem_var, double convergence_limit );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Mardia and
%          Hainsworth. The result is returned in outband.
%
%          Mardia and Hainsworth‘s method first makes an initial binarization.
%          Then several steps are iterated until convergence is reached.  First,
%          the estimated mean values mu1 and mu2, and the number of pixels n_i in
%          both foreground and background of the current binarization are calcu-
%          lated.  Then a threshold T based on these values is calculated. Then,
%          for each pixel, a weighted mean, G, of the pixel and its eight neigh-
%          bors is calculated.  If G <= T, the pixel is classified as ‘‘black’,
%          otherwise ‘‘white’. The weights on the neighbor pixels, used to calcu-
%          late G, may be equal, or they may be calculated based on a semivari-
%          ogram approximation. If use_sem_var >= 1, the semivariogram approach is
%          used. The last step of each iteration is to smooth the new binary image
%          using a (3 x 3) median filter.  convergence_limit is used to test if
%          convergence has been reached.  The number of different pixels of two
%          successive binarizations is counted, and compared to convergence_limit.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          K.V.  Mardia & T.J. Hainsworth "A spatial thresholding method for image
%          segmentation", IEEE Transactions on Pattern Analysis and Machine Intel-
%          ligence, vol T-PAMI 10, no 6, pp 919-927, 1988.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_mh.c,v 1.17 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_MH(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_mh"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int use_sem_var;
    double convergence_limit;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "use_sem_var" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        use_sem_var = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        use_sem_var = ((int *) mxGetData(prhs[1]))[0];
    }
    convergence_limit = ((double *) mxGetData(prhs[2]))[0];

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_mh(inband, outband, use_sem_var, convergence_limit);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_mh.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
