
/*C*

________________________________________________________________

        xite_bin_nr.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_nr(double sdev_limit, double mean_limit, double min_ratio, double max_ratio, double valley_to_peak, int region_size, IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_bin_nr(sdev_limit, mean_limit, min_ratio, max_ratio, valley_to_peak, region_size, inband)
%xite_bin_nr Binarize a gray-scale band using Nakagawa and Rosenfeld‘s method.
%   Calls the XITE C function bin_nr.
%   function [outband] = xite_bin_nr(sdev_limit, mean_limit, min_ratio, max_ratio, valley_to_peak, region_size, inband)
%
%   The C function has the following prototype:
%   int bin_nr(double sdev_limit, double mean_limit, double min_ratio, double max_ratio, double valley_to_peak, int region_size, IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          bin_nr -  binarize a gray-scale band using Nakagawa and Rosenfeld‘s
%          method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_nr( double sdev_limit,
%              double mean_limit, double min_ratio,
%              double max_ratio, double valley_to_peak,
%              int region_size, IBAND inband,
%              IBAND outband );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Nakagawa and
%          Rosenfeld.  The method is a slight modification of Chow and Kaneko‘s
%          method. The result is returned in outband.
%
%          This method calculates a thresholding surface.  The image is divided
%          into windows of size ( region_size x region_size), and the histograms
%          for each window are tested for bimodality.  Each histogram is approxi-
%          mated by a mixture of two Gaussian distributions. Let mu1 and mu2 be
%          the estimated mean values, and sigma1 and sigma2 be the estimated stan-
%          dard deviations of the two distributions. Also, let sigma_tot be the
%          total standard deviation, and vtp be the valley-to-peak ratio.  The
%          bimodality test requires the following five criteria to hold:
%
%          (1) sigma_tot > sdev_limit
%
%          (2) mu2 - mu1 > mean_limit
%
%          (3) sigma1/sigma2 > min_ratio
%
%          (4) sigma1/sigma2 > min_ratio
%
%          (5) vtp < valley_to_peak_limit
%
%          For each bimodal window, a threshold is calculated based on the mu_1,
%          mu_2, sigma_1 and sigma_2 parameters of the mixture distribution.
%
%          Thresholds for the other windows are calculated on basis of the thresh-
%          olds for the bimodal windows. The thresholds are first smoothed, and
%          then interpolated to give a threshold surface, having a single thresh-
%          old value for each image pixel.   The image is binarized using the
%          threshold surface.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          [[1]]    C.K. Chow & T. Kaneko "Automatic detection of the left ventricle
%                 from cineangiograms", Computers and biomedical research, vol. 5,
%                 pp 388-410, 1972.
%
%          [[2]]    Yasuo Nakagawa & Azriel Rosenfeld "Some experiments on variable
%                 thresholding", Pattern Recognition, vol 11, no 3, pp 191-204,
%                 1979.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   FILES
%          src/binarize/bin_nr.c
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_nr.c,v 1.17 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_NR(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_nr"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    double sdev_limit;
    double mean_limit;
    double min_ratio;
    double max_ratio;
    double valley_to_peak;
    int region_size;
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    sdev_limit = ((double *) mxGetData(prhs[0]))[0];
    mean_limit = ((double *) mxGetData(prhs[1]))[0];
    min_ratio = ((double *) mxGetData(prhs[2]))[0];
    max_ratio = ((double *) mxGetData(prhs[3]))[0];
    valley_to_peak = ((double *) mxGetData(prhs[4]))[0];
    
    /* If scalar double is passed in for "region_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        region_size = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        region_size = ((int *) mxGetData(prhs[5]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[6], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_nr(sdev_limit, mean_limit, min_ratio, max_ratio, valley_to_peak, region_size, inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_nr.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
