
/*C*

________________________________________________________________

        xite_bin_ot.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_ot(IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_bin_ot(inband)
%xite_bin_ot Binarize a gray-scale band using Otsu‘s method.
%   Calls the XITE C function bin_ot.
%   function [outband] = xite_bin_ot(inband)
%
%   The C function has the following prototype:
%   int bin_ot(IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          bin_ot, otsu_threshold - binarize a gray-scale band using Otsu‘s method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_ot( IBAND inband, IBAND outband );
%           int otsu_threshold( IBAND inband, double* mu_0,
%              double* mu_1, double* mu );
%
%   DESCRIPTION
%          The input band inband is binarized by bin_ot using the method of Otsu.
%          The result is returned in outband.
%
%          otsu_threshold returns Otsu‘s threshold for inband.  The total mean mu
%          and the two cluster means mu1 and mu2 are calculated.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          ..I Nobuyuki Otsu
%                 "A threshold selection method from gray-level histograms", IEEE
%                 transactions on Systems, man and cybernetics, vol T-SMC 9, no 1,
%                 pp 62-66, 1979.
%
%   RETURN VALUE
%           bin_ot :  0 : OK
%                             1 : Bad input pixel type
%                             2 : Bad output pixel type
%           otsu_threshold :  Otsu‘s threshold
%
%   FILES
%          src/binarize/bin_ot.c
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_ot.c,v 1.18 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_OT(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_ot"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_ot(inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_ot.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
