
/*C*

________________________________________________________________

        xite_bin_wr2.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_wr2(int mode, int activity_limit, double distance_limit, int mean_size, int search_vec_len, int diagonal, int max_limit, int spot_size_limit, IBAND inband, IBAND outband, int intermediate_images) */

/*M*
function [outband] = xite_bin_wr2(mode, activity_limit, distance_limit, mean_size, search_vec_len, diagonal, max_limit, spot_size_limit, inband, intermediate_images)
%xite_bin_wr2 Binarize a gray-scale band using White and Rohrer‘s method.
%   Calls the XITE C function bin_wr2.
%   function [outband] = xite_bin_wr2(mode, activity_limit, distance_limit, mean_size, search_vec_len, diagonal, max_limit, spot_size_limit, inband, intermediate_images)
%
%   The C function has the following prototype:
%   int bin_wr2(int mode, int activity_limit, double distance_limit, int mean_size, int search_vec_len, int diagonal, int max_limit, int spot_size_limit, IBAND inband, IBAND outband, int intermediate_images)
%
%   and the following manual page:
%
%   NAME
%          bin_wr2, bin_iwr - binarize a gray-scale band using White and Rohrer‘s
%          method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_wr2( int mode, int activity_limit,
%              double distance_limit, int mean_size,
%              int search_vec_len, int diagonal,
%              int max_limit, int spot_size_limit,
%              IBAND inband, IBAND outband,
%              int intermediate_images );
%
%           int bin_iwr( int mode, int activity_limit,
%              int mean_size, int max_limit,
%              int spot_size_limit, IBAND inband,
%              IBAND outband, int intermediate_images );
%
%   DESCRIPTION
%          The input band inband is binarized using the Integrated function Algo-
%          rithm of White and Rohrer, or an improved version. The result is
%          returned in outband.
%
%          Several improvements and variations of the method have been imple-
%          mented;  some of these are available in bin_wr2, but to get all the
%          improvements, bin_iwr must be used followed by remove_ghost.
%
%          White and Rohrer‘s method uses a gradient-like operator called the
%          activity on the image. Pixels with activity below activity_limit are
%          labeled 0.  The other pixels are further tested. If the Laplacian edge
%          operator of the pixel is positive, the pixel is labeled +, otherwise -.
%          That way a three-level label-image is constructed, with legal pixel
%          values +, 0 and -.  The idea is that in a sequence of labels, edges are
%          identified as -+ transitions or +-  transitions.  Object pixels are
%          assumed to be + and 0 labeled pixels between a -+ and +- pair. For each
%          pixel to be classified, a search vector of length search_vec_len for
%          each search direction is centered on the pixel. The -+ and +- must be
%          inside the seach vector to be recognized.  By using a low value on
%          search_vec_len, only lines and small filled areas will be captured.
%
%          If spot_size_limit > 1, then small + -labeled regions having less than
%          spot_size_limit pixels are relabeled as 0 -regions.
%
%          By specifying activity_limit = 0, Kitler and Illingworth‘s method is
%          used to set the activity threshold. An upper bound for this value is
%          specified by max_activity.
%
%          A lower limit on the distance between the -+ and +- for pixels to be
%          classified as object pixels is specified by distance_limit.  This can
%          be used to eliminate the thinnest lines in inband.
%
%          The first improvement suggested by Trier and Taxt [1] can be used in
%          bin_wr2:
%
%          The input image is smoothed by a mean filter of size ( mean_size x
%          mean_size) to remove stochastic noise. To run the original White and
%          Rohrer‘s method, mean_size = 1 must be used.
%
%          Variations on how pixels are classified as print or background are
%          selected though the mode parameter:
%
%          If mode = WR_TWO_BY_TWO, the original approach is used. Two times two
%          pixels are classified at a time, requiring that all four pixels are
%          inside either horizontal or vertical object pixel sequences.
%
%          The reasons for classifying (2 x 2) pixels at a time in the original
%          implementation was partly to save computational time, partly to reduce
%          noise. With faster computers, a more time-consuming algorithm may be
%          allowed.
%
%          If mode = WR_DOUBLE_VECTORS, then the idea is that a single pixel being
%          part of an object pixel sequence is not enough. It has to have support
%          from at least one of its neighbors. Either one of its vertical neigh-
%          bors and the pixel itself have to be inside horizontal object pixel
%          sequences or one of its horizontal neighbors and the pixel itself must
%          be inside vertical object pixel sequences to be recognized as object
%          pixels.
%
%          If mode = WR_SINGLE_VECTOR, only a single search vector is used in each
%          direction.
%
%          If diagonal >= 1, then the horizontal and vertical search directions
%          are complemented with two diagonal search directions. If diagonal == 0,
%          then only the horizontal and vertical search directions are used.
%
%          If intermediate_images > 0, then some intermediate images are saved
%          during the execution. This is useful for analyzing the method.
%
%          ..I bin_iwr
%
%                 bin_iwr is an improved version of bin_wr2.  The parameter list
%                 is somewhat different. The mode parameter is kept for future
%                 improvements, but is currently ignored.  The distance limit,
%                 search_vec_len, and diagonal parameters have been discarded from
%                 the parameter list, since the use of search vectors to locate
%                 print pixels in the three-level label image is abandoned in
%                 bin_iwr.
%
%                 Instead, the following approach is used. First, all " + "-marked
%                 regions are labeled print and " - "-marked regions are marked
%                 backgrond.  Then, for each " 0 "-marked region, the number of "
%                 - " and " + " pixels that are 8-connected to the region is
%                 counted.   If the number of " + " pixels is in majority, the " 0
%                 " -marked region is labeled print, otherwise background.
%
%                 This approach prodces a lot of noise which can be seen as spuri-
%                 ous print components, which must be removed by using
%                 remove_ghost afterwards.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          [[1]]    Øivind Due Trier &  Torfinn Taxt "Improvement of ‘integrated
%                 function algorithm‘  for binarization of document images", Pat-
%                 tern Recognition Letters, to appear.
%
%          [[2]]    J.M. White & G.D. Rohrer "Image Thresholding for Optical Charac-
%                 ter Recognition and Other Applications Requiring Character Image
%                 Extraction", IBM Journal of Research and Development, vol 27, no
%                 4, pp 400-411, July 1983.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   FILES
%          src/binarize/bin_wr2.c
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_wr2.c,v 1.12 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_WR2(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_wr2"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    int mode;
    int activity_limit;
    double distance_limit;
    int mean_size;
    int search_vec_len;
    int diagonal;
    int max_limit;
    int spot_size_limit;
    IBAND inband;
    IBAND outband;
    int intermediate_images;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 10) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 10) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "mode" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        mode = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        mode = ((int *) mxGetData(prhs[0]))[0];
    }
    
    /* If scalar double is passed in for "activity_limit" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        activity_limit = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        activity_limit = ((int *) mxGetData(prhs[1]))[0];
    }
    distance_limit = ((double *) mxGetData(prhs[2]))[0];
    
    /* If scalar double is passed in for "mean_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        mean_size = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        mean_size = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "search_vec_len" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        search_vec_len = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        search_vec_len = ((int *) mxGetData(prhs[4]))[0];
    }
    
    /* If scalar double is passed in for "diagonal" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        diagonal = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        diagonal = ((int *) mxGetData(prhs[5]))[0];
    }
    
    /* If scalar double is passed in for "max_limit" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        max_limit = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        max_limit = ((int *) mxGetData(prhs[6]))[0];
    }
    
    /* If scalar double is passed in for "spot_size_limit" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[7]) && mxGetM(prhs[7]) && mxGetN(prhs[7])) {
        spot_size_limit = (int) (((double *) mxGetData(prhs[7]))[0]);
    } else {
        spot_size_limit = ((int *) mxGetData(prhs[7]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[8], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "intermediate_images" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[9]) && mxGetM(prhs[9]) && mxGetN(prhs[9])) {
        intermediate_images = (int) (((double *) mxGetData(prhs[9]))[0]);
    } else {
        intermediate_images = ((int *) mxGetData(prhs[9]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_wr2(mode, activity_limit, distance_limit, mean_size, search_vec_len, diagonal, max_limit, spot_size_limit, inband, outband, intermediate_images);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_wr2.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
