
/*C*

________________________________________________________________

        xite_bin_yb.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bin_yb(IBAND inband, IBAND outband, int threshold, double beta, int max_iterations) */

/*M*
function [outband] = xite_bin_yb(inband, threshold, beta, max_iterations)
%xite_bin_yb Binarize a gray-scale band using Yanowitz and Bruckstein‘s
%   method.
%   Calls the XITE C function bin_yb.
%   function [outband] = xite_bin_yb(inband, threshold, beta, max_iterations)
%
%   The C function has the following prototype:
%   int bin_yb(IBAND inband, IBAND outband, int threshold, double beta, int max_iterations)
%
%   and the following manual page:
%
%   NAME
%          bin_yb -  binarize a gray-scale band using Yanowitz and Bruckstein‘s
%          method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_yb( IBAND inband, IBAND outband,
%              int threshold, double beta,
%              int max_iterations );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Yanowitz and
%          Bruckstein. The result is returned in outband.
%
%          A potential surface passing through the image at local maxima of the
%          gradient image is used as a thresholding surface. The gradient magni-
%          tude image is computed, using for example Sobel‘s edge operators. This
%          image is thresholded using threshold.   However, if threshold ==  0,
%          Otsu‘s method is used. The thresholded image is thinned to one-pixel
%          wide lines, to identify edge pixels. The original image is smoothed by
%          a (3x3) mean filter to remove stochastic noise. A potential surface is
%          constructedby an iterative interpolation scheme.   The interpolation
%          surface is made to pass through the edge pixels. An interpolation
%          residual R(x,y) is calculated for each non-edge pixel, and the new
%          pixel value P(x,y) at iteration n is calculated as
%
%            P[n](x,y) = P[n-1](x,y) + (beta * R[n](x,y)) / 4
%
%          Yanowitz and Bruckstein recommended to use 1.0 < beta < 2.0 for fast
%          convergence (over-relaxation). However, we found that by using beta >
%          1.0, P(x,y)  started to diverge after 5-10 iterations. So beta <= 1.0
%          must be used, and beat = 1.0 works well. A maximum number of max_itera-
%          tions is perfromed.
%
%          Next, the image is binarized using the thresholding surface.  In the
%          original algorithm, so-called "ghost" objects are removed by a valida-
%          tion step. This validation step is in a separate library function named
%          rem_ghost.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned band.
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          ..I S..D.. Yanowitz && A..M.. Bruckstein
%                 "A New Method for Image Segmentation", Comput.  Vision Graphics
%                 Image Proc., vol 46, no 1, pp 82-95, April 1989.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   FILES
%          src/binarize/bin_yb.c
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_yb.c,v 1.17 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_YB(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bin_yb"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int threshold;
    double beta;
    int max_iterations;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "threshold" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        threshold = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        threshold = ((int *) mxGetData(prhs[1]))[0];
    }
    beta = ((double *) mxGetData(prhs[2]))[0];
    
    /* If scalar double is passed in for "max_iterations" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        max_iterations = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        max_iterations = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = bin_yb(inband, outband, threshold, beta, max_iterations);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bin_yb.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
