
/*C*

________________________________________________________________

        xite_expand_edge.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int expand_edge(IBAND inband, int zero_expand, int even_expand, int expand_depth, IBAND outband) */

/*M*
function [outband] = xite_expand_edge(inband, zero_expand, even_expand, expand_depth)
%xite_expand_edge Expand image by edge mirroring.
%   Calls the XITE C function expand_edge.
%   function [outband] = xite_expand_edge(inband, zero_expand, even_expand, expand_depth)
%
%   The C function has the following prototype:
%   int expand_edge(IBAND inband, int zero_expand, int even_expand, int expand_depth, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          expand_edge - Expand image by edge mirroring.
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int expand_edge( IBAND inband, int zero_expand,
%              int even_expand, int expand_depth,
%              IBAND outband );
%
%   DESCRIPTION
%          expand_edge is used on a band to avoid some types of edge difficulties.
%
%          Expands inband by expand_depth pixels on all four edges, and writes the
%          result to outband.   If the dimensions of inband are n x m, then the
%          dimensions of outband will be (n + 2*size) x (m + 2*size).
%
%          Imagine placing mirrors on the edges of inband.  These mirror bands are
%          mapped to outband.   Using expand_depth = 3 and even_expand == 1, the
%          result will be like this:
%
%                  inband:                          outband:
%
%                                      ihg|ghi xyz|zyx
%                                      fed|def . . .     uvw|wvu
%                                      cba|abc rst|tsr
%           +------- - - - -------+    ---+------- - - - -------+---
%           |abc rst|    cba|abc rst|tsr
%           |def . . .     uvw|    fed|def . . .     uvw|wvu
%           |ghi xyz|    ihg|ghi xyz|zyx
%           |                     |       |                     |
%           .                     .       .                     .
%           .                     .       .                     .
%           .                     .       .                     .
%
%          If zero_expand == 1, the new pixels are set to 0 instead.   Else, if
%          zero_expand <=  -1, then the new pixels are set to -zero_expand.  To
%          expand a binary image with a white border, use zero_expand = -255.   If
%          even_expand ==  0, the pixels closest to the edge are not mir-
%          rored/repeated.  ‘expand_depth specifies the width in pixels of the
%          sections of inband to be mirrored.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.  ‘expand_depth
%          must not exceed the smallest of the X dimension and the Y dimension of
%          inband.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad pixel type
%           2 => parameter ’expand_depth’ too large.
%           3 => outband too small.
%
%   SEE ALSO
%          expand_edge(1)
%
%   FILES
%          src/binarize/expand_edge.c
%
%   AUTHOR
%          Øivind Due Trier, BLAB, Ifi, UiO.
%
%   ID
%          $Id: expand_edge.c,v 1.21 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 EXPAND_EDGE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"expand_edge"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    int zero_expand;
    int even_expand;
    int expand_depth;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "zero_expand" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        zero_expand = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        zero_expand = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "even_expand" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        even_expand = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        even_expand = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "expand_depth" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        expand_depth = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        expand_depth = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = expand_edge(inband, zero_expand, even_expand, expand_depth, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C expand_edge.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
