
/*C*

________________________________________________________________

        xite_kapur_threshold.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int kapur_threshold(IBAND inband) */

/*M*
function [return_value] = xite_kapur_threshold(inband)
%xite_kapur_threshold Binarize a gray-scale band using Kapur, Sahoo and
%   Wong‘s method.
%   Calls the XITE C function kapur_threshold.
%   function [return_value] = xite_kapur_threshold(inband)
%
%   The C function has the following prototype:
%   int kapur_threshold(IBAND inband)
%
%   and the following manual page:
%
%   NAME
%          bin_ksw, kapur_threshold -  binarize a gray-scale band using Kapur,
%          Sahoo and Wong‘s method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_ksw( IBAND inband, IBAND outband );
%           int kapur_threshold( IBAND inband );
%
%   DESCRIPTION
%          The input band inband is binarized using the method of Kapur, Sahoo and
%          Wong, the result returned in outband.
%
%          kapur_threshold finds the threshold used in bin_ksw.
%
%   FILES
%          src/binarize/bin_ksw.c
%
%   SEE ALSO
%          binarize(1)
%
%   REFERENCES
%          ..I Kapur,, Sahoo && Wong
%                 "A new method for Gray-level picture thresholding using the
%                 entropy of the histogram", Computer Vision, Graphics, and Image
%                 Processing, vol 29, pp 273-285, 1985.
%
%   RETURN VALUE
%          For bin_ksw: 0 - ok
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   AUTHOR
%          Oivind Due Trier
%
%   EXAMPLES
%   ID
%          $Id: bin_ksw.c,v 1.15 1997/01/14 15:46:52 svein Exp $
%
%                                                                   3(27 Feb 2014)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"kapur_threshold"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* Execute C routine. */
    return_value = kapur_threshold(inband);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
