
/*C*

________________________________________________________________

        xite_kittler_threshold.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int kittler_threshold(IBAND inband, double *mu_1, double *mu_2, double *mu) */

/*M*
function [return_value, mu_1, mu_2, mu] = xite_kittler_threshold(inband)
%xite_kittler_threshold Binarize a gray-scale band using Kittler and
%   Illingworth‘s method.
%   Calls the XITE C function kittler_threshold.
%   function [return_value, mu_1, mu_2, mu] = xite_kittler_threshold(inband)
%
%   The C function has the following prototype:
%   int kittler_threshold(IBAND inband, double *mu_1, double *mu_2, double *mu)
%
%   and the following manual page:
%
%   NAME
%          bin_ki, kittler_threshold -  binarize a gray-scale band using Kittler
%          and Illingworth‘s method
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int bin_ki( IBAND inband, IBAND outband );
%           int kittler_threshold( IBAND inband,
%              double* mu_1, double* mu_2, double* mu );
%
%   DESCRIPTION
%          bin_ki binarizes inband using the global method of Kittler and Illing-
%          worth.  The result is returned in outband.
%
%          kittler_threshold returns Kittler and Illingworth‘s threshold for
%          inband.  The total mean mu and the two cluster means mu1 and mu2 are
%          calculated.
%
%   FILES
%          src/binarize/bin_ki.c
%
%   SEE ALSO
%          binarize(1), bin_ot(3)
%
%   REFERENCES
%          ..I Josef Kittler && John Illingworth
%                 "Minimum error thresholding", Pattern Recognition, vol 19, pp
%                 41-47, 1986.
%
%   RETURN VALUE
%           bin_ki : 0 - OK
%           kittler_threshold : Kittler and Illingworth‘s
%                               threshold is returned.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: bin_ki.c,v 1.18 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 BIN_KI(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"kittler_threshold"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    double mu_1;
    double mu_2;
    double mu;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 4) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
                
    /* Execute C routine. */
    return_value = kittler_threshold(inband, &mu_1, &mu_2, &mu);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    /* Convert to MATLAB arrays. */
    if (nlhs > 1) {
        tmp_plhs[1] =
            mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
        ((double *) mxGetData(tmp_plhs[1]))[0] = mu_1;
        plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =
            mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
        ((double *) mxGetData(tmp_plhs[2]))[0] = mu_2;
        plhs[2] = tmp_plhs[2];
    }
    if (nlhs > 3) {
        tmp_plhs[3] =
            mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
        ((double *) mxGetData(tmp_plhs[3]))[0] = mu;
        plhs[3] = tmp_plhs[3];
    }

    PopMessage();

    return;
}
