
/*C*

________________________________________________________________

        xite_remove_inv_ghost.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/binarize.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int remove_inv_ghost(IBAND segmented, IBAND original, IBAND outband, int val_step_threshold) */

/*M*
function [outband] = xite_remove_inv_ghost(segmented, original, val_step_threshold)
%xite_remove_inv_ghost Remove (or mark) "ghost" objects from (or in) binary
%   image.
%   Calls the XITE C function remove_inv_ghost.
%   function [outband] = xite_remove_inv_ghost(segmented, original, val_step_threshold)
%
%   The C function has the following prototype:
%   int remove_inv_ghost(IBAND segmented, IBAND original, IBAND outband, int val_step_threshold)
%
%   and the following manual page:
%
%   NAME
%          remove_ghost, remove_inv_ghost, mark_ghost, mark_inv_ghost - remove (or
%          mark) "ghost" objects from (or in) binary image
%
%   SYNTAX
%           #include <xite/binarize.h>
%
%           int remove_ghost( IBAND segmented,
%              IBAND original, IBAND outband,
%              int val_step_threshold );
%           int remove_inv_ghost( IBAND segmented,
%              IBAND original, IBAND outband,
%              int val_step_threshold );
%           int mark_ghost( IBAND segmented,
%              IBAND original, IBAND outband,
%              int val_step_threshold );
%           int mark_inv_ghost( IBAND segmented,
%              IBAND original, IBAND outband,
%              int val_step_threshold );
%
%   DESCRIPTION
%          The original gray scale image original and a binarized version seg-
%          mented are input to the routine.
%
%          ..I remove_ghost
%                 First, inband is copied to outband.  For all 4-connected black
%                 objects in outband, the average gradient magnitude value at the
%                 edge of the object is calculated. Objects having an average gra-
%                 dient below the threshold value val_step_threshold are removed
%                 from outband.
%
%          ..I remove_inv_ghost
%                 Works like remove_ghost on 4-connected white objects.
%
%          ..I mark_ghost
%                 Works like remove_ghost , but instead of removing an object, it
%                 is colored (255 - avg_grad), where avg_grad is the average gra-
%                 dient value on the edge of the object. It can then be thresh-
%                 olded manually to give the final binary image.
%
%          ..I mark_inv_ghost
%                 Combination of mark_ghost and remove_inv_ghost.   Works like
%                 remove_inv_ghost , but instead of removing an object, it is col-
%                 ored avg_grad, where avg_grad is the average gradient value on
%                 the edge of the object.
%
%   RESTRICTIONS
%          Works on unsigned byte bands.
%
%   RETURN VALUE
%           0 : ok
%           1 : Bad pixel type for segmented
%           2 : Bad pixel type for original
%           3 : Bad pixel type for outband
%
%   SEE ALSO
%          binarize(1), bin_yb(3), bin_iwr(3)
%
%   REFERENCES
%          ..I S..D.. Yanowitz && A..M.. Bruckstein
%                 "A New Method for Image Segmentation", Comput. Vision Graphics
%                 Image Proc., vol 46, no 1, pp 82-95, April 1989.
%
%   FILES
%          src/binarize/remove_ghost.c
%
%   AUTHOR
%          Øivind Due Trier
%
%   ID
%          $Id: remove_ghost.c,v 1.19 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 REMOVE_GHOST(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"remove_inv_ghost"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND segmented;
    IBAND original;
    IBAND outband;
    int val_step_threshold;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE segmented_img;
    IMAGE original_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    segmented_img = Init_image(1, "");
    
    convert_status = matlab2biff(segmented_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    segmented = (IBAND) (segmented_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) segmented));
    output_ysize = MAX(output_ysize, Iysize((IBAND) segmented));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) segmented));
    original_img = Init_image(1, "");
    
    convert_status = matlab2biff(original_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    original = (IBAND) (original_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) original));
    output_ysize = MAX(output_ysize, Iysize((IBAND) original));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) original));
    
    /* If scalar double is passed in for "val_step_threshold" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        val_step_threshold = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        val_step_threshold = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = remove_inv_ghost(segmented, original, outband, val_step_threshold);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C remove_inv_ghost.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
