
/*C*

________________________________________________________________

        xite_canny.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/canny.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int canny(IBAND input, IBAND magnitude, IBAND angle, IBAND laplace, int lngth, int h1[], int h2[], int h3[], double scale, int thin, int verbose) */

/*M*
function [magnitude, angle, laplace] = xite_canny(input, lngth, h1, h2, h3, scale, thin, verbose)
%xite_canny Detect edges in image using Canny operator.
%   Calls the XITE C function canny.
%   function [magnitude, angle, laplace] = xite_canny(input, lngth, h1, h2, h3, scale, thin, verbose)
%
%   The C function has the following prototype:
%   int canny(IBAND input, IBAND magnitude, IBAND angle, IBAND laplace, int lngth, int h1[], int h2[], int h3[], double scale, int thin, int verbose)
%
%   and the following manual page:
%
%   NAME
%          canny, canny_filters, canny_scale -  detect edges in image using Canny
%          operator
%
%   SYNTAX
%           #include <xite/canny.h>
%
%           int canny( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND laplace, int lngth,
%              int h1[], int h2[], int h3[], double scale,
%              int thin, int verbose );
%
%           int canny_filters( float sigma, int k, int h1[],
%              int h2[], int h3[], int verbose );
%
%           double canny_scale( double sigma, int k );
%
%   DESCRIPTION
%          canny performs the canny edge detector on the input image input.   Out-
%          put from the function are the edge magnitude, the edge angle and the
%          laplace.  The filter is specified by a gaussian function h1, the first
%          derivative h2, and the second derivative h3, all of length lngth.  The
%          outputs are scaled down with a factor scale.  If thin is true, thinning
%          of the edge is performed.  The function may be silent or verbose.
%
%          canny_filters computes the filters h1, h2, and h3.  based on sigma and
%          k.  The function returns the filter length. You may try sigma=1.0 ..
%          7.0 and k=200 .. 500.  John Canny has used 2.0 and 450 as default val-
%          ues.
%
%          canny_scale returns a default scale value based on sigma and magnifica-
%          tion k.
%
%   RESTRICTIONS
%          The current implementation has a maximum filter length of 50, and a
%          maximum input image width of 1024.
%
%          All the band parameters in canny must have pixel type unsigned byte.
%
%   RETURN VALUE
%          For canny_scale
%                 The default scale value.
%
%          For canny_filters
%                 Filter length.
%
%          For canny
%                  0 : Ok
%                  1 : Bad pixeltype
%                  2 : Filter length too large
%                  3 : Input image too small compared to filter
%                      length
%
%   REFERENCE
%          John Canny: A computational Approach to Edge Detection.  IEEE Transac-
%          tions on Pattern analysis and Machine Int.  Vol. PAMI-8 No 6 Nov. 1986
%
%   AUTHOR
%           Paul Kloschinsky
%           Modified for XITE by Otto Milvang and Svein Bøe.
%
%                                     27 Feb 2014 CANNY(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"canny"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND magnitude;
    IBAND angle;
    IBAND laplace;
    int lngth;
    int *h1;
    int *h2;
    int *h3;
    double scale;
    int thin;
    int verbose;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE magnitude_img;
    IMAGE angle_img;
    IMAGE laplace_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 8) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 8) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 3) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "lngth" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        lngth = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        lngth = ((int *) mxGetData(prhs[1]))[0];
    }
    h1 = ((int **) mxGetData(prhs[2]))[0];
    h2 = ((int **) mxGetData(prhs[3]))[0];
    h3 = ((int **) mxGetData(prhs[4]))[0];
    scale = ((double *) mxGetData(prhs[5]))[0];
    
    /* If scalar double is passed in for "thin" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        thin = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        thin = ((int *) mxGetData(prhs[6]))[0];
    }
    
    /* If scalar double is passed in for "verbose" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[7]) && mxGetM(prhs[7]) && mxGetN(prhs[7])) {
        verbose = (int) (((double *) mxGetData(prhs[7]))[0]);
    } else {
        verbose = ((int *) mxGetData(prhs[7]))[0];
    }

    /* Prepare C result variable. */
    magnitude_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    magnitude = (IBAND) (magnitude_img[1]);
    angle_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    angle = (IBAND) (angle_img[1]);
    laplace_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    laplace = (IBAND) (laplace_img[1]);
    
    /* Execute C routine. */
    return_value = canny(input, magnitude, angle, laplace, lngth, h1, h2, h3, scale, thin, verbose);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C canny.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(magnitude_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        convert_status =
    	biff2matlab(angle_img, NULL, 1, &tmp_plhs[1], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 1) plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        convert_status =
    	biff2matlab(laplace_img, NULL, 1, &tmp_plhs[2], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 2) plhs[2] = tmp_plhs[2];
    }

    PopMessage();

    return;
}
