
/*C*

________________________________________________________________

        xite_canny_scale.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/canny.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* double canny_scale(double sig, int k) */

/*M*
function [return_value] = xite_canny_scale(sig, k)
%xite_canny_scale Detect edges in image using Canny operator.
%   Calls the XITE C function canny_scale.
%   function [return_value] = xite_canny_scale(sig, k)
%
%   The C function has the following prototype:
%   double canny_scale(double sig, int k)
%
%   and the following manual page:
%
%   NAME
%          canny, canny_filters, canny_scale -  detect edges in image using Canny
%          operator
%
%   SYNTAX
%           #include <xite/canny.h>
%
%           int canny( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND laplace, int lngth,
%              int h1[], int h2[], int h3[], double scale,
%              int thin, int verbose );
%
%           int canny_filters( float sigma, int k, int h1[],
%              int h2[], int h3[], int verbose );
%
%           double canny_scale( double sigma, int k );
%
%   DESCRIPTION
%          canny performs the canny edge detector on the input image input.   Out-
%          put from the function are the edge magnitude, the edge angle and the
%          laplace.  The filter is specified by a gaussian function h1, the first
%          derivative h2, and the second derivative h3, all of length lngth.  The
%          outputs are scaled down with a factor scale.  If thin is true, thinning
%          of the edge is performed.  The function may be silent or verbose.
%
%          canny_filters computes the filters h1, h2, and h3.  based on sigma and
%          k.  The function returns the filter length. You may try sigma=1.0 ..
%          7.0 and k=200 .. 500.  John Canny has used 2.0 and 450 as default val-
%          ues.
%
%          canny_scale returns a default scale value based on sigma and magnifica-
%          tion k.
%
%   RESTRICTIONS
%          The current implementation has a maximum filter length of 50, and a
%          maximum input image width of 1024.
%
%          All the band parameters in canny must have pixel type unsigned byte.
%
%   RETURN VALUE
%          For canny_scale
%                 The default scale value.
%
%          For canny_filters
%                 Filter length.
%
%          For canny
%                  0 : Ok
%                  1 : Bad pixeltype
%                  2 : Filter length too large
%                  3 : Input image too small compared to filter
%                      length
%
%   REFERENCE
%          John Canny: A computational Approach to Edge Detection.  IEEE Transac-
%          tions on Pattern analysis and Machine Int.  Vol. PAMI-8 No 6 Nov. 1986
%
%   AUTHOR
%           Paul Kloschinsky
%           Modified for XITE by Otto Milvang and Svein Bøe.
%
%                                     27 Feb 2014 CANNY(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"canny_scale"};
    double return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    double sig;
    int k;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    sig = ((double *) mxGetData(prhs[0]))[0];
    
    /* If scalar double is passed in for "k" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        k = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        k = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* Execute C routine. */
    return_value = canny_scale(sig, k);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
    ((double *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
