
/*C*

________________________________________________________________

        xite_classifyNearest.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/classification.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int classifyNearest(IMAGE input, IBAND oldres, IBAND newres, float *means[], int k, int metric, int *changes) */

/*M*
function [newres, changes] = xite_classifyNearest(input, oldres, means, k, metric)
%xite_classifyNearest Classify to nearest class.
%   Calls the XITE C function classifyNearest.
%   function [newres, changes] = xite_classifyNearest(input, oldres, means, k, metric)
%
%   The C function has the following prototype:
%   int classifyNearest(IMAGE input, IBAND oldres, IBAND newres, float *means[], int k, int metric, int *changes)
%
%   and the following manual page:
%
%   NAME
%          classifyNearest - classify to nearest class
%
%   SYNTAX
%           #include <xite/classification.h>
%
%           int classifyNearest( IMAGE input, IBAND oldres,
%              IBAND newres, float* means[], int k,
%              int metric, int* changes );
%
%   DESCRIPTION
%          Classifies the input image to the nearest of the specified class means.
%
%          input may be multi band image.   oldres is previous classification
%          result.   newres is the classification result.  means refers to the
%          class mean vectors.  k is the number of classes.  metric must be one of
%             1 => Euclidian distance
%             2 => City block distance
%             3 => Chess distance
%
%          changes is the number of changes from previous classification.
%
%          Identical arguments may be given for oldres and newres.
%
%   RESTRICTIONS
%          input, oldres and newres must have pixel type unsigned byte.  means
%          must be an array of pointers to arrays of floats, indexable on the
%          interval
%           means[0..k-1][1..Inbands(input)].
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type in input image
%           2 => bad pixel type in oldres
%           3 => bad pixel type in newres
%           4 => bad k value (less than 2)
%           5 => unknown metric
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO.
%
%   ID
%          $Id: kMeansCluster.c,v 1.27 2014/01/31 12:53:46 svein Exp $
%
%                                     27 Feb 2014 CLASSIFYNEAREST(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"classifyNearest"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IMAGE input;
    IBAND oldres;
    IBAND newres;
    float **means;
    int k;
    int metric;
    int changes;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE oldres_img;
    IMAGE newres_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 5) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 5) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    if (mxIsStruct(prhs[0])) {
      input = Init_image(mxGetNumberOfFields(prhs[0]), "");
    } else {
      input = Init_image(1, "");
    }
    convert_status = matlab2biff(input, NULL, 1,
                                 &prhs[0], NULL, NULL);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input[1]));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input[1]));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input[1]));
    oldres_img = Init_image(1, "");
    
    convert_status = matlab2biff(oldres_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    oldres = (IBAND) (oldres_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) oldres));
    output_ysize = MAX(output_ysize, Iysize((IBAND) oldres));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) oldres));
    means = ((float ***) mxGetData(prhs[2]))[0];
    
    /* If scalar double is passed in for "k" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        k = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        k = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "metric" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        metric = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        metric = ((int *) mxGetData(prhs[4]))[0];
    }

    /* Prepare C result variable. */
    newres_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    newres = (IBAND) (newres_img[1]);
        
    /* Execute C routine. */
    return_value = classifyNearest(input, oldres, newres, means, k, metric, &changes);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C classifyNearest.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(newres_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        tmp_plhs[1] =
            mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        ((int *) mxGetData(tmp_plhs[1]))[0] = changes;
        plhs[1] = tmp_plhs[1];
    }

    PopMessage();

    return;
}
