
/*C*

________________________________________________________________

        xite_combine.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/combine.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int combine(IBAND b1, IBAND b2, IBAND b3, int expr1, int expr2, int expr3, int expr4, char *relop) */

/*M*
function [b3] = xite_combine(b1, b2, expr1, expr2, expr3, expr4, relop)
%xite_combine Combine two bands.
%   Calls the XITE C function combine.
%   function [b3] = xite_combine(b1, b2, expr1, expr2, expr3, expr4, relop)
%
%   The C function has the following prototype:
%   int combine(IBAND b1, IBAND b2, IBAND b3, int expr1, int expr2, int expr3, int expr4, char *relop)
%
%   and the following manual page:
%
%   NAME
%          combine - combine two bands
%
%   SYNTAX
%           #include <xite/combine.h>
%
%           int combine( IBAND b1, IBAND b2, IBAND b3,
%              int expr1, int expr2, int expr3, int expr4,
%              char* relop );
%
%   DESCRIPTION
%          Combines two bands into a third as follows:
%
%           FOR (every pixel) DO
%             IF (expr1 relop expr2)
%             THEN b3 = expr3
%             ELSE b3 = expr4;
%
%          -where:
%
%           b1 - first band to combine
%           b2 - second band to combine
%           b3 - output band
%           expr1, expr2, expr3, expr4 can have values:
%                -1 for b1[y][x]
%                -2 for b2[y][x]
%                0..255 for 0..255
%           relop - one of ("EQ", "NE", "LT", "LE", "GT", "GE")
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel types
%           2 => bad sizes
%           3 => unknown relop
%
%   AUTHOR
%          Tor L|nnestad, BLAB, ifi, UiO.
%
%   ID
%          $Id: combine.c,v 1.26 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 COMBINE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"combine"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b1;
    IBAND b2;
    IBAND b3;
    int expr1;
    int expr2;
    int expr3;
    int expr4;
    char *relop;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b1_img;
    IMAGE b2_img;
    IMAGE b3_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    b1_img = Init_image(1, "");
    
    convert_status = matlab2biff(b1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    b1 = (IBAND) (b1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b1));
    b2_img = Init_image(1, "");
    
    convert_status = matlab2biff(b2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    b2 = (IBAND) (b2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b2));
    
    /* If scalar double is passed in for "expr1" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        expr1 = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        expr1 = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "expr2" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        expr2 = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        expr2 = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "expr3" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        expr3 = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        expr3 = ((int *) mxGetData(prhs[4]))[0];
    }
    
    /* If scalar double is passed in for "expr4" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        expr4 = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        expr4 = ((int *) mxGetData(prhs[5]))[0];
    }
    relop = ((char **) mxGetData(prhs[6]))[0];

    /* Prepare C result variable. */
    b3_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b3 = (IBAND) (b3_img[1]);
    
    /* Execute C routine. */
    return_value = combine(b1, b2, b3, expr1, expr2, expr3, expr4, relop);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C combine.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(b3_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
