
/*C*

________________________________________________________________

        xite_crossing.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/convolve.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int crossing(IBAND iband, IBAND oband, int t) */

/*M*
function [oband] = xite_crossing(iband, t)
%xite_crossing Find zero crossing or any other crossing.
%   Calls the XITE C function crossing.
%   function [oband] = xite_crossing(iband, t)
%
%   The C function has the following prototype:
%   int crossing(IBAND iband, IBAND oband, int t)
%
%   and the following manual page:
%
%   NAME
%          crossing - find zero crossing or any other crossing
%
%   SYNTAX
%           #include <xite/convolve.h>
%
%           int crossing( IBAND iband, IBAND oband, int t );
%
%   DESCRIPTION
%          Can be used to find zero crossing of second derivative, laplace. Can
%          also be used to determine any other kind of crossing. Pixels below or
%          equal to t are set if the pixel to the right or below is above t, pix-
%          els above t are set if the pixel to the right or below is below or
%          equal to t.  Pixels that are "set" are set to the maximum absolute dif-
%          ference of the two neighbours, to indicate the strength of the edge.
%
%           IF (crossing t)
%             out(x,y) = MAX(ABS(in(x,y)-in(x+1,y)),
%                            ABS(in(x,y)-in(x,y+1)))
%           ELSE
%             out(x,y) = 0
%
%          Identical iband and oband arguments are allowed.
%
%   RESTRICTIONS
%          iband and oband must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type input band
%           2 => bad pixel type output band
%
%   SEE ALSO
%          crossing(1), contour(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: crossing.c,v 1.26 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 CROSSING(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"crossing"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND iband;
    IBAND oband;
    int t;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE iband_img;
    IMAGE oband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    iband_img = Init_image(1, "");
    
    convert_status = matlab2biff(iband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    iband = (IBAND) (iband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) iband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) iband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) iband));
    
    /* If scalar double is passed in for "t" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        t = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        t = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    oband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    oband = (IBAND) (oband_img[1]);
    
    /* Execute C routine. */
    return_value = crossing(iband, oband, t);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C crossing.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(oband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
