
/*C*

________________________________________________________________

        xite_lapZeroCross.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/convolve.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus lapZeroCross(IBAND iband, IBAND oband, int a, int b, int c, double scale, double offset, int v) */

/*M*
function [oband] = xite_lapZeroCross(iband, a, b, c, scale, offset, v)
%xite_lapZeroCross Find zero crossing of lapacian.
%   Calls the XITE C function lapZeroCross.
%   function [oband] = xite_lapZeroCross(iband, a, b, c, scale, offset, v)
%
%   The C function has the following prototype:
%   BiffStatus lapZeroCross(IBAND iband, IBAND oband, int a, int b, int c, double scale, double offset, int v)
%
%   and the following manual page:
%
%   NAME
%          lapZeroCross - find zero crossing of lapacian
%
%   SYNTAX
%           #include <xite/convolve.h>
%
%           BiffStatus lapZeroCross( IBAND iband,
%              IBAND oband, int a, int b, int c,
%              double scale, double offset, int v );
%
%   DESCRIPTION
%          lapZeroCross finds the zero crossing of the laplacian of the input
%          band. The routine starts by calling the laplace operator, followed by
%          calling the crossing routine.
%
%          The output is zero where the laplacian has no zero crossing, and pro-
%          portional to the edge strength where the laplacian has a zero crossing.
%
%          The arguments a, b, c, scale, offset and v are as described for the
%          laplace operator. Typical values are a = 0, b = -1, c = 4, scale = 1.0,
%          offset = 127.0.
%
%          The whole algorithm is:
%
%           <Calculate laplacian>;
%           FOR <every pixel (x,y)>:
%           IF <in(x,y) have zero crossing>
%             out(x,y) = MAX(ABS(in(x,y)-in(x+1,y)), \
%                            ABS(in(x,y)-in(x,y+1)))
%           ELSE
%             out(x,y) = 0
%
%   RESTRICTIONS
%          iband and oband must have pixel type unsigned byte.
%
%   SEE ALSO
%          lapZeroCross(1), laplace(3), gradient(3), grad(3), roberts(3),
%          sobel(3), prewitt(3), contour(3), crossing(3)
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type input band
%           2 => bad pixel type output band
%           3 => some error in crossing
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: lapZeroCross.c,v 1.25 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 LAPZEROCROSS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"lapZeroCross"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND iband;
    IBAND oband;
    int a;
    int b;
    int c;
    double scale;
    double offset;
    int v;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE iband_img;
    IMAGE oband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    iband_img = Init_image(1, "");
    
    convert_status = matlab2biff(iband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    iband = (IBAND) (iband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) iband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) iband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) iband));
    
    /* If scalar double is passed in for "a" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        a = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        a = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "b" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        b = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        b = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "c" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        c = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        c = ((int *) mxGetData(prhs[3]))[0];
    }
    scale = ((double *) mxGetData(prhs[4]))[0];
    offset = ((double *) mxGetData(prhs[5]))[0];
    
    /* If scalar double is passed in for "v" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        v = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        v = ((int *) mxGetData(prhs[6]))[0];
    }

    /* Prepare C result variable. */
    oband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    oband = (IBAND) (oband_img[1]);
    
    /* Execute C routine. */
    return_value = lapZeroCross(iband, oband, a, b, c, scale, offset, v);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C lapZeroCross.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(oband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
