
/*C*

________________________________________________________________

        xite_laplace.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/convolve.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void laplace(IBAND input, IBAND output, int a, int b, int c, double scale, double offset, int verbose) */

/*M*
function [output] = xite_laplace(input, a, b, c, scale, offset, verbose)
%xite_laplace Edge and line detection.
%   Calls the XITE C function laplace.
%   function [output] = xite_laplace(input, a, b, c, scale, offset, verbose)
%
%   The C function has the following prototype:
%   void laplace(IBAND input, IBAND output, int a, int b, int c, double scale, double offset, int verbose)
%
%   and the following manual page:
%
%   NAME
%          laplace - edge and line detection
%
%   SYNTAX
%           #include <xite/convolve.h>
%
%           void laplace( IBAND input, IBAND output, int a,
%              int b, int c, double scale, double offset,
%              int verbose );
%
%   DESCRIPTION
%          laplace performs the convolution between a band input and a convolution
%          kernel conv.
%
%                  a b a
%                  b c b
%                  a b a
%
%          The result of the convolution is scaled
%             pix = (conv. result) * scale + offset;
%
%          If scale is zero an autoscaling is performed.  If verbose flag is set,
%          the number of underflows and overflows are printed. If verbose flag is
%          set and autoscale is performed the scale and offset factors are dis-
%          played
%
%          conv3x3_overflow and conv3x3_underflow are global variabels and may be
%          declared as
%           extern int conv3x3_overflow, conv3x3_underflow
%
%   SEE ALSO
%          laplace(1), conv3x3(3), lapZeroCross(3)
%
%   AUTHOR
%          Otto Milvang
%
%   EXAMPLES
%   ID
%          $Id: laplace.c,v 1.26 1998/02/26 10:37:13 svein Exp $
%
%                                     27 Feb 2014 LAPLACE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"laplace"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    int a;
    int b;
    int c;
    double scale;
    double offset;
    int verbose;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "a" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        a = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        a = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "b" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        b = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        b = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "c" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        c = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        c = ((int *) mxGetData(prhs[3]))[0];
    }
    scale = ((double *) mxGetData(prhs[4]))[0];
    offset = ((double *) mxGetData(prhs[5]))[0];
    
    /* If scalar double is passed in for "verbose" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        verbose = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        verbose = ((int *) mxGetData(prhs[6]))[0];
    }

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    laplace(input, output, a, b, c, scale, offset, verbose);

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
