
/*C*

________________________________________________________________

        xite_sobel.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/convolve.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int sobel(IBAND input, IBAND magnitude, IBAND angle, IBAND pseudo, int verbose) */

/*M*
function [magnitude, angle, pseudo] = xite_sobel(input, verbose)
%xite_sobel Gradient operator for edge detection.
%   Calls the XITE C function sobel.
%   function [magnitude, angle, pseudo] = xite_sobel(input, verbose)
%
%   The C function has the following prototype:
%   int sobel(IBAND input, IBAND magnitude, IBAND angle, IBAND pseudo, int verbose)
%
%   and the following manual page:
%
%   NAME
%          gradient, grad, prewitt, roberts, sobel - gradient operator for edge
%          detection
%
%   SYNTAX
%           #include <xite/convolve.h>
%
%           int gradient( IBAND input, ISS_BAND convx,
%              ISS_BAND convy, double xscale,
%              double yscale, IBAND magnitude, IBAND angle,
%              IBAND pseudo, int verbose );
%
%           int sobel( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND pseudo, int verbose );
%
%           int prewitt( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND pseudo, int verbose );
%
%           int roberts( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND pseudo, int verbose );
%
%           int grad( IBAND input, IBAND magnitude,
%              IBAND angle, IBAND pseudo, int verbose );
%
%   DESCRIPTION
%          gradient performs a convolution of input and convx, scaled by xscale
%          (or 0.0 for autoscaling), giving gx, and a convolution of input and
%          convy, scaled by yscale ((or 0.0 for autoscaling), giving gy.   gx and
%          gy are combined into magnitude, angle and a pseudo image.  If verbose
%          is TRUE some messages are witten with Info().
%
%           magnitude = |gx| + |gy|
%           angle =  atan(gy/gx);
%           pseudo = combination of magnitude and angle.
%
%          sobel calculates gx and gy as
%
%                  -1 0 1 -1 -2 -1
%           gx =   -2 0 2 gy =   0 0 0
%                  -1 0 1 1 2 1
%
%          prewitt calculates gx and gy as
%
%                  -1 0 1 -1 -1 -1
%           gx =   -1 0 1 gy =   0 0 0
%                  -1 0 1 1 1 1
%
%          roberts calculates gx and gy as
%
%                  1 0 0 1
%           gx =               gy =
%                  0 -1 -1 0
%
%          grad calculates gx and gy as
%
%                 -1 1 -1 0
%           gx =               gy =
%                  0 0 1 0
%
%   PSEUDO
%          The pseudo comination of magnitude and pseudo:
%           mag level = magnitude / 32;
%            Level First Length
%                0 0 4
%                1 4 12
%                2 16 20
%                3 36 28
%                4 64 36
%                5 100 44
%                6 144 52
%                7 196 60
%           pseudo = first[mag_level] + angle/256* length[mag_level];
%
%          The pseudo-image may be displayed with a "gradient" color-table.
%
%   SEE ALSO
%          convolve(3), conv3x3(3), ascii2biff(3)
%
%   AUTHOR
%          Otto Milvang
%
%   ID
%          $Id: gradient_op.c,v 1.38 1998/03/03 14:32:05 svein Exp $
%
%                                     27 Feb 2014 GRADIENT(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"sobel"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND magnitude;
    IBAND angle;
    IBAND pseudo;
    int verbose;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE magnitude_img;
    IMAGE angle_img;
    IMAGE pseudo_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 3) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "verbose" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        verbose = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        verbose = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    magnitude_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    magnitude = (IBAND) (magnitude_img[1]);
    angle_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    angle = (IBAND) (angle_img[1]);
    pseudo_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    pseudo = (IBAND) (pseudo_img[1]);
    
    /* Execute C routine. */
    return_value = sobel(input, magnitude, angle, pseudo, verbose);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C sobel.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(magnitude_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        convert_status =
    	biff2matlab(angle_img, NULL, 1, &tmp_plhs[1], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 1) plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        convert_status =
    	biff2matlab(pseudo_img, NULL, 1, &tmp_plhs[2], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 2) plhs[2] = tmp_plhs[2];
    }

    PopMessage();

    return;
}
