
/*C*

________________________________________________________________

        xite_overlap.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/copy.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int overlap(IBAND ib1, IBAND ib2, IBAND *ob1, IBAND *ob2) */

/*M*
function [ob1, ob2] = xite_overlap(ib1, ib2)
%xite_overlap Calculate overlapping area of two bands.
%   Calls the XITE C function overlap.
%   function [ob1, ob2] = xite_overlap(ib1, ib2)
%
%   The C function has the following prototype:
%   int overlap(IBAND ib1, IBAND ib2, IBAND *ob1, IBAND *ob2)
%
%   and the following manual page:
%
%   NAME
%          overlap - calculate overlapping area of two bands
%
%   SYNTAX
%           #include <xite/copy.h>
%
%           int overlap( IBAND ib1, IBAND ib2, IBAND* ob1,
%              IBAND* ob2 );
%
%   DESCRIPTION
%          Input bands ib1 and ib2 are placed in a common coordinate system
%          according to their (xstart, ystart) attributes. The overlapping area is
%          calculated, and two subbands ob1 and ob2 are created and returned,
%          pointing out the overlapping area in each band.  ob1 will thus be a
%          subband of ib1, and ob2 will be a subband of ib2.
%
%   RESTRICTIONS
%          Only accepts unsigned byte pixeltype.
%
%   RETURN VALUE
%           0 => ok,
%           1 => bad pixel type input band 1
%           2 => bad pixel type input band 2
%           3 => no overlap
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%                                     27 Feb 2014 OVERLAP(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"overlap"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND ib1;
    IBAND ib2;
    IBAND ob1;
    IBAND ob2;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE ib1_img;
    IMAGE ib2_img;
    IMAGE ob1_img;
    IMAGE ob2_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    ib1_img = Init_image(1, "");
    
    convert_status = matlab2biff(ib1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    ib1 = (IBAND) (ib1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) ib1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) ib1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) ib1));
    ib2_img = Init_image(1, "");
    
    convert_status = matlab2biff(ib2_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    ib2 = (IBAND) (ib2_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) ib2));
    output_ysize = MAX(output_ysize, Iysize((IBAND) ib2));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) ib2));

    /* Prepare C result variable. */
            
    /* Execute C routine. */
    return_value = overlap(ib1, ib2, &ob1, &ob2);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C overlap.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        ob1_img = Init_image(1, "");
        ob1_img[1] = (IBAND) ob1;
        convert_status =
    	biff2matlab(ob1_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        ob2_img = Init_image(1, "");
        ob2_img[1] = (IBAND) ob2;
        convert_status =
    	biff2matlab(ob2_img, NULL, 1, &tmp_plhs[1], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 1) plhs[1] = tmp_plhs[1];
    }

    PopMessage();

    return;
}
