
/*C*

________________________________________________________________

        xite_fht1d.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/fht.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void fht1d(float *data, float *sinTab, float *cosTab, float *work, int *perm, int size, int power, int forw) */

/*M*
function [data] = xite_fht1d(data, sinTab, cosTab, work, perm, size, power, forw)
%xite_fht1d 1 dimensional fast Hartley transform.
%   Calls the XITE C function fht1d.
%   function [data] = xite_fht1d(data, sinTab, cosTab, work, perm, size, power, forw)
%
%   The C function has the following prototype:
%   void fht1d(float *data, float *sinTab, float *cosTab, float *work, int *perm, int size, int power, int forw)
%
%   and the following manual page:
%
%   NAME
%          fht1d - 1 dimensional fast Hartley transform
%
%   SYNTAX
%           #include <xite/fht.h>
%
%           void fht1d( float* data, float* sinTab,
%              float* cosTab, float* work, int* perm,
%              int size, int power, int forw );
%
%   DESCRIPTION
%          fht1d performs 1 dimensional fast Hartley transform, forward transform
%          if forw is true, inverse otherwise.  After transformation, the result
%          is placed in data.
%
%   INPUT ARGUMENTS
%          data The array to be transformed in place.
%
%          sinTab Sine table; 0..2PI with size elems.
%
%          cosTab Cosine table; 0..2PI with size elems.
%
%          work Work space: float array with size elems.
%
%          perm Permutation table with size elems.
%
%          size The size of the arrays.
%
%          power The power of two (log2 of size).
%
%          forw TRUE => forward transform, FALSE => inverse.
%
%            The tables sinTab, cosTab and perm must be initialized before calling
%            this routione. Initialization can be done with the fhtInitTables rou-
%            tine.
%
%   RESTRICTIONS
%          For efficiency reasons, input arguments are not tested.
%
%   SEE ALSO
%          fht2d(3), haar1d(3), fhtInitTables(3)
%
%   RETURN VALUE
%          none
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: fht2d.c,v 1.33 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 FHT1D(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"fht1d"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    float *data;
    float *sinTab;
    float *cosTab;
    float *work;
    int *perm;
    int size;
    int power;
    int forw;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 8) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 8) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    data = ((float **) mxGetData(prhs[0]))[0];
    sinTab = ((float **) mxGetData(prhs[1]))[0];
    cosTab = ((float **) mxGetData(prhs[2]))[0];
    work = ((float **) mxGetData(prhs[3]))[0];
    perm = ((int **) mxGetData(prhs[4]))[0];
    
    /* If scalar double is passed in for "size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        size = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        size = ((int *) mxGetData(prhs[5]))[0];
    }
    
    /* If scalar double is passed in for "power" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        power = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        power = ((int *) mxGetData(prhs[6]))[0];
    }
    
    /* If scalar double is passed in for "forw" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[7]) && mxGetM(prhs[7]) && mxGetN(prhs[7])) {
        forw = (int) (((double *) mxGetData(prhs[7]))[0]);
    } else {
        forw = ((int *) mxGetData(prhs[7]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    fht1d(data, sinTab, cosTab, work, perm, size, power, forw);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(size, 1, mxSINGLE_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), data, (size) * sizeof(float));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
