
/*C*

________________________________________________________________

        xite_fht2d.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/fht.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus fht2d(IR_BAND inband, IR_BAND outband, int forward) */

/*M*
function [outband] = xite_fht2d(inband, forward)
%xite_fht2d 2 dimensional fast Hartley transform.
%   Calls the XITE C function fht2d.
%   function [outband] = xite_fht2d(inband, forward)
%
%   The C function has the following prototype:
%   BiffStatus fht2d(IR_BAND inband, IR_BAND outband, int forward)
%
%   and the following manual page:
%
%   NAME
%          fht2d - 2 dimensional fast Hartley transform
%
%   SYNTAX
%           #include <xite/fht.h>
%
%           BiffStatus fht2d( IR_BAND inband,
%              IR_BAND outband, int forward );
%
%   DESCRIPTION
%          fht2d performs 2 dimensional fast Hartley transform, forward transform
%          if forward is true, inverse transform otherwise.
%
%          The forward and inverse discrete Hartley transforms are given respec-
%          tively as:
%
%                     M N
%           H(u,v) = Sum Sum f(x,y) [cas(2*Pi(u*x/M + v*y/N))]
%                    x=0 y=0
%
%                     M N
%           f(x,y) = Sum Sum H(u,v) [cas(2*Pi(u*x/M + v*y/N))]
%                    x=0 y=0
%
%          where
%           cas(x) = cos(x) + sin(x)
%
%          The relationships between the Fourier and Hartley transforms are
%          described in the documentation of ht2ft(3) and ft2ht(3).
%
%          For a description of how to use Hartley transforms to accomplish convo-
%          lution, refer to fht2d(1).
%
%   RESTRICTIONS
%          Only real pixels are accepted as input and output.  Input and output
%          bands should be of equal size and be some power of two.
%
%   REFERENCES
%          [[1]]    Mark A. O Neill "Faster than Fast Fourier", BYTE, pp 293-300,
%                 April 1988
%
%          [[2]]    R.  N.  Bracewell "The Hartley Transform", Oxford Univ. Press
%                 1986, ISBN 0-19-503969-6
%
%   SEE ALSO
%          fht2d(1), fft2d(3), haar2d(3), fht1d(3), fhtInitTables(3), fhtPower(3),
%          fhtPhase(3), ht2ft(3), ft2ht(3)
%
%   RETURN VALUE
%           0 => ok
%           1 => bad input pixel type
%           2 => bad output pixel type
%           3 => band size is not power of two
%           4 => bands are not of equal size
%           5 => error in copyBand()
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   REVISED
%          Svein Bøe, BLAB, Ifi, UiO
%
%   ID
%          $Id: fht2d.c,v 1.33 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 FHT2D(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"fht2d"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IR_BAND inband;
    IR_BAND outband;
    int forward;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IR_BAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "forward" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        forward = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        forward = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IR_BAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = fht2d(inband, outband, forward);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C fht2d.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
