
/*C*

________________________________________________________________

        xite_fhtInitTables.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/fht.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int fhtInitTables(float *sinTab, float *cosTab, int *perm, int size) */

/*M*
function [sinTab, cosTab, perm] = xite_fhtInitTables(size)
%xite_fhtInitTables Initialize tables for fast Hartley transform.
%   Calls the XITE C function fhtInitTables.
%   function [sinTab, cosTab, perm] = xite_fhtInitTables(size)
%
%   The C function has the following prototype:
%   int fhtInitTables(float *sinTab, float *cosTab, int *perm, int size)
%
%   and the following manual page:
%
%   NAME
%          fhtInitTables - initialize tables for fast Hartley transform
%
%   SYNTAX
%           #include <xite/fht.h>
%
%           int fhtInitTables( float* sinTab,
%              float* cosTab, int* perm, int size );
%
%   DESCRIPTION
%          Initializes the three tables sinTab, cosTab and perm.  The tables must
%          be allocated by the caller, and have size elements.
%
%   RESTRICTIONS
%          Size must be >= 2 and power of two
%
%   SEE ALSO
%          fht1d(3), fht2d(3)
%
%   RETURN VALUE
%           0 => ok
%           1 => bad size
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   REVISED
%          Svein Bøe, BLAB, Ifi, UiO
%
%   ID
%          $Id: fht2d.c,v 1.33 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 FHTINITTABLES(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"fhtInitTables"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    float *sinTab;
    float *cosTab;
    int *perm;
    int size;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 3) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        size = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        size = ((int *) mxGetData(prhs[0]))[0];
    }

    /* Prepare C result variable. */
    sinTab = (float *) mxCalloc(size, sizeof(float));
    cosTab = (float *) mxCalloc(size, sizeof(float));
    perm = (int *) mxCalloc(size, sizeof(int));
    
    /* Execute C routine. */
    return_value = fhtInitTables(sinTab, cosTab, perm, size);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C fhtInitTables.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(size, 1, mxSINGLE_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), sinTab, (size) * sizeof(float));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }
    if (nlhs > 1) {
        tmp_plhs[1] =    mxCreateNumericMatrix(size, 1, mxSINGLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[1]), cosTab, (size) * sizeof(float));
        plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =    mxCreateNumericMatrix(size, 1, mxINT32_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[2]), perm, (size) * sizeof(int));
        plhs[2] = tmp_plhs[2];
    }

    PopMessage();

    return;
}
