
/*C*

________________________________________________________________

        xite_ft2ht.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/fht.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int ft2ht(IC_BAND inband, IR_BAND outband) */

/*M*
function [outband] = xite_ft2ht(inband)
%xite_ft2ht 2d fourier to hartley conversion.
%   Calls the XITE C function ft2ht.
%   function [outband] = xite_ft2ht(inband)
%
%   The C function has the following prototype:
%   int ft2ht(IC_BAND inband, IR_BAND outband)
%
%   and the following manual page:
%
%   NAME
%          ft2ht - 2d fourier to hartley conversion
%
%   SYNTAX
%           #include <xite/fht.h>
%
%           int ft2ht( IC_BAND inband, IR_BAND outband );
%
%   DESCRIPTION
%          ft2ht converts a Fourier transform to a Hartley transform.  The Hartley
%          transform is given by the Fourier transform as follows:
%
%           H(u,v) = Re{F(u,v)} - Im{F(u,v)}
%                  = H_e(u,v) + H_o(u,v)
%
%          where H_e and H_o are the even and odd parts of H, i.e.
%           H_e(u,v) = (H(u,v) + H(-u,-v))/2
%           H_o(u,v) = (H(u,v) - H(-u,-v))/2
%
%          For images with horizontal and vertical even symmetry, H_o(u,v)  equals
%          zero, so that
%           H(u,v) = F(u,v)
%
%   RESTRICTIONS
%          bands should have equal size, be quadratic, and have size some power of
%          two. Currently, only single precision float and complex values are
%          accepted.
%
%   SEE ALSO
%          ft2ht(1), fft2d(3), fht2d(3), ht2ft(3)
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type input
%           2 => bad pixel type output
%           3 => bad sizes
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: ft2ht.c,v 1.27 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 FT2HT(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"ft2ht"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IC_BAND inband;
    IR_BAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IC_BAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IR_BAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = ft2ht(inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C ft2ht.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
