
/*C*

________________________________________________________________

        xite_quadratic.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/geometry.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int quadratic(IBAND inband, IBAND outband, double *xa, double *ya, int ip, int bg) */

/*M*
function [outband] = xite_quadratic(inband, xa, ya, ip, bg)
%xite_quadratic Quadratic geometric transform.
%   Calls the XITE C function quadratic.
%   function [outband] = xite_quadratic(inband, xa, ya, ip, bg)
%
%   The C function has the following prototype:
%   int quadratic(IBAND inband, IBAND outband, double *xa, double *ya, int ip, int bg)
%
%   and the following manual page:
%
%   NAME
%          quadratic - quadratic geometric transform
%
%   SYNTAX
%           #include <xite/geometry.h>
%
%           int quadratic( IBAND inband, IBAND outband,
%              double* xa, double* ya, int ip, int bg );
%
%   DESCRIPTION
%          quadratic performs a quadratic transformation of the coordinate system
%          (x, y) to another coordinate system (u,v)
%
%          All coordinates are adjusted acording to (xstart, ystart).
%           outband(x,y) = inband(u,v)
%           u = xa[0] + xa[1] * x + xa[2] * y
%             + xa[3] * x * x + xa[4] * x * y + xa[5] * y * y
%           v = ya[0] + ya[1] * x + ya[2] * y
%             + ya[3] * x * x + ya[4] * x * y + ya[5] * y * y
%
%          The interpolation method ip is
%            0 : Nearest neighbor
%            1 : Bilinear interpolation
%                g(x) = bi(f(x), f(x+1), d)
%                bi(x, y, d) = x(1-d) + yd
%            3 ; Cubic interpolation
%                g(x) = cc(f(x-1), f(x), f(x+1), f(x+2), d)
%                cc(x, y, z, v, d) = y + d((-x+z) +
%                       d((2x-2y+z-v) + d(-x+y-z+v)))
%           The bg parameter specifies the background pixel value.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Bad input or output pixel type
%
%   REFERENCE
%          Wayne Niblack. Digital Image Processing, p 131-150
%
%   SEE ALSO
%          quadratic(1), affine(3), rotate(3), shift_band(3), transpose_band(3)
%
%   AUTHOR
%          Otto Milvang
%
%   EXAMPLES
%   ID
%          $Id: quadratic.c,v 1.31 1998/02/26 10:40:53 svein Exp $
%
%                                     27 Feb 2014 QUADRATIC(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"quadratic"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    double *xa;
    double *ya;
    int ip;
    int bg;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 5) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 5) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    xa = ((double **) mxGetData(prhs[1]))[0];
    ya = ((double **) mxGetData(prhs[2]))[0];
    
    /* If scalar double is passed in for "ip" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        ip = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        ip = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "bg" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        bg = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        bg = ((int *) mxGetData(prhs[4]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = quadratic(inband, outband, xa, ya, ip, bg);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C quadratic.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
