
/*C*

________________________________________________________________

        xite_shift_band.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/geometry.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int shift_band(IBAND in_band, IBAND out_band, int shiftx, int shifty) */

/*M*
function [out_band] = xite_shift_band(in_band, shiftx, shifty)
%xite_shift_band Set new origo on a band.
%   Calls the XITE C function shift_band.
%   function [out_band] = xite_shift_band(in_band, shiftx, shifty)
%
%   The C function has the following prototype:
%   int shift_band(IBAND in_band, IBAND out_band, int shiftx, int shifty)
%
%   and the following manual page:
%
%   NAME
%          shift_band - Set new origo on a band.
%
%   SYNTAX
%           #include <xite/geometry.h>
%
%           int shift_band( IBAND in_band, IBAND out_band,
%              int shiftx, int shifty );
%
%   DESCRIPTION
%          shift an image band. Works on all pixel types.  in_band and out_band
%          may be the same band.
%
%   RESTRICTIONS
%          in_band and outband must be of same type and size
%
%   RETURN VALUE
%           0 => ok
%           2 => Bands have different pixel type
%           3 => Bands have different size
%
%   SEE ALSO
%          shift_img(1), transpose_band(3), affine(3), quadratic(3), rotate(3)
%
%   AUTHOR
%          Otto Milvang
%
%   ID
%          $Id: shift_img.c,v 1.26 1998/02/26 10:40:55 svein Exp $
%
%                                     27 Feb 2014 SHIFT_BAND(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"shift_band"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND in_band;
    IBAND out_band;
    int shiftx;
    int shifty;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE in_band_img;
    IMAGE out_band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    in_band_img = Init_image(1, "");
    
    convert_status = matlab2biff(in_band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    in_band = (IBAND) (in_band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) in_band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) in_band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) in_band));
    
    /* If scalar double is passed in for "shiftx" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        shiftx = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        shiftx = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "shifty" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        shifty = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        shifty = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
    out_band_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    out_band = (IBAND) (out_band_img[1]);
    
    /* Execute C routine. */
    return_value = shift_band(in_band, out_band, shiftx, shifty);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C shift_band.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(out_band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
