
/*C*

________________________________________________________________

        xite_transpose_band.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/geometry.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int transpose_band(IBAND I) */

/*M*
function [I] = xite_transpose_band(I)
%xite_transpose_band Transpose a BIFF image band.
%   Calls the XITE C function transpose_band.
%   function [I] = xite_transpose_band(I)
%
%   The C function has the following prototype:
%   int transpose_band(IBAND I)
%
%   and the following manual page:
%
%   NAME
%          transpose_band - transpose a BIFF image band
%
%   SYNTAX
%           #include <xite/geometry.h>
%
%           int transpose_band( IBAND I );
%
%   DESCRIPTION
%          Transpose an image band. Works on all pixel types.
%
%   RESTRICTIONS
%          xsize == ysize
%
%   RETURN VALUE
%           0 => ok
%           2 => Bad pixel type
%           3 => Bad size, xsize <> ysize
%
%   SEE ALSO
%          transpose_img(1), shift_band(3), affine(3), quadratic(3), rotate(3)
%
%   AUTHOR
%          Otto Milvang
%
%   ID
%          $Id: transpose_img.c,v 1.25 1998/02/26 10:40:56 svein Exp $
%
%                                     27 Feb 2014 TRANSPOSE_BAND(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"transpose_band"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND I;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE I_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    I_img = Init_image(1, "");
    
    convert_status = matlab2biff(I_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    I = (IBAND) (I_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) I));
    output_ysize = MAX(output_ysize, Iysize((IBAND) I));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) I));

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = transpose_band(I);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C transpose_band.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(I_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
