
/*C*

________________________________________________________________

        xite_warp.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/geometry.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int warp(IMAGE inimg, IMAGE outimg, IBAND points, int npoint, int ncoeffs, int ip, int bg, int rss, int qr) */

/*M*
function [outimg] = xite_warp(inimg, points, npoint, ncoeffs, ip, bg, rss, qr)
%xite_warp Polynomial control point mapping.
%   Calls the XITE C function warp.
%   function [outimg] = xite_warp(inimg, points, npoint, ncoeffs, ip, bg, rss, qr)
%
%   The C function has the following prototype:
%   int warp(IMAGE inimg, IMAGE outimg, IBAND points, int npoint, int ncoeffs, int ip, int bg, int rss, int qr)
%
%   and the following manual page:
%
%   NAME
%          warp, warp_resample, oiWarpMapping - polynomial control point mapping
%
%   SYNTAX
%           #include <xite/geometry.h>
%
%           int warp( IMAGE inimg, IMAGE outimg, IBAND points,
%              int npoints, int ncoeffs, int ip, int bg,
%              int rss, int qr );
%
%           int warp_resample( IMAGE inimg, IMAGE outimg,
%              double *xxa, double *yya, int ncoeffs, int ip,
%              int bg );
%
%           int oiWarpMapping( xiteMatrix *M, xiteMatrix *Mp,
%              xiteMatrix *Q, xiteMatrix *R, xiteMatrix *K,
%              xiteMatrix *L, IBAND points, int npoints,
%              int ncoeffs );
%
%   DESCRIPTION
%          warp performs an output to input coordinate mapping (k,l) = M(i,j).
%          Given an input image I(k, l), calculate O(i,j).
%
%          points must supply the coordinates on which the mapping is based.  It
%          must be an npoints by 4 image band with real pixel type. Each row is a
%          set of i, j, k and l.  i and j are coordinates in the reference image.
%          k and l are coordinates in the image to be transformed.
%
%          npoints must be at least as large as ncoeffs.
%
%          ncoeffs is the number of polynomial coefficients used in the coordinate
%          mapping.
%
%          ip determines the degree of the gray level interpolation polynomial
%          used when determining pixel values for the warp function. Legal values
%          are 0 for nearest neighbour, 1 for bilinear interpolation, and 3 for
%          cubic interpolation.  Bilinear interpolation:
%             g(x) = bi(f(x), f(x+1), d)
%             bi(x, y, d) = x(1-d) + yd
%
%          Cubic interpolation:
%             g(x) = cc(f(x-1), f(x), f(x+1), f(x+2), d)
%             cc(x, y, z, v, d) = y + d((-x+z) +
%                                 d((2x-2y+z-v) + d(-x+y-z+v)))
%
%          bg is used as value for the background pixels.
%
%          rss determines whether or not to print all control points and root mean
%          square errors (Root Sum of Squares),
%           dk = k - Q(i,j)
%           dl = l - R(i,j)
%           r(m) = sqrt(dk*dk + dl*dl)
%
%          qr determines whether or not to print the polynomial coefficients used
%          by the warp function when mapping coordinates. The output is on the
%          form (with three polynomial coefficients):
%
%             # Q =
%                -230.173 1.46376 0.0805173
%             # R =
%               -63.7828 0.0869576 1.36232
%
%          The xstart and ystart values of the images are honored.
%
%          warp_resample is used by warp to resample the input image after the
%          coordinate mapping polynomials have been computed.  xxa and yya are
%          arrays of these polynomial coefficients, maximum six of each, at least
%          as many as ncoeffs.
%
%          warp_resample calls affine or quadratic, depending on the value of nco-
%          effs.
%
%          oiWarpMapping computes the actual coordinate mapping polynomials. It is
%          used by warp.  Memory is allocated for the returning matrices, and must
%          be freed by the caller, with calls to MFree.  M is the ( npoints x nco-
%          effs)  output-to-input coordinate mapping matrix in (k,l) = M(i,j), Mp
%          is the pseudoinverse of M, Q and R are ( ncoeffs x 1) vectors given by
%          k = Q(i,j) and l = R(i,j) respectively, K and L are ( npoints x 1) vec-
%          tors containing the k and l coordinates from points.
%
%   RESTRICTIONS
%          All the bands of inimg and outimg must have pixel type unsigned byte.
%          The pixel type of points must be real.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Error
%
%   SEE ALSO
%          warp(1), xwarp(1), affine(3), quadratic(3)
%
%   REFERENCE
%          Wayne Niblack. Digital Image Processing, p 131-150
%
%   AUTHOR
%          Svein Bøe, based on program warp and xwarp.
%
%   ID
%          $Id: warp.c,v 1.40 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 WARP(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"warp"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IMAGE inimg;
    IMAGE outimg;
    IBAND points;
    int npoint;
    int ncoeffs;
    int ip;
    int bg;
    int rss;
    int qr;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE points_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 8) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 8) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    if (mxIsStruct(prhs[0])) {
      inimg = Init_image(mxGetNumberOfFields(prhs[0]), "");
    } else {
      inimg = Init_image(1, "");
    }
    convert_status = matlab2biff(inimg, NULL, 1,
                                 &prhs[0], NULL, NULL);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inimg[1]));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inimg[1]));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inimg[1]));
    points_img = Init_image(1, "");
    
    convert_status = matlab2biff(points_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    points = (IBAND) (points_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) points));
    output_ysize = MAX(output_ysize, Iysize((IBAND) points));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) points));
    
    /* If scalar double is passed in for "npoint" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        npoint = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        npoint = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "ncoeffs" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        ncoeffs = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        ncoeffs = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "ip" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        ip = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        ip = ((int *) mxGetData(prhs[4]))[0];
    }
    
    /* If scalar double is passed in for "bg" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        bg = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        bg = ((int *) mxGetData(prhs[5]))[0];
    }
    
    /* If scalar double is passed in for "rss" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        rss = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        rss = ((int *) mxGetData(prhs[6]))[0];
    }
    
    /* If scalar double is passed in for "qr" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[7]) && mxGetM(prhs[7]) && mxGetN(prhs[7])) {
        qr = (int) (((double *) mxGetData(prhs[7]))[0]);
    } else {
        qr = ((int *) mxGetData(prhs[7]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = warp(inimg, outimg, points, npoint, ncoeffs, ip, bg, rss, qr);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C warp.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status = 
            biff2matlab((IMAGE) outimg, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
