
/*C*

________________________________________________________________

        xite_haarTexture.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/haar.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int haarTexture(IR_BAND input, IR_BAND output, int xp, int yp) */

/*M*
function [output] = xite_haarTexture(input, xp, yp)
%xite_haarTexture Texture extraction from 2D Haar transform.
%   Calls the XITE C function haarTexture.
%   function [output] = xite_haarTexture(input, xp, yp)
%
%   The C function has the following prototype:
%   int haarTexture(IR_BAND input, IR_BAND output, int xp, int yp)
%
%   and the following manual page:
%
%   NAME
%          haarTexture - texture extraction from 2D Haar transform
%
%   SYNTAX
%           #include <xite/haar.h>
%
%           int haarTexture( IR_BAND input, IR_BAND output,
%              int xp, int yp );
%
%   DESCRIPTION
%          Extracts texture information from the input band, returns it through
%          the output band. The input is assumed to be the result of a forward 2D
%          haar transform, haar2d.
%
%          xp (and yp) specifies the kind of textural information, and may be var-
%          ied independently between 1 (high frequency information)   and
%          log2(XSIZE)+1 (low frequency), where XSIZE is the horisontal size of
%          the image. The horisontal size of the texture element will be 2^xp
%          (2^yp)  except for xp = log2(XSIZE)+1, which will use a texture element
%          of size XSIZE.  Correspondingly for yp/YSIZE/vertical.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type input band
%           2 => bad pixel type output band
%           3 => bands have different sizes
%           4 => sizes are not power of two
%           5 => bad xp or yp arguments
%           6 => malloc error
%
%   RESTRICTIONS
%          Input and output band must be of type IR_BAND and have equal size, both
%          horizontal and vertical size must be some power of two. The input band
%          must be the result of a forward 2D Haar transform.
%
%   SEE ALSO
%          haar2d(3), haari2d(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%                                     27 Feb 2014 HAARTEXTURE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"haarTexture"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IR_BAND input;
    IR_BAND output;
    int xp;
    int yp;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IR_BAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "xp" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        xp = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        xp = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "yp" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        yp = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        yp = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IR_BAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = haarTexture(input, output, xp, yp);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C haarTexture.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
