
/*C*

________________________________________________________________

        xite_haari1d.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/haar.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int haari1d(float *a1, float *a2, int size) */

/*M*
function [a2] = xite_haari1d(a1, size)
%xite_haari1d 1D and 2D forward and inverse Haar Transform.
%   Calls the XITE C function haari1d.
%   function [a2] = xite_haari1d(a1, size)
%
%   The C function has the following prototype:
%   int haari1d(float *a1, float *a2, int size)
%
%   and the following manual page:
%
%   NAME
%          haar, haar1d, haari1d, haar2d, haari2d - 1D and 2D forward and inverse
%          Haar Transform
%
%   SYNTAX
%           #include <xite/haar.h>
%
%           int haar1d( float* a1, float* a2, int size );
%
%           int haari1d( float* a1, float* a2, int size );
%
%           int haar2d( IR_BAND b1, IR_BAND b2 );
%
%           int haari2d( IR_BAND b1, IR_BAND b2 );
%
%   DESCRIPTION
%           ‘haar1d‘ - 1D forward Haar Transform (HT).
%           ‘haari1d‘ - 1D inverse HT
%           ‘haar2d‘ - 2D forward HT
%           ‘haari2d‘ - 2D inverse HT
%
%           a1 : input vector
%           a2 : output vector
%           size : size of a1 and a2.
%           b1 : input band
%           b2 : output band
%
%          Identical actual arguments are allowed for b1 and b2.
%
%   REFERENCES
%          N. Ahmed and K.R. Rao, Orthogonal Transforms for Digital Signal Pro-
%          cessing, p. 263, Springer-Verlag, 1975.
%
%          H.C.  Andrews and K.L. Caspari, "A generalized technique for spectral
%          analysis", IEEE Trans., Comput., Vol. C-19, pp. 16-25, 1970.
%
%          N. Ahmed, T. Natarajan and K.R. Rao, "Cooley-Tukey type algorithm for
%          the Haar transform", Electronics Letters, Vol. 9, pp. 276-278, 1973.
%
%   RESTRICTIONS
%          Vectors a1 and a2 should have size some power of two.  Bands b1 and b2
%          should have overlapping size some power of two. Both bands should have
%          pixel type Ireal_typ.
%
%   RETURN VALUE
%          haar1d and haari1d:
%           0 => ok
%           1 => negative size
%           2 => size not power of 2
%
%          haar2d and haari2d:
%           0 => ok
%           1 => bad pixel type band 1
%           2 => bad pixel type band 2
%           3 => bad size (not power of two)
%
%   SEE ALSO
%          fft2d(3), fft2d(1), fht2d(3), fht2d(1), haarTexture(3), haarTexture(1)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, ifi, UiO
%
%   ID
%          $Id: haar.c,v 1.13 2014/01/31 12:53:47 svein Exp $
%
%                                     27 Feb 2014 HAAR(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"haari1d"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    float *a1;
    float *a2;
    int size;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    a1 = ((float **) mxGetData(prhs[0]))[0];
    
    /* If scalar double is passed in for "size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        size = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        size = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    a2 = (float *) mxCalloc(size, sizeof(float));
    
    /* Execute C routine. */
    return_value = haari1d(a1, a2, size);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C haari1d.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(size, 1, mxSINGLE_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), a2, (size) * sizeof(float));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
