
/*C*

________________________________________________________________

        xite_gammaAdjust.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus gammaAdjust(IBAND inband, IBAND outband, double adjust) */

/*M*
function [outband] = xite_gammaAdjust(inband, adjust)
%xite_gammaAdjust Simple gamma correction.
%   Calls the XITE C function gammaAdjust.
%   function [outband] = xite_gammaAdjust(inband, adjust)
%
%   The C function has the following prototype:
%   BiffStatus gammaAdjust(IBAND inband, IBAND outband, double adjust)
%
%   and the following manual page:
%
%   NAME
%          gammaAdjust - Simple gamma correction
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           BiffStatus gammaAdjust( IBAND inband,
%              IBAND outband, double adjust );
%
%   DESCRIPTION
%          A simple gamma corretion to make images darker or brigter. The adjust
%          parameter must be in the range -1.0 (much darker) to 1 (much brighter).
%          If adjust = 0.0 the band is not changed.
%
%           If adjust > 0:
%           a= adjust**2/0.16;
%           b= 255.0 / log(a*255.0+1.0);
%           c= 256.0/a * log(1.0 + (exp(a) - 1.0) * i / 256.0);
%           col[i] = c;
%
%           If adjust < 0;
%           a= adjust**2/0.16;
%           b= 255.0 / log(a*255.0+1.0);
%           c= 256.0/a * log(1.0 + (exp(a) - 1.0) * (255-i) / 256.0);
%           col[i] = 255-c;
%
%   RETURN VALUE
%           0 - OK
%           2 - Bad pixtype
%           3 - Bad size
%           4 - Illegal value for adjust.
%
%   RESTRICTIONS
%          Works on unsigned byte bands only.
%
%   AUTHOR
%          Otto Milvang
%
%                                     27 Feb 2014 GAMMAADJUST(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"gammaAdjust"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    double adjust;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    adjust = ((double *) mxGetData(prhs[1]))[0];

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = gammaAdjust(inband, outband, adjust);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C gammaAdjust.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
