
/*C*

________________________________________________________________

        xite_histoEq.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus histoEq(IBAND b1, IBAND b2, int graylevels) */

/*M*
function [b2] = xite_histoEq(b1, graylevels)
%xite_histoEq Histogram equalization.
%   Calls the XITE C function histoEq.
%   function [b2] = xite_histoEq(b1, graylevels)
%
%   The C function has the following prototype:
%   BiffStatus histoEq(IBAND b1, IBAND b2, int graylevels)
%
%   and the following manual page:
%
%   NAME
%          histoEq - histogram equalization
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           BiffStatus histoEq( IBAND input, IBAND output,
%              int graylevels );
%
%   DESCRIPTION
%          Perform histogram equalization.  input and output may be identical. The
%          whole input band is used to generate the histogram transformation, but
%          only the largest rectangle common to input and output is transformed.
%
%          graylevels is the maximum number of levels in output.
%
%   RESTRICTIONS
%          input and output must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type b1
%           2 => bad pixel type b2
%           3 => bad value graylevels
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: histoEq.c,v 1.23 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 HISTOEQ(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"histoEq"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b1;
    IBAND b2;
    int graylevels;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b1_img;
    IMAGE b2_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    b1_img = Init_image(1, "");
    
    convert_status = matlab2biff(b1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    b1 = (IBAND) (b1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b1));
    
    /* If scalar double is passed in for "graylevels" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        graylevels = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        graylevels = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    b2_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b2 = (IBAND) (b2_img[1]);
    
    /* Execute C routine. */
    return_value = histoEq(b1, b2, graylevels);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C histoEq.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(b2_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
