
/*C*

________________________________________________________________

        xite_histoEqCol.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void histoEqCol(IBAND b, Color_tab ct, int graylevels) */

/*M*
function [ct] = xite_histoEqCol(b, graylevels)
%xite_histoEqCol Histogram equalization color table.
%   Calls the XITE C function histoEqCol.
%   function [ct] = xite_histoEqCol(b, graylevels)
%
%   The C function has the following prototype:
%   void histoEqCol(IBAND b, Color_tab ct, int graylevels)
%
%   and the following manual page:
%
%   NAME
%          histoEqCol - histogram equalization color table
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           void histoEqCol( IBAND b, Color_tab ct,
%              int graylevels );
%
%   DESCRIPTION
%          Make a color table doing histogram equalization of a band. Assuming
%          byte pixels.
%
%           b - input band
%           ct - output color table
%           graylevels : max number of levels in color table
%
%   RETURN VALUE
%   AUTHOR
%          Tor L|nnestad
%
%   ID
%          $Id: histoEqCol.c,v 1.21 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 HISTOEQCOL(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"histoEqCol"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b;
    Color_tab ct;
    int graylevels;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b_img;
    const char *color_field_names[] = {
        "pixel",
        "red",
        "green",
        "blue",
        "flags",
        "pad",
    };
    int color_field_classes[] = {
        mxUINT32_CLASS,
        mxUINT16_CLASS,
        mxUINT16_CLASS,
        mxUINT16_CLASS,
        mxCHAR_CLASS,
        mxCHAR_CLASS,
    };
    int cell_cnt, field_cnt;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    b_img = Init_image(1, "");
    
    convert_status = matlab2biff(b_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    b = (IBAND) (b_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b));
    
    /* If scalar double is passed in for "graylevels" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        graylevels = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        graylevels = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    histoEqCol(b, ct, graylevels);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateStructMatrix(256, 1, 6, color_field_names);
    
    for (field_cnt = 0; field_cnt < 6; field_cnt++) {
        const char *name = color_field_names[field_cnt];
        int class  = color_field_classes[field_cnt];
    
        for (cell_cnt = 0; cell_cnt < 256; cell_cnt++) {
    	mxArray *arr;
    
    	if (!strcmp(name, "pixel")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].pixel;
    	} else if (!strcmp(name, "red")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].red;
    	} else if (!strcmp(name, "green")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].green;
    	} else if (!strcmp(name, "blue")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].blue;
    	} else if (!strcmp(name, "flags")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].flags;
    	} else if (!strcmp(name, "pad")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = ct[cell_cnt].pad;
    	}
    
    	mxSetFieldByNumber(tmp_plhs[0], cell_cnt, field_cnt, arr);
        }
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
