
/*C*

________________________________________________________________

        xite_mkCumHisto.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void mkCumHisto(histogram histo, histogram cumHisto) */

/*M*
function [cumHisto] = xite_mkCumHisto(histo)
%xite_mkCumHisto Make a cumulative histogram.
%   Calls the XITE C function mkCumHisto.
%   function [cumHisto] = xite_mkCumHisto(histo)
%
%   The C function has the following prototype:
%   void mkCumHisto(histogram histo, histogram cumHisto)
%
%   and the following manual page:
%
%   NAME
%          mkCumHisto - make a cumulative histogram
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           void mkCumHisto( histogram histo,
%              histogram cumHisto );
%
%   DESCRIPTION
%          make a cumulative histogram based on a histogram.  The output histogram
%          is not scaled. To scale it, call scaleHisto. Identical in and out param
%          are legal.
%
%           histo - input histogram
%           cumHisto - output cumulative histogram
%
%   RETURN VALUE
%   AUTHOR
%          Tor L|nnestad
%
%                                     27 Feb 2014 MKCUMHISTO(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkCumHisto"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram histo;
    histogram cumHisto;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr histo_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    histo_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(histo, histo_ptr, sizeof(int) * 256);

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    mkCumHisto(histo, cumHisto);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(256, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), cumHisto, 256 * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
