
/*C*

________________________________________________________________

        xite_mkDhistogram.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus mkDhistogram(IBAND band, int histo[], double limits[], int n) */

/*M*
function [histo] = xite_mkDhistogram(band, limits, n)
%xite_mkDhistogram Make a histogram.
%   Calls the XITE C function mkDhistogram.
%   function [histo] = xite_mkDhistogram(band, limits, n)
%
%   The C function has the following prototype:
%   BiffStatus mkDhistogram(IBAND band, int histo[], double limits[], int n)
%
%   and the following manual page:
%
%   NAME
%          mkHisto, mkIhistogram, mkDhistogram - make a histogram
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           BiffStatus mkHisto( IBAND band,
%              histogram histo );
%
%           BiffStatus mkIhistogram( IBAND band,
%              int histo[], int limits[], int n );
%
%           BiffStatus mkDhistogram( IBAND band,
%              int histo[], double limits[], int n );
%
%   DESCRIPTION
%          Generate the histogram of a band.
%           band : input band
%           histo : output histogram
%           limits : input limits for the accumulator cells in histo
%           n : input number of limits (n >= 1)
%
%          limits should be indexable on 0..  n -1, and (NOTE!)  histo should be
%          indexable on 0..  n.  At return, histo contains:
%           histo[0] = #pixels in < -inf .. limits[0] ]
%           histo[i] = #pixels in < limits[i-1] .. limits[i] ]
%           histo[n] = #pixels in < limits[n-1] .. inf. >
%
%          Choose mkIhistogram or mkDhistogram, depending on whether your limits
%          are integers or real numbers.
%
%   RESTRICTIONS
%          mkHisto accepts only unsigned byte pixels, mkIhistogram and mkDhis-
%          togram accept all pixel types but complex and double complex.   limits
%          should be sorted in ascending order.
%
%   RETURN VALUE
%          status, see diagnostics
%
%   DIAGNOSTICS
%           Status 0 => ok
%           Status 2 => Error: Bad pixel type
%           Status 3 => Error: Bad ’n’ argument (mkHistogram only)
%
%   SEE ALSO
%          histo(3), scale(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: mkHisto.c,v 1.26 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 MKHISTO(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkDhistogram"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    int *histo;
    double *limits;
    int n;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    limits = ((double **) mxGetData(prhs[1]))[0];
    
    /* If scalar double is passed in for "n" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        n = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        n = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
    histo = (int *) mxCalloc(n+1, sizeof(int));
    
    /* Execute C routine. */
    return_value = mkDhistogram(band, histo, limits, n);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mkDhistogram.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(n+1, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), histo, (n+1) * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
