
/*C*

________________________________________________________________

        xite_scaleHisto.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void scaleHisto(histogram cumHisto, histogram scHisto, int range) */

/*M*
function [scHisto] = xite_scaleHisto(cumHisto, range)
%xite_scaleHisto Scale a cumulative histogram.
%   Calls the XITE C function scaleHisto.
%   function [scHisto] = xite_scaleHisto(cumHisto, range)
%
%   The C function has the following prototype:
%   void scaleHisto(histogram cumHisto, histogram scHisto, int range)
%
%   and the following manual page:
%
%   NAME
%          scaleHisto - scale a cumulative histogram
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           void scaleHisto( histogram cumHisto,
%              histogram scHisto, int range );
%
%   DESCRIPTION
%          Scale a cumulative histgram for use as histogram transformation.  scale
%          implies that all values are linearly scaled to the interval
%          [0..<range>].  Identical in- and out-parameter is legal.
%
%           cumHisto - input cumulative histogram
%           scHisto - output scaled cumulative histogram
%           range - Number of levels in scHisto, typically 256
%
%   RETURN VALUE
%   AUTHOR
%          Tor L|nnestad & Otto Milvang
%
%   ID
%          $Id: scaleHisto.c,v 1.17 1995/08/23 14:55:44 svein Exp $
%
%                                     27 Feb 2014 SCALEHISTO(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"scaleHisto"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram cumHisto;
    histogram scHisto;
    int range;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr cumHisto_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    cumHisto_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(cumHisto, cumHisto_ptr, sizeof(int) * 256);
    
    /* If scalar double is passed in for "range" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        range = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        range = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    scaleHisto(cumHisto, scHisto, range);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(256, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), scHisto, 256 * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
