
/*C*

________________________________________________________________

        xite_scaled2col.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void scaled2col(histogram scHisto, Color_tab colTab) */

/*M*
function [colTab] = xite_scaled2col(scHisto)
%xite_scaled2col Convert scaled cumulative histogram.
%   Calls the XITE C function scaled2col.
%   function [colTab] = xite_scaled2col(scHisto)
%
%   The C function has the following prototype:
%   void scaled2col(histogram scHisto, Color_tab colTab)
%
%   and the following manual page:
%
%   NAME
%          scaled2col - convert scaled cumulative histogram
%
%   SYNTAX
%           #include <xite/histo.h>
%
%           void scaled2col( histogram scHisto,
%              Color_tab colTab );
%
%   DESCRIPTION
%          Make a color table according to a scaled cumulative histogram.
%
%           scHisto - input scaled cumulative histogram
%           colTab - output color table
%
%   RETURN VALUE
%   AUTHOR
%          Tor L|nnestad
%
%   ID
%          $Id: scaled2col.c,v 1.15 1995/08/23 14:55:46 svein Exp $
%
%                                     27 Feb 2014 SCALED2COL(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"scaled2col"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram scHisto;
    Color_tab colTab;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr scHisto_ptr;
    const char *color_field_names[] = {
        "pixel",
        "red",
        "green",
        "blue",
        "flags",
        "pad",
    };
    int color_field_classes[] = {
        mxUINT32_CLASS,
        mxUINT16_CLASS,
        mxUINT16_CLASS,
        mxUINT16_CLASS,
        mxCHAR_CLASS,
        mxCHAR_CLASS,
    };
    int cell_cnt, field_cnt;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 2) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    scHisto_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(scHisto, scHisto_ptr, sizeof(int) * 256);

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    scaled2col(scHisto, colTab);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateStructMatrix(256, 1, 6, color_field_names);
    
    for (field_cnt = 0; field_cnt < 6; field_cnt++) {
        const char *name = color_field_names[field_cnt];
        int class  = color_field_classes[field_cnt];
    
        for (cell_cnt = 0; cell_cnt < 256; cell_cnt++) {
    	mxArray *arr;
    
    	if (!strcmp(name, "pixel")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].pixel;
    	} else if (!strcmp(name, "red")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].red;
    	} else if (!strcmp(name, "green")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].green;
    	} else if (!strcmp(name, "blue")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].blue;
    	} else if (!strcmp(name, "flags")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].flags;
    	} else if (!strcmp(name, "pad")) {
    	    arr = mxCreateNumericMatrix(1, 1, class, mxREAL);
    	    *mxGetPr(arr) = colTab[cell_cnt].pad;
    	}
    
    	mxSetFieldByNumber(tmp_plhs[0], cell_cnt, field_cnt, arr);
        }
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
