
/*C*

________________________________________________________________

        xite_houghCircles.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/hough.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus houghCircles(IBAND input, IUS_IMAGE output, int rmin, int rmax) */

/*M*
function [output] = xite_houghCircles(input, rmin, rmax)
%xite_houghCircles Hough transform for circle detection.
%   Calls the XITE C function houghCircles.
%   function [output] = xite_houghCircles(input, rmin, rmax)
%
%   The C function has the following prototype:
%   BiffStatus houghCircles(IBAND input, IUS_IMAGE output, int rmin, int rmax)
%
%   and the following manual page:
%
%   NAME
%          houghCircle, houghCircles - Hough transform for circle detection
%
%   SYNTAX
%           #include <xite/hough.h>
%
%           BiffStatus houghCircle( IBAND input,
%              IUS_BAND output, int radius );
%
%           BiffStatus houghCircles( IBAND input,
%              IUS_IMAGE output, int rmin, int rmax );
%
%   DESCRIPTION
%          Performs a Hough transform to detect circles.
%
%          houghCircle transforms the input band input to Hough space, output,
%          looking at circles with radius only. The input band should be something
%          like a thresholded edge image. All pixels larger than 0 are transformed
%          to Hough space.
%
%          houghCircles transforms for all radii between rmin and rmax.   First
%          band of output will correspond to radius rmin, second band to radius
%          rmin +1 and so on.  output should thus have a number of bands at least
%          equal to rmax - rmin + 1.
%
%          Notice that output band(s) are of type unsigned short to avoid overflow
%          in accumulation.
%
%   RESTRICTIONS
%          input must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 - OK
%           1 - bad input pixel type
%           2 - bad output pixel type
%           3 - bad radius value
%           4 - not enough bands
%
%   SEE ALSO
%          houghCircle(1), houghLine(3), drawLine(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: houghCircle.c,v 1.37 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 HOUGHCIRCLE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"houghCircles"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IUS_IMAGE output;
    int rmin;
    int rmax;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "rmin" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        rmin = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        rmin = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "rmax" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        rmax = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        rmax = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = houghCircles(input, output, rmin, rmax);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C houghCircles.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status = 
            biff2matlab((IMAGE) output, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
