
/*C*

________________________________________________________________

        xite_bandpass.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ideal.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int bandpass(IBAND cb, double low_cut_frequency, double high_cut_frequency, int filter_size, window_type win_type) */

/*M*
function [cb] = xite_bandpass(cb, low_cut_frequency, high_cut_frequency, filter_size, win_type)
%xite_bandpass "Ideal" filters truncated with window function in the spatial
%   domain.
%   Calls the XITE C function bandpass.
%   function [cb] = xite_bandpass(cb, low_cut_frequency, high_cut_frequency, filter_size, win_type)
%
%   The C function has the following prototype:
%   int bandpass(IBAND cb, double low_cut_frequency, double high_cut_frequency, int filter_size, window_type win_type)
%
%   and the following manual page:
%
%   NAME
%          lowpass, highpass, bandpass, bandstop, lowpassu - "Ideal" filters trun-
%          cated with window function in the spatial domain
%
%   SYNTAX
%           #include <xite/ideal.h>
%
%           int lowpass (IBAND band, double cut_frequency,
%              int filter_size, window_type win_type);
%
%           int highpass (IBAND band, double cut_frequency,
%              int *filter_size, window_type win_type);
%
%           int bandpass (IBAND band, double low_cut_frequency,
%              double high_cut_frequency, int filter_size,
%              window_type win_type);
%
%           int bandstop (IBAND band, double low_cut_frequency,
%              double high_cut_frequency, int *filter_size,
%              window_type win_type);
%
%           int lowpassu (IBAND band, double cut_frequency,
%              int filter_size, window_type win_type, double *sum);
%
%   DESCRIPTION
%          lowpass generates a lowpass filter in the spatial domain.   The filter
%          is made from an ideal lowpass-filter which is truncated with a window
%          function in the spatial domain.
%
%          filter_size specifies filter diameter.  cut_frequency is relative to
%          the Nyquist frequency, i.e. a cut_frequency of 1.0 means half the sam-
%          pling frequency.  win_type specifies window function. Legal values for
%          win_type are the constants
%
%          BARTLETT
%                 Circularly symmetric Bartlett window
%
%          HAMMING
%                 Circularly symmetric Hamming window
%
%          HANNING
%                 Circularly symmetric Hanning (von Hann) window
%
%          RECTANGLE
%                 Circularly symmetric rectangular (box, boxcar) window function
%
%          TRIANGLE
%                 Circularly symmetric triangular window function
%
%            The circular symmetry is achieved using Huang’s method.
%
%            lowpass is normalized so that the sum of the filter coefficients
%            equals 1.0.   lowpassu is not normalized.  For lowpassu, sum is
%            returned as the sum of the filter coefficients.
%
%            For odd filter_size, the symmetry-center of the filter is located in
%            pixel (1,1) in band.  For even filter_size, the symmetry-center of
%            the filter is located at the upper-left corner of pixel (1,1), i.e.
%            the symmetry-center does not fall into the center of a pixel.
%
%            For even filter_size and even horizontal band-size, the resulting
%            filter_size may become smaller than you’ld expect, because of the
%            location of the symmetry-center of the filter, as explained above.
%
%            highpass, bandpass and bandstop behave correspondingly.  highpass is
%            implemented as delta(n1, n2)  -  lowpass, with sum of coefficients
%            equal to 0.0.  bandpass is implemented as the difference between two
%            lowpass-filters.   bandstop is implemented as delta(n1, n2) - band-
%            pass.
%
%            For highpass and bandstop, filter_size should be odd.  If it is even,
%            a warning is issued and filter_size will be made odd. For these func-
%            tions, filter_size is a reference.
%
%   RESTRICTIONS
%          band must be one of the pixeltypes real, double, complex or double com-
%          plex.
%
%          filter_size must not exceed horizontal or vertical size of band.
%
%          For highpass and bandstop, filter_size must be odd.  If it is even, it
%          will be increased by one (or decreased by one if this exceeds the size
%          of band).
%
%   RETURN VALUE
%           0 : Ok
%           1 : Bad pixeltype
%           2 : Unknown window type
%           3 : Filtersize larger than bandsize
%
%   SEE ALSO
%          bartlett(3), window(3), lowpassf(3), lowpass(1)
%
%   REFERENCE
%          Jae S.. Lim
%                 "Two-dimensional Signal And Image Processing" Prentice-Hall,
%                 1990
%
%   AUTHOR
%          Trond Muri Sørensen. Major revision by Svein Bøe.
%
%   DOC
%          Svein Bøe
%
%   BANDSTOP, BANDPASS, LOWPASS,(HIGHPASS,)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"bandpass"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND cb;
    double low_cut_frequency;
    double high_cut_frequency;
    int filter_size;
    window_type win_type;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE cb_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 5) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 5) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    cb_img = Init_image(1, "");
    
    convert_status = matlab2biff(cb_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    cb = (IBAND) (cb_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) cb));
    output_ysize = MAX(output_ysize, Iysize((IBAND) cb));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) cb));
    low_cut_frequency = ((double *) mxGetData(prhs[1]))[0];
    high_cut_frequency = ((double *) mxGetData(prhs[2]))[0];
    
    /* If scalar double is passed in for "filter_size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        filter_size = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        filter_size = ((int *) mxGetData(prhs[3]))[0];
    }
    win_type = ((window_type *) mxGetData(prhs[4]))[0];

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = bandpass(cb, low_cut_frequency, high_cut_frequency, filter_size, win_type);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C bandpass.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(cb_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
