
/*C*

________________________________________________________________

        xite_lowpassIdealf.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ideal.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus lowpassIdealf(IBAND out_band, double cut_frequency) */

/*M*
function [out_band] = xite_lowpassIdealf(out_band, cut_frequency)
%xite_lowpassIdealf Make a filter in the Fourier domain.
%   Calls the XITE C function lowpassIdealf.
%   function [out_band] = xite_lowpassIdealf(out_band, cut_frequency)
%
%   The C function has the following prototype:
%   BiffStatus lowpassIdealf(IBAND out_band, double cut_frequency)
%
%   and the following manual page:
%
%   NAME
%          lowpassf, highpassf, bandpassf, bandstopf, lowpassIdealf, butterworthf,
%          exponentialf, parseFilterTypeName, fftMakeLp, fftMakeHp, fftMakeBp,
%          fftMakeBs - make a filter in the Fourier domain
%
%   SYNTAX
%           #include <xite/ideal.h>
%
%           int lowpassf( IBAND band,
%              double cut_frequency, int filter_size,
%              window_type win_type );
%
%           int highassf( IBAND band,
%              double cut_frequency, int *filter_size,
%              window_type win_type );
%
%           int bandpassf( IBAND band,
%              double low_cut_frequency, double high_cut_frequency,
%              int filter_size, window_type win_type );
%
%           int bandstopf( IBAND band,
%              double low_cut_frequency, double high_cut_frequency,
%              int *filter_size, window_type win_type );
%
%           int lowpassIdealf( IBAND band,
%              double cut_frequency );
%
%           int butterworthf( IBAND band, double cut_frequency,
%              int filter_order );
%
%           int exponentialf( IBAND band, double cut_frequency,
%              int filter_order );
%
%           filter_type parseFilterTypeName( char *name );
%
%           #include <xite/fft.h>
%
%           int fftMakeLp( IC_BAND band,
%              double cut_frequency, int filter_size );
%
%           int fftMakeHp( IC_BAND band,
%              double cut_frequency, int filter_size );
%
%           int fftMakeBp( IC_BAND band,
%              double low_cut_frequency,
%              double high_cut_frequency, int filter_size );
%
%           int fftMakeBs( IC_BAND band,
%              double low_cut_frequency,
%              double high_cut_frequency, int filter_size );
%
%   DESCRIPTION
%          lowpassf makes a lowpass-filter in the Fourier domain. The filter used
%          is an ideal lowpass-filter truncated with a window in the spatial
%          domain.   lowpassf calls lowpass to generate the filter in the spatial
%          domain, then calls fft2d to get the Fourier transform. The scaling in
%          the fft is 1.0, although the standard/default scaling in fft2d forward
%          transform is 1/(xsize*ysize). The choice of scaling will give a DC
%          (frequency (0,0)) value of 1.0.
%
%          filter_size specifies filter diameter.  cut_frequency is relative to
%          the Nyquist frequency, i.e. a cut_frequency of 1.0 means half the sam-
%          pling frequency.  win_type specifies the type of window function. Legal
%          value are the constants
%
%          RECTANGLE
%                 Circularly symmetric rectangular (box, boxcar) window function
%
%          BARTLETT
%                 Circularly symmetric Bartlett window
%
%          TRIANGLE
%                 Circularly symmetric triangular window function
%
%          HAMMING
%                 Circularly symmetric Hamming window
%
%          HANNING
%                 Circularly symmetric Hanning (von Hann) window
%
%            The circular symmetry is achieved using Huang’s method.
%
%            highpassf, bandpassf and bandstopf all call their spatial-domain
%            counterparts and fft2d with scaling equal to 1.0.
%
%            lowpassIdealf makes an ideal lowpass-filter in the Fourier domain.
%            Unlike lowpassf, lowpassIdealf is not made from a window-truncated
%            spatial-domain filter. It is the box-shaped ideal frequency response
%            which corresponds to a non-realizable spatial-domain filter.
%
%            butterworthf makes a Butterworth filter in the Fourier domain.
%
%            exponentialf makes an exponential filter in the Fourier domain.
%
%            parseFilterTypeName returns a symbolic integer value for various
%            forms of filter-type name.  Legal forms of name are
%
%             Full name Short name Number
%             lowpass lp LOWPASS
%             highpass hp HIGHPASS
%             bandpass bp BANDPASS
%             bandstop bs BANDSTOP
%             lowpass_ideal ilp LOWPASS_IDEAL
%             butterworth_lp blp BUTTER_LP
%             exponential_lp elp EXP_LP
%
%            fftMakeLp was a function similar (but not identical) to lowpassf in
%            previous XITE versions. It is retained for compatibility, but now as
%            a macro call to lowpassf with win_type equal to HAMMING. The old fft-
%            MakeLp gave results slightly different from the results of lowpassf.
%            This is due to differences in the implementation of the window func-
%            tion. The main difference is that filter_size will be reduced by two
%            in lowpass, compared to fftMakeLp.
%
%   RESTRICTIONS
%          Horizontal and vertical size of band must be power of two.
%
%          filter_size may not be smaller than horizontal or vertical size of
%          band.
%
%          For highpassf and bandstopf, filter_size should be odd. If it is even,
%          it will be made odd and a warning is issued. For these two functions,
%          filter_size is a reference.
%
%          band must be of pixeltype complex.
%
%   RETURN VALUE
%           0 - OK
%           1 - Bad pixel type
%           2 - Unknown window type
%           3 - Filter_size larger than bandsize
%           4 - Error in fft2d
%
%   SEE ALSO
%          bartlett(3), window(3), lowpass(3), lowpass(1), fft2d(3), phase(3),
%          absValue(3), power(3), real(3), imag(3)
%
%   REFERENCE
%          Jae S.. Lim
%                 "Two-dimensional Signal And Image Processing" Prentice-Hall,
%                 1990
%
%   AUTHOR
%          Trond Muri Sørensen
%
%   BANDSTOPF, BANDPASSF, LOWPASSF,(HIGHPASSF,)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"lowpassIdealf"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND out_band;
    double cut_frequency;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE out_band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    out_band_img = Init_image(1, "");
    
    convert_status = matlab2biff(out_band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    out_band = (IBAND) (out_band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) out_band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) out_band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) out_band));
    cut_frequency = ((double *) mxGetData(prhs[1]))[0];

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = lowpassIdealf(out_band, cut_frequency);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C lowpassIdealf.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(out_band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
