
/*C*

________________________________________________________________

        xite_parseWindowName.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ideal.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* window_type parseWindowName(char *name) */

/*M*
function [return_value] = xite_parseWindowName(name)
%xite_parseWindowName Circularly symmetric window functions for filters and
%   parsing of window function name.
%   Calls the XITE C function parseWindowName.
%   function [return_value] = xite_parseWindowName(name)
%
%   The C function has the following prototype:
%   window_type parseWindowName(char *name)
%
%   and the following manual page:
%
%   NAME
%          bartlett, hamming, hanning, rectangle, triangle, parseWindowName - cir-
%          cularly symmetric window functions for filters and parsing of window
%          function name
%
%   SYNTAX
%           #include <xite/ideal.h>
%
%           double bartlett (double n, int size);
%           double hamming (double n, int size);
%           double hanning (double n, int size);
%           double rectangle (double n, int size);
%           double triangle (double n, int size);
%           window_type parseWindowName(char *name);
%
%   DESCRIPTION
%          The bartlett, hamming, hanning (von Hann), rectangle (box, boxcar) and
%          triangle functions return function values for the various windows, at a
%          certain distance n.  size is the diameter of the window function.
%
%          Window functions are typically used to multiply with a filter in the
%          spatial domain.
%
%          These functions should only be called when
%           0 <= n < size
%
%          Outside this interval, the functions should be considered equal to
%          zero.
%
%          rectangle is given by
%           w(n)=1.0
%
%          bartlett is given by
%                2n/(size-1), n<=(size-1)/2
%           w(n)=
%                2-2n/(size-1), n>(size-1)/2
%
%          triangle is given by
%                2(n+1)/(size+1), n<=(size-1)/2
%           w(n)=
%                2-2(n+1)/(size+1), n>(size-1)/2
%
%          triangle is almost the same as bartlett, but triangle does not include
%          zeros at the end-points.
%
%          hamming is given by
%           w(n)=0.54-0.46*cos(2 Pi n/(size-1))
%
%          hanning is given by
%           w(n)=0.5-0.5*cos(2 Pi (n+1)/(size+1))
%
%          hanning is sometimes defined slightly different. This version does not
%          include zeros at the end-points.
%
%          parseWindowName returns a symbolic integer value for various forms of
%          window name.  Legal forms of name are
%
%           Full name Short name Number
%           -----------------------------
%           rectangle r RECTANGLE
%           bartlett b BARTLETT
%           triangle t TRIANGLE
%           hamming hm HAMMING
%           hanning hn HANNING
%
%   RETURN VALUE
%          Function value.
%
%   SEE ALSO
%          window(3), lowpass(3), lowpassf(3)
%
%   AUTHOR
%          Trond Muri Sørensen, Svein Bøe
%
%   RECTANGLE, HANNING, BARTLETT,(HAMMING,)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"parseWindowName"};
    window_type return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    char *name;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    name = ((char **) mxGetData(prhs[0]))[0];
    
    /* Execute C routine. */
    return_value = parseWindowName(name);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((window_type *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
