
/*C*

________________________________________________________________

        xite_window.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ideal.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int window(IBAND band, int size, window_type win_type) */

/*M*
function [band] = xite_window(band, size, win_type)
%xite_window Insert a window-function in a BIFF band.
%   Calls the XITE C function window.
%   function [band] = xite_window(band, size, win_type)
%
%   The C function has the following prototype:
%   int window(IBAND band, int size, window_type win_type)
%
%   and the following manual page:
%
%   NAME
%          window - Insert a window-function in a BIFF band.
%
%   SYNTAX
%           #include <xite/ideal.h>
%
%           int window(IBAND band, int size,
%              window_type win_type);
%
%   DESCRIPTION
%          Produce a circularly symmetric window of diameter size in band.  Legal
%          values for win_type are
%
%          BARTLETT
%                 Circularly symmetric Bartlett (triangular) window
%
%          HAMMING
%                 Circularly symmetric Hamming window
%
%          HANNING
%                 Circularly symmetric Hanning (von Hann) window
%
%          RECTANGLE
%                 Circularly symmetric rectangular (box, boxcar) window function
%
%          TRIANGLE
%                 Circularly symmetric triangular window function
%
%            The window is returned with its center in the center of band.   This
%            means that the actual center will be located between two pixels in
%            the horizontal direction when the horizontal band size is even.  When
%            the vertical band size is even, the center will be located between
%            two pixels in the vertical direction. When both horizontal and verti-
%            cal band size is even, the window center will be located between four
%            pixels. With both horizontal and vertical band size odd, the window
%            center will be located exactly inside the center pixel of the band.
%
%            To be applied as a multiplicative window to an image-domain filter,
%            make sure that also the filter-function is centered around the band
%            center, or if the filter-function is centered around a band corner,
%            the window must be shifted correspondingly before it can be applied.
%            With center on a band corner, the resulting filter may be zero phase
%            (if the filter function itself obeys the necessary symmetry rela-
%            tions).
%
%   RETURN VALUE
%           0 : Ok
%           1 : Unknown window type
%
%   SEE ALSO
%          bartlett(3), window(1), lowpass(3), lowpassf(3)
%
%   AUTHOR
%          Svein Bøe, Blab, Ifi, UiO
%
%   ID
%          $Id: window.c,v 1.25 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 WINDOW(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"window"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    int size;
    window_type win_type;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    
    /* If scalar double is passed in for "size" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        size = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        size = ((int *) mxGetData(prhs[1]))[0];
    }
    win_type = ((window_type *) mxGetData(prhs[2]))[0];

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = window(band, size, win_type);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C window.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
