
/*C*

________________________________________________________________

        xite_ihs2rgb_img.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ihs.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int ihs2rgb_img(IBAND i, IBAND h, IBAND s, IBAND r, IBAND g, IBAND b) */

/*M*
function [r, g, b] = xite_ihs2rgb_img(i, h, s)
%xite_ihs2rgb_img Convert image from IHS to RGB.
%   Calls the XITE C function ihs2rgb_img.
%   function [r, g, b] = xite_ihs2rgb_img(i, h, s)
%
%   The C function has the following prototype:
%   int ihs2rgb_img(IBAND i, IBAND h, IBAND s, IBAND r, IBAND g, IBAND b)
%
%   and the following manual page:
%
%   NAME
%          ihs2rgb_img - Convert image from IHS to RGB
%
%   SYNTAX
%           #include <xite/ihs.h>
%
%           int ihs2rgb_img( IBAND i, IBAND h, IBAND s,
%              IBAND r, IBAND g, IBAND b );
%
%   DESCRIPTION
%          For each pixel convert (i, h, s) to (r, g, b).
%
%   RETURN VALUE
%           0 - Ok
%           1 - Bad pixtyp
%           2 - Bad size
%
%   SEE ALSO
%          ihs2rgb(1), rgb2ihs(1), rgb2ihs_img(3), ihs(3)
%
%   AUTHOR
%          Otto Milvang
%
%                                     27 Feb 2014 IHS2RGB_IMG(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"ihs2rgb_img"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND i;
    IBAND h;
    IBAND s;
    IBAND r;
    IBAND g;
    IBAND b;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE i_img;
    IMAGE h_img;
    IMAGE s_img;
    IMAGE r_img;
    IMAGE g_img;
    IMAGE b_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 3) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    i_img = Init_image(1, "");
    
    convert_status = matlab2biff(i_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    i = (IBAND) (i_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) i));
    output_ysize = MAX(output_ysize, Iysize((IBAND) i));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) i));
    h_img = Init_image(1, "");
    
    convert_status = matlab2biff(h_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    h = (IBAND) (h_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) h));
    output_ysize = MAX(output_ysize, Iysize((IBAND) h));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) h));
    s_img = Init_image(1, "");
    
    convert_status = matlab2biff(s_img, NULL, 1,
                                 &prhs[2], NULL, NULL);
    s = (IBAND) (s_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) s));
    output_ysize = MAX(output_ysize, Iysize((IBAND) s));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) s));

    /* Prepare C result variable. */
    r_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    r = (IBAND) (r_img[1]);
    g_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    g = (IBAND) (g_img[1]);
    b_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b = (IBAND) (b_img[1]);
    
    /* Execute C routine. */
    return_value = ihs2rgb_img(i, h, s, r, g, b);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C ihs2rgb_img.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(r_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        convert_status =
    	biff2matlab(g_img, NULL, 1, &tmp_plhs[1], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 1) plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        convert_status =
    	biff2matlab(b_img, NULL, 1, &tmp_plhs[2], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 2) plhs[2] = tmp_plhs[2];
    }

    PopMessage();

    return;
}
