
/*C*

________________________________________________________________

        xite_ihs2rgb_word.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/ihs.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* void ihs2rgb_word(unsigned short i, unsigned short h, unsigned short s, unsigned short *r, unsigned short *g, unsigned short *b) */

/*M*
function [r, g, b] = xite_ihs2rgb_word(i, h, s)
%xite_ihs2rgb_word Convert between RGB and IHS.
%   Calls the XITE C function ihs2rgb_word.
%   function [r, g, b] = xite_ihs2rgb_word(i, h, s)
%
%   The C function has the following prototype:
%   void ihs2rgb_word(unsigned short i, unsigned short h, unsigned short s, unsigned short *r, unsigned short *g, unsigned short *b)
%
%   and the following manual page:
%
%   NAME
%          ihs, rgb2ihs, ihs2rgb, rgb2ihs_byte, ihs2rgb_byte, rgb2ihs_word,
%          ihs2rgb_word - Convert between RGB and IHS
%
%   SYNTAX
%           #include <xite/ihs.h>
%
%           void rgb2ihs( double r, double g, double b,
%              double* i, double* h, double* s, int norm );
%
%           void ihs2rgb( double i, double h, double s,
%              double* r, double* g, double* b, int norm );
%
%           void rgb2ihs_byte( unsigned char r,
%              unsigned char g, unsigned char b,
%              unsigned char *i, unsigned char *h,
%              unsigned char *s );
%
%           void ihs2rgb_byte( unsigned char i,
%              unsigned char h, unsigned char s,
%              unsigned char *r, unsigned char *g,
%              unsigned char *b );
%
%           void rgb2ihs_word( unsigned short r,
%              unsigned short g, unsigned short b,
%              unsigned short *i, unsigned short *h,
%              unsigned short *s );
%
%           void ihs2rgb_word( unsigned short i,
%              unsigned short h, unsigned short s,
%              unsigned short *r, unsigned short *g,
%              unsigned short *b );
%
%   DESCRIPTION
%          rgb2ihs converts from RGB-space to IHS-space.   ihs2rgb converts from
%          IHS-space to RGB-space.  rgb values are normalized
%           0.0 <= r <= 1.0, 0.0 <= b <= 1.0, 0.0 <= b <= 1.0,
%
%          If norm = TRUE the IHS values are normalized.
%           0.0 <= i <= 1.0, 0.0 <= h <= 1.0, 0.0 <= s <= 1.0,
%
%          If norm = FALSE:
%           0.0 <= i <= 1.0
%           0.0 <= h <  1.0
%           0.0 <= s <= Smax(I,H)
%
%          rgb2ihs_xxxx and ihs2rgb_xxxx normalize the data in the range 0-255 for
%          xxxx=byte and 0-65535 for xxxx=word
%
%   RGB2IHS
%           i = A11 * r + A12 * g + A13 * b;
%           u = A21 * r + A22 * g + A23 * b;
%           v = A31 * r + A32 * g + A33 * b;
%
%           h = atan2(v, u) , 0 <= h < 2*pi
%           s = sqrt(u*u + v*v)
%
%   IHS2RGB
%           u = s*cos(h)
%           v = s*sin(h)
%
%           r =  i + A21 * u + A31 * v;
%           g =  i + A22 * u + A32 * v;
%           b =  i + A23 * u + A33 * v;
%
%   VALUES
%           A11 = A12 = A13 =  1./3.
%           A21 = A22 = -1./sqrt(6.)
%           A23 =  2./sqrt(6.)
%           A31 = -A32 =  1./sqrt(2.)
%           A33 =  0.
%
%   NORMALIZATION
%           i’ = i / sqrt(3.0)
%           h’ = h / 2*pi
%           s’ = s / (Max s for (i,h))
%
%   SEE ALSO
%          ihs2rgb(1), ihs2rgb_img(3), rgb2ihs(1), rgb2ihs_img(3)
%
%   AUTHOR
%          Otto Milvang
%
%                                     27 Feb 2014 IHS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"ihs2rgb_word"};
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    unsigned short i;
    unsigned short h;
    unsigned short s;
    unsigned short r;
    unsigned short g;
    unsigned short b;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 4) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "i" which expects */
    /* "unsigned short", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        i = (unsigned short) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        i = ((unsigned short *) mxGetData(prhs[0]))[0];
    }
    
    /* If scalar double is passed in for "h" which expects */
    /* "unsigned short", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        h = (unsigned short) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        h = ((unsigned short *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "s" which expects */
    /* "unsigned short", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        s = (unsigned short) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        s = ((unsigned short *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
                
    /* Execute C routine. */
    ihs2rgb_word(i, h, s, &r, &g, &b);

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxUINT16_CLASS, mxREAL);
    ((unsigned short *) mxGetData(tmp_plhs[0]))[0] = r;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }
    if (nlhs > 1) {
        tmp_plhs[1] =
            mxCreateNumericMatrix(1, 1, mxUINT16_CLASS, mxREAL);
        ((unsigned short *) mxGetData(tmp_plhs[1]))[0] = g;
        plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =
            mxCreateNumericMatrix(1, 1, mxUINT16_CLASS, mxREAL);
        ((unsigned short *) mxGetData(tmp_plhs[2]))[0] = b;
        plhs[2] = tmp_plhs[2];
    }

    PopMessage();

    return;
}
