
/*C*

________________________________________________________________

        xite_mbkncn.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/kncn.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int mbkncn(IMAGE i1, IMAGE i2, int k) */

/*M*
function [i2] = xite_mbkncn(i1, k)
%xite_mbkncn Multi band k nearest connected neighbour noise reduction.
%   Calls the XITE C function mbkncn.
%   function [i2] = xite_mbkncn(i1, k)
%
%   The C function has the following prototype:
%   int mbkncn(IMAGE i1, IMAGE i2, int k)
%
%   and the following manual page:
%
%   NAME
%          mbkncn - multi band k nearest connected neighbour noise reduction
%
%   SYNTAX
%           #include <xite/kncn.h>
%
%           int mbkncn( IMAGE i1, IMAGE i2, int k );
%
%   DESCRIPTION
%          Performs multi band k nearest connected neighbours noise filtering. All
%          bands are filtered simultaneously. Taxi distance is used as distance
%          measure.
%
%          i1 is the input image, i2 is the output image.
%
%   RESTRICTIONS
%          The bands of i1 and i2 must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad pixel type input image
%           2 => Bad pixel type output image
%           3 => Bad k value
%
%   SEE ALSO
%          mbkncn(1), kncn(3), knn(3), mbknn(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: mbkncn.c,v 1.26 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 MBKNCN(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mbkncn"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IMAGE i1;
    IMAGE i2;
    int k;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    if (mxIsStruct(prhs[0])) {
      i1 = Init_image(mxGetNumberOfFields(prhs[0]), "");
    } else {
      i1 = Init_image(1, "");
    }
    convert_status = matlab2biff(i1, NULL, 1,
                                 &prhs[0], NULL, NULL);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) i1[1]));
    output_ysize = MAX(output_ysize, Iysize((IBAND) i1[1]));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) i1[1]));
    
    /* If scalar double is passed in for "k" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        k = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        k = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = mbkncn(i1, i2, k);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mbkncn.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status = 
            biff2matlab((IMAGE) i2, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
