
/*C*

________________________________________________________________

        xite_fractile_mask.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/median.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int fractile_mask(IBAND inband, IBAND outband, ISS_BAND mask, int mirror, double fractile) */

/*M*
function [outband] = xite_fractile_mask(inband, mask, mirror, fractile)
%xite_fractile_mask Noise reduction filtering.
%   Calls the XITE C function fractile_mask.
%   function [outband] = xite_fractile_mask(inband, mask, mirror, fractile)
%
%   The C function has the following prototype:
%   int fractile_mask(IBAND inband, IBAND outband, ISS_BAND mask, int mirror, double fractile)
%
%   and the following manual page:
%
%   NAME
%          fractile_mask - noise reduction filtering
%
%   SYNTAX
%           #include <xite/median.h>
%
%           int fractile_mask( IBAND inband, IBAND outband,
%              ISS_BAND mask, int mirror, double fractile );
%
%   DESCRIPTION
%          Filters the input band into the output band using a sorting filter
%          using mask.  Sorts all elements in the inimage according to mask and
%          selects the fractile pixel.
%
%          If fractile = 0.5, this is a median filter. If fractile is 0.0, this is
%          a mean filter. If fractile is 1.0, this is a max-filter. If mirror is
%          true, the mask is mirrored with respect to origo before it is applied.
%
%   RESTRICTIONS
%          inband and outband must have pixeltype unsigned byte.  All non-complex
%          pixeltypes are handled. The two bands must also have the same size.
%
%   RETURN VALUE
%           0 => ok
%           2 => input band must be of same type as output band
%           3 => input band must be of same size as output band
%
%   AUTHOR
%          Otto Milvang, BLAB, Ifi, UiO
%
%   ID
%          $Id: fractile_mask.c,v 1.20 1998/03/03 14:36:22 svein Exp $
%
%                                     27 Feb 2014 FRACTILE_MASK(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"fractile_mask"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    ISS_BAND mask;
    int mirror;
    double fractile;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;
    IMAGE mask_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    mask_img = Init_image(1, "");
    
    convert_status = matlab2biff(mask_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    mask = (ISS_BAND) (mask_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) mask));
    output_ysize = MAX(output_ysize, Iysize((IBAND) mask));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) mask));
    
    /* If scalar double is passed in for "mirror" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        mirror = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        mirror = ((int *) mxGetData(prhs[2]))[0];
    }
    fractile = ((double *) mxGetData(prhs[3]))[0];

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = fractile_mask(inband, outband, mask, mirror, fractile);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C fractile_mask.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
