
/*C*

________________________________________________________________

        xite_lit.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/median.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int lit(IBAND input, IBAND output, int dx, int dy, int dp) */

/*M*
function [output] = xite_lit(input, dx, dy, dp)
%xite_lit Local Information Transform texture measure.
%   Calls the XITE C function lit.
%   function [output] = xite_lit(input, dx, dy, dp)
%
%   The C function has the following prototype:
%   int lit(IBAND input, IBAND output, int dx, int dy, int dp)
%
%   and the following manual page:
%
%   NAME
%          lit - Local Information Transform texture measure
%
%   SYNTAX
%           #include <xite/median.h>
%
%           int lit( IBAND input, IBAND output, int dx,
%              int dy, int dp );
%
%   DESCRIPTION
%          lit filters the input band into the output band using a local filter
%          with window size dx * dy.  Even values of dx and dy are increased by
%          one.   Uses histogram updating when moving horizontally from pixel to
%          pixel.
%
%          Algorithm: lit(x,y) is number of pixels inside the processing window
%          (including the center pixel itself) with pixel value in the interval
%          cp-dp..cp+dp, where cp is center pixel value.
%
%          If histogram values become larger than 255, the pixels receive the
%          value 255.
%
%   RESTRICTIONS
%          input and output must have pixel type unsigned byte.
%
%   REFERENCE
%          Lowitz, Gabriel E.: "Extracting the Local Information Content of a Spa-
%          tial Image" in Proc.  7.ICPR, Montral, Canada, Aug. 1984, 1266-1268.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel value input band
%           2 => bad pixel value output band
%           3 => bad dx value (less than 1 or greater than xsize)
%           4 => bad dy value (less than 1 or greater than ysize)
%           5 => bad dp value (less than 0 or greater than 255)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO.
%
%   ID
%          $Id: lit.c,v 1.26 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 LIT(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"lit"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    int dx;
    int dy;
    int dp;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "dx" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        dx = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        dx = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "dy" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        dy = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        dy = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "dp" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        dp = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        dp = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = lit(input, output, dx, dy, dp);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C lit.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
