
/*C*

________________________________________________________________

        xite_closestMinMax.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/minmax.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus closestMinMax(IBAND in_band, IBAND out_band, int dx, int dy) */

/*M*
function [out_band] = xite_closestMinMax(in_band, dx, dy)
%xite_closestMinMax Closest of minimum and maximum, noise reduction.
%   Calls the XITE C function closestMinMax.
%   function [out_band] = xite_closestMinMax(in_band, dx, dy)
%
%   The C function has the following prototype:
%   BiffStatus closestMinMax(IBAND in_band, IBAND out_band, int dx, int dy)
%
%   and the following manual page:
%
%   NAME
%          closestMinMax - closest of minimum and maximum, noise reduction
%
%   SYNTAX
%           #include <xite/minmax.h>
%
%           BiffStatus closestMinMax( IBAND in_band,
%              IBAND out_band, int dx, int dy );
%
%   DESCRIPTION
%          Compute minimum and maximum over a local region of size dx * dy.  New
%          center pixel value is the one of the two that is closest to the old
%          center pixel value.
%
%   RESTRICTIONS
%          in_band and out_band must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad input pixel type
%           2 => bad output pixel type
%           3 => bad dx argument (less than 3)
%           3 => bad dy argument (less than 3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   EXAMPLES
%          closestMinMax(b1, b2, 3, 5);
%
%   ID
%          $Id: closestMinMax.c,v 1.27 2014/01/31 12:53:48 svein Exp $
%
%                                     27 Feb 2014 CLOSESTMINMAX(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"closestMinMax"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND in_band;
    IBAND out_band;
    int dx;
    int dy;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE in_band_img;
    IMAGE out_band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    in_band_img = Init_image(1, "");
    
    convert_status = matlab2biff(in_band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    in_band = (IBAND) (in_band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) in_band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) in_band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) in_band));
    
    /* If scalar double is passed in for "dx" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        dx = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        dx = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "dy" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        dy = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        dy = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
    out_band_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    out_band = (IBAND) (out_band_img[1]);
    
    /* Execute C routine. */
    return_value = closestMinMax(in_band, out_band, dx, dy);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C closestMinMax.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(out_band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
