
/*C*

________________________________________________________________

        xite_MorphScale.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/morph.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int MorphScale(IBAND band, int neg) */

/*M*
function [band] = xite_MorphScale(band, neg)
%xite_MorphScale Raise or lower morphological structuring element to get
%   maximum or minimum equal to zero.
%   Calls the XITE C function MorphScale.
%   function [band] = xite_MorphScale(band, neg)
%
%   The C function has the following prototype:
%   int MorphScale(IBAND band, int neg)
%
%   and the following manual page:
%
%   NAME
%          MorphScale -  raise or lower morphological structuring element to get
%          maximum or minimum equal to zero
%
%   SYNTAX
%           #include <xite/morph.h>
%
%           void MorphScale( IBAND band, int neg );
%
%   DESCRIPTION
%          On input, it is assumed that the structuring element in band consists
%          of all the non-negative pixels.
%
%          With neg equal to zero, subtract a constant value from the structuring
%          element pixels so that the minimum value inside the structuring element
%          becomes equal to zero.  This means that the height profile will rise
%          from zero.
%
%          With nonzero neg, add a constant value to the structuring element pix-
%          els in band so that the maximum value inside the structuring becomes
%          zero. This means that the top of the height profile touches zero from
%          below.  This is to prevent too large or small values when eroding or
%          dilating an image.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Illegal pixel type
%
%   SEE ALSO
%          mkMorphHeight(3), MorphScale(3), mkMorph(1)
%
%   AUTHOR
%          Martin Torpe Lie
%
%   REVISED
%          Svein Bøe
%
%   DOC
%          Svein Bøe
%
%                                     27 Feb 2014 MORPHSCALE(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"MorphScale"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    int neg;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    
    /* If scalar double is passed in for "neg" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        neg = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        neg = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = MorphScale(band, neg);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C MorphScale.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
