
/*C*

________________________________________________________________

        xite_mkMorphForm.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/morph.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* IBAND mkMorphForm(morph_height_type type, double xsize, double ysize, double theta) */

/*M*
function [return_value] = xite_mkMorphForm(type, xsize, ysize, theta)
%xite_mkMorphForm Make a flat morphological structuring element.
%   Calls the XITE C function mkMorphForm.
%   function [return_value] = xite_mkMorphForm(type, xsize, ysize, theta)
%
%   The C function has the following prototype:
%   IBAND mkMorphForm(morph_height_type type, double xsize, double ysize, double theta)
%
%   and the following manual page:
%
%   NAME
%          mkMorphForm - make a flat morphological structuring element
%
%   SYNTAX
%           #include <xite/morph.h>
%
%           IBAND mkMorphForm( morph_height_type type,
%              double xsize, double ysize, double theta );
%
%   DESCRIPTION
%          Generates and returns a band with an object as defined by type, xsize
%          and ysize.  The resulting band is a square, with horizontal and verti-
%          cal sizes equal to the length of the rectangle’s diagonal,
%           sqrt(2) * MAX(xsize, ysize),
%
%          rounded up to the nearest odd integer.
%
%          The object pixels are set equal to zero, the background pixels are set
%          equal to the minimum for a signed short pixel, typically -32768.
%
%          Use one of the constants SQUARE, ELLIPSE, LINE, CIRCLE, RECTANGLE for
%          type.  These constants are of type morph_height_type.
%
%          For type equal to LINE, ysize is not used. It is hardcoded to a width
%          of 1.
%
%   RETURN VALUE
%          Band pointer.
%
%   SEE ALSO
%          mkMorphHeight(3), MorphScale(3), mkMorph(1)
%
%   AUTHOR
%          Martin Torpe Lie
%
%   REVISED
%          Svein Bøe
%
%   DOC
%          Martin Torpe Lie and Svein Bøe
%
%                                     27 Feb 2014 MKMORPHFORM(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkMorphForm"};
    IBAND return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    morph_height_type type;
    double xsize;
    double ysize;
    double theta;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE return_value_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    type = ((morph_height_type *) mxGetData(prhs[0]))[0];
    xsize = ((double *) mxGetData(prhs[1]))[0];
    ysize = ((double *) mxGetData(prhs[2]))[0];
    theta = ((double *) mxGetData(prhs[3]))[0];
    
    /* Execute C routine. */
    return_value = mkMorphForm(type, xsize, ysize, theta);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    return_value_img = Imake_image(1, "", Ipixtyp((IBAND) return_value),
    			       Ixsize((IBAND) return_value),
    			       Iysize((IBAND) return_value));
    return_value_img[1] = return_value;
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(return_value_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
