
/*C*

________________________________________________________________

        xite_mkMorphHeight.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/morph.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int mkMorphHeight(IBAND band, int type, double xsize, double ysize, int zsize, double theta) */

/*M*
function [band] = xite_mkMorphHeight(band, type, xsize, ysize, zsize, theta)
%xite_mkMorphHeight Make a height on a morphological structuring element.
%   Calls the XITE C function mkMorphHeight.
%   function [band] = xite_mkMorphHeight(band, type, xsize, ysize, zsize, theta)
%
%   The C function has the following prototype:
%   int mkMorphHeight(IBAND band, int type, double xsize, double ysize, int zsize, double theta)
%
%   and the following manual page:
%
%   NAME
%          mkMorphHeight - make a height on a morphological structuring element
%
%   SYNTAX
%           #include <xite/morph.h>
%
%           int mkMorphHeight( IBAND band,
%              int type, double xsize, double ysize,
%              int zsize, double theta );
%
%   DESCRIPTION
%          Modifies the height of a structuring element in band, using a shape as
%          defined by type.  Only the zero-valued pixels in band are modified,
%          they are assumed to be inside the structuring element.
%
%          Use one of the constants ELLIPSE, CIRCLE, PARABOL, CONE, LINEAR for
%          type.  These constants are of type morph_height_type.
%
%   RESTRICTIONS
%          band must be of pixel type signed short.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Illegal pixel type
%
%   SEE ALSO
%          mkMorphForm(3), MorphScale(3), mkMorph(1)
%
%   AUTHOR
%          Martin Torpe Lie
%
%   REVISED
%          Svein Bøe
%
%   DOC
%          Martin Torpe Lie and Svein Bøe
%
%                                     27 Feb 2014 MKMORPHHEIGHT(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkMorphHeight"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    int type;
    double xsize;
    double ysize;
    int zsize;
    double theta;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 6) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 6) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    
    /* If scalar double is passed in for "type" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        type = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        type = ((int *) mxGetData(prhs[1]))[0];
    }
    xsize = ((double *) mxGetData(prhs[2]))[0];
    ysize = ((double *) mxGetData(prhs[3]))[0];
    
    /* If scalar double is passed in for "zsize" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        zsize = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        zsize = ((int *) mxGetData(prhs[4]))[0];
    }
    theta = ((double *) mxGetData(prhs[5]))[0];

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = mkMorphHeight(band, type, xsize, ysize, zsize, theta);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mkMorphHeight.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
