
/*C*

________________________________________________________________

        xite_morphOpen.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/morph.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int morphOpen(IBAND inband, IBAND outband, ISS_BAND B) */

/*M*
function [outband] = xite_morphOpen(inband, B)
%xite_morphOpen Morphological grayscale operations.
%   Calls the XITE C function morphOpen.
%   function [outband] = xite_morphOpen(inband, B)
%
%   The C function has the following prototype:
%   int morphOpen(IBAND inband, IBAND outband, ISS_BAND B)
%
%   and the following manual page:
%
%   NAME
%          morphDilate, morphErode, morphClose, morphOpen -  morphological
%          grayscale operations
%
%   SYNTAX
%           #include <xite/morph.h>
%
%           int morphDilate( IBAND inband, IBAND outband,
%              ISS_BAND B );
%           int morphErode( IBAND inband, IBAND outband,
%              ISS_BAND B );
%           int morphClose( IBAND inband, IBAND outband,
%              ISS_BAND B );
%           int morphOpen( IBAND inband, IBAND outband,
%              ISS_BAND B );
%
%   DESCRIPTION
%          morphDilate calculates the morphological operation dilate on inband,
%          with a structuring element B.
%
%          morphErode calculates the morphological operation erode on inband.
%
%          morphClose calculates the morphological operation close on inband.
%
%          morphOpen calculates the morphological operation open on inband.
%
%          The result of the operations is sent to the band specified in outband.
%
%          The actual calculation for the different operations are described by
%          the expressions below (where i and j are inside the structuring element
%          B, and (0,0) is the center pixel of B).
%
%          dilate outband(x,y) = max[inband(x+i,y+j)+B(i,j)]
%
%          erode outband(x,y) = min[inband(x+i,y+j)-B(-i,-j)]
%
%          close outband = morphErode(morphDilate(inband))
%
%          open outband = morphDilate(morphErode(inband))
%
%            The output values are clipped outside the interval [0, 255]  to make
%            the result fit in pixels of type unsigned byte.
%
%            Be careful not to use too large values in B.  If, e.g. with dilation,
%            inband is binary with the two values 0 and 255, and all the pixels in
%            B equal 255, then all the pixels in outband will equal 255, regard-
%            less of the distribution of zeros in inband.
%
%   RESTRICTIONS
%          B must have pixel type signed short, inband and outband must have pixel
%          type unsigned byte.
%
%   SEE ALSO
%          morphDilate(1), mkMorphForm(3), mkMorphHeight(3), MorphScale(3)
%
%   RETURN VALUE
%           0 :  OK
%           1 :  Bad input pixel type
%           2 :  Bad output pixel type
%           3 :  Bad pixel type for structuring element
%
%   AUTHOR
%          Martin Torpe Lie, BLAB, Ifi, UiO
%
%   ID
%          $Id: morphDilate.c,v 1.30 1998/02/26 10:50:48 svein Exp $
%
%   MORPHOPEN MORPHCLOSE, MORPHDILATE,(MORPHERODE,)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"morphOpen"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    ISS_BAND B;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;
    IMAGE B_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    B_img = Init_image(1, "");
    
    convert_status = matlab2biff(B_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    B = (ISS_BAND) (B_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) B));
    output_ysize = MAX(output_ysize, Iysize((IBAND) B));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) B));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = morphOpen(inband, outband, B);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C morphOpen.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
