
/*C*

________________________________________________________________

        xite_mkGauss.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/noise.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus mkGauss(IBAND b, double my, double sigma) */

/*M*
function [b] = xite_mkGauss(my, sigma)
%xite_mkGauss Make a band with gaussian random noise.
%   Calls the XITE C function mkGauss.
%   function [b] = xite_mkGauss(my, sigma)
%
%   The C function has the following prototype:
%   BiffStatus mkGauss(IBAND b, double my, double sigma)
%
%   and the following manual page:
%
%   NAME
%          mkGauss - make a band with gaussian random noise
%
%   SYNTAX
%           #include <xite/noise.h>
%
%           BiffStatus mkGauss( IBAND b, double my,
%              double sigma );
%
%   DESCRIPTION
%          mkGauss inserts random noise with Gaussian distribution in band b. The
%          distribution has mean value my and standard deviation sigma. If result-
%          ing value is larger than 255 output is set to 255, and if resulting
%          value is less than 0 output is set to 0.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad pixel type
%           2 => Bad my value
%           3 => Bad sigma value
%
%   AUTHOR
%          Tor L|nnestad, BLAB, ifi, UiO
%
%   ID
%          $Id: mkGauss.c,v 1.24 2008/09/25 15:05:17 svein Exp $
%
%                                     27 Feb 2014 MKGAUSS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkGauss"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b;
    double my;
    double sigma;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    my = ((double *) mxGetData(prhs[0]))[0];
    sigma = ((double *) mxGetData(prhs[1]))[0];

    /* Prepare C result variable. */
    b_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b = (IBAND) (b_img[1]);
    
    /* Execute C routine. */
    return_value = mkGauss(b, my, sigma);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mkGauss.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(b_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
