
/*C*

________________________________________________________________

        xite_mkPoisson.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/noise.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int mkPoisson(double alfa, IBAND output) */

/*M*
function [output] = xite_mkPoisson(alfa)
%xite_mkPoisson Make a band with Poisson random noise.
%   Calls the XITE C function mkPoisson.
%   function [output] = xite_mkPoisson(alfa)
%
%   The C function has the following prototype:
%   int mkPoisson(double alfa, IBAND output)
%
%   and the following manual page:
%
%   NAME
%          mkPoisson - make a band with Poisson random noise.
%
%   SYNTAX
%           #include <xite/noise.h>
%
%           int mkPoisson( double alfa, IBAND output );
%
%   DESCRIPTION
%          mkPoisson inserts random noise with Poisson distribution in band out-
%          put. The distribution has mean value alfa and variance alfa.   If
%          resulting value is larger than 255, output is set to 255, and if
%          resulting value is less than 0, output is set to 0.
%
%   RESTRICTIONS
%          Accepts only unsigend byte pixeltype.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad pixel type
%
%   REFERENCE
%          J. Banks: Discrete-event system simulation.
%
%   AUTHOR
%          Helene Schulerud, BLAB, Ifi, UiO
%
%   ID
%          $Id: mkPoisson.c,v 1.22 2002/02/12 13:44:25 svein Exp $
%
%                                     27 Feb 2014 MKPOISSON(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkPoisson"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    double alfa;
    IBAND output;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    alfa = ((double *) mxGetData(prhs[0]))[0];

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = mkPoisson(alfa, output);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mkPoisson.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
