
/*C*

________________________________________________________________

        xite_linearTrans.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/principal.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus linearTrans(IMAGE input, IMAGE output, IR_BAND weights) */

/*M*
function [output] = xite_linearTrans(input, weights)
%xite_linearTrans Linear image to image transformation.
%   Calls the XITE C function linearTrans.
%   function [output] = xite_linearTrans(input, weights)
%
%   The C function has the following prototype:
%   BiffStatus linearTrans(IMAGE input, IMAGE output, IR_BAND weights)
%
%   and the following manual page:
%
%   NAME
%          linearTrans - Linear image to image transformation
%
%   SYNTAX
%           #include <xite/principal.h>
%
%           BiffStatus linearTrans( IMAGE input,
%              IMAGE output, IR_BAND weights );
%
%   DESCRIPTION
%          linearTrans performs a linear transformation from pixels in the input
%          image to pixels in the output image,
%           Pout = W*Pin
%
%          where W is the weight matrix weights, which should be of size
%          nbands*nbands, where nbands is the number of bands in input and output
%          images.
%
%   RESTRICTIONS
%          All bands in input and output must have pixel type unsigned byte.
%
%   RETURN VALUE
%          0 => ok 1 => bad pixel type in input image 2 => bad pixel type in out-
%          put image
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: linearTrans.c,v 1.29 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 LINEARTRANS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"linearTrans"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IMAGE input;
    IMAGE output;
    IR_BAND weights;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE weights_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    if (mxIsStruct(prhs[0])) {
      input = Init_image(mxGetNumberOfFields(prhs[0]), "");
    } else {
      input = Init_image(1, "");
    }
    convert_status = matlab2biff(input, NULL, 1,
                                 &prhs[0], NULL, NULL);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input[1]));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input[1]));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input[1]));
    weights_img = Init_image(1, "");
    
    convert_status = matlab2biff(weights_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    weights = (IR_BAND) (weights_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) weights));
    output_ysize = MAX(output_ysize, Iysize((IBAND) weights));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) weights));

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = linearTrans(input, output, weights);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C linearTrans.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status = 
            biff2matlab((IMAGE) output, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
