
/*C*

________________________________________________________________

        xite_pyramidRank2.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/pyramid.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int pyramidRank2(IBAND b1, IBAND b2, int rank) */

/*M*
function [b2] = xite_pyramidRank2(b1, rank)
%xite_pyramidRank2 Create a pyramidal raster data structure.
%   Calls the XITE C function pyramidRank2.
%   function [b2] = xite_pyramidRank2(b1, rank)
%
%   The C function has the following prototype:
%   int pyramidRank2(IBAND b1, IBAND b2, int rank)
%
%   and the following manual page:
%
%   NAME
%          pyramid, pyramidAver2, pyramidRank2, pyramidGauss5, mkPyramidAver2,
%          mkPyramidRank2, mkPyramidGauss5 -  create a pyramidal raster data
%          structure
%
%   SYNTAX
%           #include <xite/pyramid.h>
%
%           int pyramidAver2( IBAND b1, IBAND b2 );
%
%           int pyramidRank2( IBAND b1, IBAND b2,
%              int rank );
%
%           int pyramidGauss5( IBAND b1, IBAND b2,
%              ISS_BAND conv );
%
%           int mkPyramidAver2( IBAND b, IMAGE* i, int n );
%
%           int mkPyramidRank2( IBAND b, IMAGE* i,
%              int rank, int n );
%
%           int mkPyramidGauss5( IBAND b, IMAGE* i,
%              ISS_BAND conv, int n );
%
%   DESCRIPTION
%          pyramidAver2 performs a 2 by 2 average of pixels in b1 and places the
%          result in b2.  The 2 by 2 window is then moved two positions, and the
%          process is repeated over the whole band.  b2 should thus be half the
%          size of b1 vertically as well as horizontally.   If this is not the
%          case, only pixels occurring in b2, and with four "subpixels" in b1, are
%          handled.  The final division by four will always truncate.
%
%          pyramidRank2 sorts the four pixel values in a 2 by 2 window in b1,
%          looks for the rank sorted sequence, and puts this single value into b2.
%          The 2 by 2 window is then moved two positions, and the process is
%          repeated over the whole band.  b2 should thus be half the size of b1
%          vertically as well as horizontally. If this is not the case, only pix-
%          els occurring in b2, and with four "subpixels" in b1, are handled.
%
%          pyramidGauss5 performs a 5 by 5 gaussian filtering of pixels in b1 with
%          the filter conv and subsampels the image by 2. The result is placed in
%          b2.   b2 should thus be half the size of b1 vertically as well as hori-
%          zontally.
%
%          mkPyramidAver2 creates a pyramid data structure with n levels from the
%          input band b and returns the result as the output image i.  The bottom
%          layer will have the same size as the input band rounded down to nearest
%          even, and each layer upwards the pyramid will have half the size of the
%          previous (integer division by 2). The routine uses pyramidAver2 repeat-
%          edly. With n equal to zero, use the maximum number of levels.
%
%          mkPyramideRank2 creates a pyramid data structure with n levels from the
%          input band b and returns the result as the output image i.  The bottom
%          layer will have the same size as the input band rounded down to nearest
%          even, and ach layer upwards the pyramid will have half the size of the
%          previous (integer division by 2). From each 2 by 2 window in one layer,
%          the smallest (rank=1), second smallest (rank=2), second largest
%          (rank=3)  or largest (rank=4) pixel value will be inserted in the layer
%          above. The rank parameter should thus be in 1..4.  The routine uses
%          pyramidRank2 repeatedly.  With n equal to zero, use the maximum number
%          of levels.
%
%          mkPyramidGauss5 creates a pyramid data structure with n levels from the
%          input band b and returns the result as the output image i.  The bottom
%          layer will have the same size as the input band rounded down to nearest
%          even, and each layer upwards the pyramid will have half the size of the
%          previous (integer division by 2).  The routine uses pyramidGauss5
%          repeatedly. With n equal to zero, use the maximum number of levels.
%
%   RESTRICTIONS
%          Input and output pixel types must be unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type b1
%           2 => bad pixel type b2
%           3 => bad rank value (outside 1..4)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, ifi, UiO.
%
%   REVISED
%          Helene Schulerud, BLAB, ifi, UiO.
%
%   ID
%          $Id: pyramid.c,v 1.32 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 PYRAMID(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"pyramidRank2"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b1;
    IBAND b2;
    int rank;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b1_img;
    IMAGE b2_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    b1_img = Init_image(1, "");
    
    convert_status = matlab2biff(b1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    b1 = (IBAND) (b1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b1));
    
    /* If scalar double is passed in for "rank" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        rank = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        rank = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    b2_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b2 = (IBAND) (b2_img[1]);
    
    /* Execute C routine. */
    return_value = pyramidRank2(b1, b2, rank);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C pyramidRank2.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(b2_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
