
/*C*

________________________________________________________________

        xite_reclassifyBoundary.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/segmentation.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus reclassifyBoundary(IBAND orig, IBAND result, IBAND boundary, histogram map) */

/*M*
function [result] = xite_reclassifyBoundary(orig, boundary, map)
%xite_reclassifyBoundary Reclassify the boundary pixels.
%   Calls the XITE C function reclassifyBoundary.
%   function [result] = xite_reclassifyBoundary(orig, boundary, map)
%
%   The C function has the following prototype:
%   BiffStatus reclassifyBoundary(IBAND orig, IBAND result, IBAND boundary, histogram map)
%
%   and the following manual page:
%
%   NAME
%          reclassifyBoundary - reclassify the boundary pixels
%
%   SYNTAX
%           #include <xite/segmentation.h>
%
%           BiffStatus reclassifyBoundary( IBAND orig,
%              IBAND result, IBAND boundary,
%              histogram map );
%
%   DESCRIPTION
%          reclassify the boundary pixels.  boundary describes what is boundary
%          pixels, orig have the original pixel values, map is the classification
%          map, and the result goes into result.
%
%   RESTRICTIONS
%          orig, result and boundary must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Bad input pixel type
%           2 : Bad result pixel type
%           3 : Bad boundary pixel type
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%                                     27 Feb 2014 RECLASSIFYBOUNDARY(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"reclassifyBoundary"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND orig;
    IBAND result;
    IBAND boundary;
    histogram map;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE orig_img;
    IMAGE result_img;
    IMAGE boundary_img;
    typedef int * histogram_ptr;
    histogram_ptr map_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    orig_img = Init_image(1, "");
    
    convert_status = matlab2biff(orig_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    orig = (IBAND) (orig_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) orig));
    output_ysize = MAX(output_ysize, Iysize((IBAND) orig));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) orig));
    boundary_img = Init_image(1, "");
    
    convert_status = matlab2biff(boundary_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    boundary = (IBAND) (boundary_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) boundary));
    output_ysize = MAX(output_ysize, Iysize((IBAND) boundary));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) boundary));
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[2]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[2]) == 256 && mxGetN(prhs[2]) == 1) ||
          (mxGetN(prhs[2]) == 256 && mxGetM(prhs[2]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    map_ptr = ((histogram_ptr) mxGetData(prhs[2]));
    memcpy(map, map_ptr, sizeof(int) * 256);

    /* Prepare C result variable. */
    result_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    result = (IBAND) (result_img[1]);
    
    /* Execute C routine. */
    return_value = reclassifyBoundary(orig, result, boundary, map);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C reclassifyBoundary.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(result_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
