
/*C*

________________________________________________________________

        xite_segmRandom.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/segmentation.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int segmRandom(IBAND band, double prob[3][2], int lpv, int rpv) */

/*M*
function [band] = xite_segmRandom(band, prob, lpv, rpv)
%xite_segmRandom Draw two regions, random border.
%   Calls the XITE C function segmRandom.
%   function [band] = xite_segmRandom(band, prob, lpv, rpv)
%
%   The C function has the following prototype:
%   int segmRandom(IBAND band, double prob[3][2], int lpv, int rpv)
%
%   and the following manual page:
%
%   NAME
%          segmRandom - draw two regions, random border
%
%   SYNTAX
%           #include <xite/segmentation.h>
%
%           int segmRandom( IBAND band, double prob[3][2],
%              int lpv, int rpv );
%
%   DESCRIPTION
%          band is altered into a band with two regions with value lpv (left pixel
%          value) on the left part, and rpv on the right.  The border separating
%          the regions is made with a kind of random walk, or 2. order markov pro-
%          cess, starting in the center of the upper line.  In each step, the bor-
%          der may go down-left (here denoted left), down, or down-right (right).
%
%          The probablity of going in each of the three directions left, down and
%          right, are given in the table prob.  The three rows in the table indi-
%          cate the previous direction of the walk. The two columns in the table
%          is the probability for going left and right. The probability for going
%          down is computed, because the total probability always is equal one.
%
%          The table prob:
%             prob[0][0] = p(left | left)
%             prob[1][0] = p(left | down)
%             prob[2][0] = p(left | rigth)
%             prob[0][1] = p(right | left)
%             prob[1][1] = p(right | down)
%             prob[2][1] = p(right | right)
%
%   RESTRICTIONS
%          Accepts only UNS_BYTE pixles.
%
%   RETURN VALUE
%           0 : OK
%           2 : Bad pixle type
%           3 : Bad probabilities
%           4 : Bad pixel values
%
%   AUTHOR
%          Martin Torpe Lie, BLAB, Ifi, UiO
%
%   ID
%          $Id: segmRandom.c,v 1.18 1998/02/26 10:53:28 svein Exp $
%
%                                     27 Feb 2014 SEGMRANDOM(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"segmRandom"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    double **prob;
    int lpv;
    int rpv;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    prob = ((double ***) mxGetData(prhs[1]))[0];
    
    /* If scalar double is passed in for "lpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        lpv = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        lpv = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "rpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        rpv = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        rpv = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = segmRandom(band, prob, lpv, rpv);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C segmRandom.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(band_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
