
/*C*

________________________________________________________________

        xite_segmSpannWilson.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/segmentation.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* BiffStatus segmSpannWilson(IBAND inband, IBAND outband, int n, int m, int method) */

/*M*
function [outband] = xite_segmSpannWilson(inband, n, m, method)
%xite_segmSpannWilson Segmentation by the Spann and Wilson method.
%   Calls the XITE C function segmSpannWilson.
%   function [outband] = xite_segmSpannWilson(inband, n, m, method)
%
%   The C function has the following prototype:
%   BiffStatus segmSpannWilson(IBAND inband, IBAND outband, int n, int m, int method)
%
%   and the following manual page:
%
%   NAME
%          segmSpannWilson - segmentation by the Spann and Wilson method
%
%   SYNTAX
%           #include <xite/segmentation.h>
%
%           BiffStatus segmSpannWilson( IBAND inband,
%              IBAND outband, int n, int m, int method );
%
%   DESCRIPTION
%          Performs segmentation of the input band into the output band. The algo-
%          rithm has three steps:
%
%          Quad--tree smoothing
%
%                 A quad-tree is created, and pixels are averaged 4 by 4 up in the
%                 quad-tree.  n quad-tree levels are created, including the bottom
%                 level, which is inband.
%
%          Local centroid clustering
%
%                 The topmost quad-tree level is clustered by moving histogram
%                 mass to the centroid (mean or median) of a local window moved
%                 over the histogram.  Applied iteratively, until stable.  See
%                 thresMlCentroid.
%
%          Boundary estimation
%
%                 The classification result is copied down the quad-tree, but at
%                 every level, all pixels close to a differently classified pixel
%                 are reclassified to determine the border more precisely.
%
%   ARGUMENTS
%          inband Input band to be segmented.
%
%          outband
%                 The result of the segmentation.
%
%          n Levels in the quad-tree (must be at least two).
%
%          m Size of histogram window, like 10-20.
%
%          method Local centriod clustering: 0 -> mean, 1 -> median.
%
%            Note that if you specify histogram window size m =0, the algorithm
%            will try to find an m -value where the clustering result is rela-
%            tively stable over a range of m -values.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.  outband should
%          have the same size as inband.  Only the largest common rectangle of
%          inband and outband is handled.
%
%   SEE ALSO
%          segmSpannWilson(1), thresMlCentroid(3), segmRandom(3)
%
%   REFERENCE
%          M.. Spann and R.. Wilson,,
%                 "A Quad-Tree Approach to Image Segmentation which Combines Sta-
%                 tistical and Spatial Information" Patteren Recognition, Vol 18,
%                 Nos 3/4, pp 257-269, 1985
%
%   RETURN VALUE
%           0 => ok
%           2 => error: bad inband pixel type
%           3 => error: bad outband pixel type
%           4 => error: bad n value
%           5 => error: bad m value
%           6 => error: bad method value
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: segmSpannWilson.c,v 1.27 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 SEGMSPANNWILSON(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"segmSpannWilson"};
    BiffStatus return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int n;
    int m;
    int method;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "n" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        n = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        n = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "m" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        m = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        m = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "method" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        method = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        method = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = segmSpannWilson(inband, outband, n, m, method);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C segmSpannWilson.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
