
/*C*

________________________________________________________________

        xite_band_minmax_disp.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/statistics.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int band_minmax_disp(IBAND band, float *minval, float *maxval) */

/*M*
function [return_value] = xite_band_minmax_disp(band, minval, maxval)
%xite_band_minmax_disp Find minimum and maximum in a BIFF image band.
%   Calls the XITE C function band_minmax_disp.
%   function [return_value] = xite_band_minmax_disp(band, minval, maxval)
%
%   The C function has the following prototype:
%   int band_minmax_disp(IBAND band, float *minval, float *maxval)
%
%   and the following manual page:
%
%   NAME
%          band_minmax, band_minmax_disp -  find minimum and maximum in a BIFF
%          image band
%
%   SYNTAX
%           #include <xite/statistics.h>
%
%           int band_minmax( IBAND band, double* minval,
%              double* maxval );
%
%           int band_minmax_disp( IBAND band, double *minval,
%              double *maxval );
%
%   DESCRIPTION
%          band_minmax finds minval (minimum value) and maxval (maximum value)  in
%          band by calling statistic(3).
%
%          band_minmax_disp is a wrapper around band_minmax.  It returns minval
%          and maxval so that they may be used as XtNminval and XtNmaxval
%          resources for the Image and ImageOverlay widgets.
%
%   RESTRICTIONS
%          band should not be complex-valued. In this case, the return value of
%          both functions equals 2.  band_minmax issues an error message by call-
%          ing Error, while band_minmax_disp sets minval and maxval equal to zero.
%
%   RETURN VALUE
%           0 - Ok
%           2 - Complex-valued band
%
%   SEE ALSO
%          statistics(3)
%
%   AUTHOR
%          Otto Milvang and Svein Bøe
%
%   REVISED
%          Svein Bøe
%
%                                     27 Feb 2014 BAND_MINMAX(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"band_minmax_disp"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    float *minval;
    float *maxval;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    minval = ((float **) mxGetData(prhs[1]))[0];
    maxval = ((float **) mxGetData(prhs[2]))[0];
    
    /* Execute C routine. */
    return_value = band_minmax_disp(band, minval, maxval);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
