
/*C*

________________________________________________________________

        xite_statisticMasks.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/statistics.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int statisticMasks(IBAND band, IBAND mask, long *size, double *min, double *max, double *mean, double *stdev) */

/*M*
function [size, min, max, mean, stdev] = xite_statisticMasks(band, mask)
%xite_statisticMasks Extract statistical information from a band.
%   Calls the XITE C function statisticMasks.
%   function [size, min, max, mean, stdev] = xite_statisticMasks(band, mask)
%
%   The C function has the following prototype:
%   int statisticMasks(IBAND band, IBAND mask, long *size, double *min, double *max, double *mean, double *stdev)
%
%   and the following manual page:
%
%   NAME
%          statistics, statistic, cstatistic, hstatistics, statisticMask, statis-
%          ticMasks - extract statistical information from a band
%
%   SYNTAX
%           #include <xite/statistics.h>
%
%           int statistics( IBAND band, int* min, int* max,
%              int* median, double* mean, double* stdev );
%
%           int hstatistics( histogram histo, int* min,
%              int* max, int* median, double* mean,
%              double* stdev );
%
%           int statistic( IBAND band, long* size,
%              double* min, double* max, double* mean,
%              double* stdev );
%
%           int cstatistic( IBAND band, long* size,
%              double* min, double* max, DCOMPLEX* mean,
%              double* stdev );
%
%           int statisticMask( IBAND band, IBAND mask,
%              long* size, double* min, double* max,
%              double* mean, double* stdev, int maskval );
%
%           int statisticMasks( IBAND band, IBAND mask,
%              long* size, double* min, double* max,
%              double* mean, double* stdev );
%
%   DESCRIPTION
%          hstatistics calculates min (minimum value), max (maximum value),
%          median, mean and stdev (standard deviation) in a band from the his-
%          togram histo.
%
%          statistics calculates min, max, median, mean and standard deviation in
%          band, by first producing the histogram and sending it to hstatistics.
%          band must have pixel type unsigned byte.
%
%          statistic calculates size (number of elements), min, max, mean and
%          stdev (standard deviation)  for band.   band must have a non-complex
%          pixel type.  By passing NULL for min and max, they are not calculated.
%          By passing NULL for mean and stdev, they are not calculated. If any of
%          these four parameters are passed as NULL, they are not changed upon
%          return (even if they were calculated). Make sure to set those parame-
%          ters who should be calculated and returned to non-NULL.
%
%          cstatistic calculates the same statistics for a band with pixel type
%          complex or double complex.  In this case min is the minimum absolute
%          value, max is the maximum absolute value, and mean is complex.  The
%          interpretation of input values for the parameters min, max, mean and
%          stdev is the same as for statistic.
%
%          statisticMask calculates the same statistics for all pixels where
%          mask[y,x] == maskval. Refer to statistic for meaning of input values.
%
%          statisticMasks calculates the same statistics for all pixels and all
%          mask values.  sizearr, minarr, maxarr, meanarr and stdevarr are arrays
%          of 256 elements. Refer to statistic for meaning of input values.
%
%          One or more of the parameters min, minarr, max, maxarr, mean, meanarr,
%          stdev and stdevarr may be NULL.
%
%   RESTRICTIONS
%          statistics only works for bands with unsigned byte pixels.   statistic,
%          statisticMask and statisticMasks only work for non-complex bands.
%          cstatistic only works for complex or double complex bands.
%
%          mask must be of type unsigned byte and have the same size as band.
%
%   RETURN VALUE
%             0 - Ok.
%             1 - Mask is not same size as band.
%             2 - Wrong pixel type
%
%   SEE ALSO
%          statistics(1)
%
%   AUTHOR
%          Tor Lønnestad, Otto Milvang and Svein Bøe, BLAB, Ifi, UiO
%
%   ID
%          $Id: statistics.c,v 1.54 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 STATISTICS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"statisticMasks"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    IBAND mask;
    long *size;
    double *min;
    double *max;
    double *mean;
    double *stdev;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;
    IMAGE mask_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 5) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    mask_img = Init_image(1, "");
    
    convert_status = matlab2biff(mask_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    mask = (IBAND) (mask_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) mask));
    output_ysize = MAX(output_ysize, Iysize((IBAND) mask));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) mask));

    /* Prepare C result variable. */
    size = (long *) mxCalloc(256, sizeof(long));
    min = (double *) mxCalloc(256, sizeof(double));
    max = (double *) mxCalloc(256, sizeof(double));
    mean = (double *) mxCalloc(256, sizeof(double));
    stdev = (double *) mxCalloc(256, sizeof(double));
    
    /* Execute C routine. */
    return_value = statisticMasks(band, mask, size, min, max, mean, stdev);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C statisticMasks.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(256, 1, mxINT64_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), size, (256) * sizeof(long));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }
    if (nlhs > 1) {
        tmp_plhs[1] =    mxCreateNumericMatrix(256, 1, mxDOUBLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[1]), min, (256) * sizeof(double));
        plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =    mxCreateNumericMatrix(256, 1, mxDOUBLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[2]), max, (256) * sizeof(double));
        plhs[2] = tmp_plhs[2];
    }
    if (nlhs > 3) {
        tmp_plhs[3] =    mxCreateNumericMatrix(256, 1, mxDOUBLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[3]), mean, (256) * sizeof(double));
        plhs[3] = tmp_plhs[3];
    }
    if (nlhs > 4) {
        tmp_plhs[4] =    mxCreateNumericMatrix(256, 1, mxDOUBLE_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[4]), stdev, (256) * sizeof(double));
        plhs[4] = tmp_plhs[4];
    }

    PopMessage();

    return;
}
