
/*C*

________________________________________________________________

        xite_maxSimilar3x3.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/stdiff.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int maxSimilar3x3(IBAND input, IBAND output) */

/*M*
function [output] = xite_maxSimilar3x3(input)
%xite_maxSimilar3x3 Maximum similarity 3x3 noise reduction.
%   Calls the XITE C function maxSimilar3x3.
%   function [output] = xite_maxSimilar3x3(input)
%
%   The C function has the following prototype:
%   int maxSimilar3x3(IBAND input, IBAND output)
%
%   and the following manual page:
%
%   NAME
%          maxSimilar3x3 - maximum similarity 3x3 noise reduction
%
%   SYNTAX
%           #include <xite/stdiff.h>
%
%           int maxSimilar3x3( IBAND input, IBAND output );
%
%   DESCRIPTION
%          For every pixel p, calculate the five 3x3 averages:
%
%           x x x x x x x x x x x p p x x
%           x x x x x x x p x x x x x x x
%           x x p p x x x x x x x x x x x
%
%          Replace p in the output band with the average that is closest to p. The
%          1 pixel wide edge of the band is copied from input without modifica-
%          tion. The next line/sample is partly processed.
%
%   RESTRICTIONS
%          Bands must have pixel type UNS_BYTE.
%
%   REFERENCE
%          W. Niblack: "Digital Image Processing", Prentice/Hall, p80.
%
%   SEE ALSO
%   RETURN VALUE
%           0 => ok
%           1 => Bad input pixel type
%           2 => Bad output pixel type
%           3 => Malloc failed
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO.
%
%   ID
%          $Id: maxSimilar3x3.c,v 1.24 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 MAXSIMILAR3X3(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"maxSimilar3x3"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = maxSimilar3x3(input, output);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C maxSimilar3x3.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
