
/*C*

________________________________________________________________

        xite_stdiff.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/stdiff.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int stdiff(IBAND input, IBAND output, int deltax, int deltay, double alpha, double beta, double m0, double s0) */

/*M*
function [output] = xite_stdiff(input, deltax, deltay, alpha, beta, m0, s0)
%xite_stdiff Local statistical differencing.
%   Calls the XITE C function stdiff.
%   function [output] = xite_stdiff(input, deltax, deltay, alpha, beta, m0, s0)
%
%   The C function has the following prototype:
%   int stdiff(IBAND input, IBAND output, int deltax, int deltay, double alpha, double beta, double m0, double s0)
%
%   and the following manual page:
%
%   NAME
%          stdiff - local statistical differencing
%
%   SYNTAX
%           #include <xite/stdiff.h>
%
%           void stdiff( IBAND input,
%              IBAND output, int deltax,
%              int deltay, double alpha, double beta,
%              double m0, double s0 );
%
%   DESCRIPTION
%          Local Pixel Value Mapping.
%
%           r(x,y) = u(x,y) + s(x,y)
%           u(x,y) = alpha * m0 + (1-alpha) * bm(x,y)
%           s(x,y) = (b(x,y)-bm(x,y)) * (beta*s0 / (s0+bs(x,y)))
%
%           r(x,y)  = result image
%           u(x,y)  = mean component
%           s(x,y)  = st. dev component
%           m0 = The mean to force locally
%           alpha = fraction m0 <--> local mean
%           b(b,y)  = pixel value in (x,y)
%           bm(x,y) = local mean
%           bs(x,y) = local st. dev
%           s0 = new st. dev
%           beta = parameter to control s0 <--> local st.dev
%
%          deltax is local area width, deltay is local area height.
%
%   RESTRICTIONS
%          input and output must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 : Ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%
%   REFERENCE
%          Wayne Niblack: An introduction to Digital Image Processing
%
%   AUTHOR
%          Otto Milvang
%
%                                     27 Feb 2014 STDIFF(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"stdiff"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    IBAND output;
    int deltax;
    int deltay;
    double alpha;
    double beta;
    double m0;
    double s0;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "deltax" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        deltax = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        deltax = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "deltay" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        deltay = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        deltay = ((int *) mxGetData(prhs[2]))[0];
    }
    alpha = ((double *) mxGetData(prhs[3]))[0];
    beta = ((double *) mxGetData(prhs[4]))[0];
    m0 = ((double *) mxGetData(prhs[5]))[0];
    s0 = ((double *) mxGetData(prhs[6]))[0];

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (IBAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = stdiff(input, output, deltax, deltay, alpha, beta, m0, s0);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C stdiff.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
