
/*C*

________________________________________________________________

        xite_glcm.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/texture.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int glcm(IBAND input, ID_BAND output, int dx, int dy, int nogreylevel) */

/*M*
function [output] = xite_glcm(input, dx, dy, nogreylevel)
%xite_glcm Computes Grey Level Cooccurrence Matrix.
%   Calls the XITE C function glcm.
%   function [output] = xite_glcm(input, dx, dy, nogreylevel)
%
%   The C function has the following prototype:
%   int glcm(IBAND input, ID_BAND output, int dx, int dy, int nogreylevel)
%
%   and the following manual page:
%
%   NAME
%          glcm - Computes Grey Level Cooccurrence Matrix
%
%   SYNTAX
%           #include <xite/texture.h>
%
%           int glcm( IBAND input, ID_BAND output, int dx,
%              int dy, int num_greylevel );
%
%   DESCRIPTION
%          Computes the Grey Level Cooccurence Matrix of an image. The angle and
%          the displacement will be computed from the given dx and dy.  dx and dy
%          can be negative.
%
%          All the pixel values in the input band will be increased by one prior
%          to computing the GLCM (though not changing the contents of the input
%          band). This means that if there is a greylevel combination (0,p) in the
%          input band, then the value at (1,p+1)  in the output band will be
%          increased.
%
%          Note that the number of gray levels is not set to num_graylevel.  This
%          parameter is merely used to check whether input contains too large
%          pixel values, and a warning is given to this effect. If a reduction of
%          the number of gray levels is desired, this should be taken care of
%          prior to calling this function.
%
%   RESTRICTIONS
%          input must have pixel type unsigned byte.  output must have pixel type
%          double.
%
%   RETURN VALUE
%           0 => ok
%           Warning:
%             1 => there are grey levels > "num_greylevel" in
%                  the input band
%           Errors:
%             2 => input is not unsigned byte
%             3 => output is not double type
%             4 => horizontal size of output is less than
%                  "num_greylevel"
%             5 => vertical size of output is less than
%                  "num_greylevel"
%
%   REFERENCE
%          R.. M.. Haralick,, K.. Shanmugam,, and I.. Dinstein
%                 "Textural Features for Image Classification", IEEE Trans.  on
%                 Systems, Man and Cybernetics, Vol. SMC-3, pp. 610-621, 1973.
%
%   AUTHOR
%          Yogesan K., BLAB, Ifi, UiO
%
%   ID
%          $Id: glcm.c,v 1.22 1998/12/18 10:20:43 svein Exp $
%
%                                     27 Feb 2014 GLCM(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"glcm"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND input;
    ID_BAND output;
    int dx;
    int dy;
    int nogreylevel;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE input_img;
    IMAGE output_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    input_img = Init_image(1, "");
    
    convert_status = matlab2biff(input_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    input = (IBAND) (input_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) input));
    output_ysize = MAX(output_ysize, Iysize((IBAND) input));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) input));
    
    /* If scalar double is passed in for "dx" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        dx = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        dx = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "dy" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        dy = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        dy = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "nogreylevel" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        nogreylevel = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        nogreylevel = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    output_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    output = (ID_BAND) (output_img[1]);
    
    /* Execute C routine. */
    return_value = glcm(input, output, dx, dy, nogreylevel);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C glcm.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(output_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
