
/*C*

________________________________________________________________

        xite_glcmProm.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/texture.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* double glcmProm(ID_BAND inband) */

/*M*
function [return_value] = xite_glcmProm(inband)
%xite_glcmProm Computes features from the Grey Level Co-occurrence Matrix
%   (GLCM).
%   Calls the XITE C function glcmProm.
%   function [return_value] = xite_glcmProm(inband)
%
%   The C function has the following prototype:
%   double glcmProm(ID_BAND inband)
%
%   and the following manual page:
%
%   NAME
%          glcmParameter, glcmAsm, glcmCont, glcmCorr, glcmIdm, glcmEntropy, glcm-
%          Var, glcmProm, glcmShade, glcmInertia, glcmDiagmoment -  Computes fea-
%          tures from the Grey Level Co-occurrence Matrix (GLCM)
%
%   SYNTAX
%           #include <xite/texture.h>
%
%           double glcmAsm( ID_BAND inband );
%
%           double glcmCont( ID_BAND inband );
%
%           double glcmCorr( ID_BAND inband );
%
%           double glcmIdm( ID_BAND inband );
%
%           double glcmEntropy( ID_BAND inband );
%
%           double glcmVar( ID_BAND inband );
%
%           double glcmProm( ID_BAND inband );
%
%           double glcmShade( ID_BAND inband );
%
%           double glcmInertia( ID_BAND inband );
%
%           double glcmDiagmoment( ID_BAND inband );
%
%   DESCRIPTION
%          The input parameters are normalized bands.
%
%          glcmAsm
%                 Calculates Angular Second Moment of a GLCM. See reference 1.
%
%          glcmCont
%                 Calculates Contrast of a GLCM. See reference 3.
%
%          glcmCorr
%                 Calculates Correlation of a GLCM. See reference 1.
%
%          glcmIdm
%                 Calculates Inverse Difference Moment of a GLCM. See reference 1.
%
%          glcmEntropy
%                 Calculates Entropy of a GLCM. See reference 1.
%
%          glcmVar
%                 Calculates Variance of a GLCM. See reference 1.
%
%          glcmProm
%                 Calculates Cluster Prominence of a GLCM. See reference 2.
%
%          glcmShade
%                 Calculates Cluster Shade of a GLCM. See reference 2.
%
%          glcmInertia
%                 Calculates Inertia of a GLCM. See reference 2.
%
%          glcmDiagmoment
%                 Calculates Diagonal moment of a GLCM.
%
%   RESTRICTIONS
%          inband must have pixel type double.
%
%   SEE ALSO
%          glcm(3)
%
%   REFERENCE
%          R.. M.. Haralick et al..
%                 "Textural Features for Image Classification", IEEE Trans.  on
%                 Systems, Man and Cybernetics, Vol. SMC-3, pp. 610-621, 1973.
%
%          R.. W.. Conners et al..
%                 "Segmentation of a High-Resolution Urban Scene Using Texture
%                 Operators", Computer Vision, Graphics and Image Processing, Vol.
%                 25, pp. 273-310, 1984.
%
%          C.. C.. Gotlieb et al..
%                 "Texture Descriptors Based on Co-ooccurrence Matrices", Computer
%                 Vision, Graphics and Image Processing, Vol. 51, pp. 70-86, 1990.
%
%   RETURN VALUE
%          GLCM-Parameter value (double)
%
%   AUTHOR
%          Yogesan, BLAB, Ifi, UiO
%
%   ID
%          $Id: glcmParameter.c,v 1.31 2007/09/14 09:49:47 svein Exp $
%
%                                     27 Feb 2014 GLCMPARAMETER(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"glcmProm"};
    double return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    ID_BAND inband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (ID_BAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* Execute C routine. */
    return_value = glcmProm(inband);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
    ((double *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
