
/*C*

________________________________________________________________

        xite_glrlSgre.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/texture.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* double glrlSgre(II_BAND inband, int maxlength, int noofruns) */

/*M*
function [return_value] = xite_glrlSgre(inband, maxlength, noofruns)
%xite_glrlSgre Extract grey level run length parameters.
%   Calls the XITE C function glrlSgre.
%   function [return_value] = xite_glrlSgre(inband, maxlength, noofruns)
%
%   The C function has the following prototype:
%   double glrlSgre(II_BAND inband, int maxlength, int noofruns)
%
%   and the following manual page:
%
%   NAME
%          glrlParameters, glrlSre, glrlLre, glrlGln, glrlRln, glrlRp, glrlSgre,
%          glrlLgre - extract grey level run length parameters
%
%   SYNTAX
%           #include <xite/texture.h>
%
%           double glrlSre( II_BAND inband, int maxlength,
%              int noofruns );
%           double glrlLre( II_BAND inband, int maxlength,
%              int noofruns );
%           double glrlGln( II_BAND inband, int maxlength,
%              int noofruns );
%           double glrlRp( II_BAND inband, int maxlength,
%              int noofruns, int noofpixels );
%           double glrlRln( II_BAND inband, int maxlength,
%              int noofruns );
%           double glrlSgre( II_BAND inband, int maxlength,
%              int noofruns );
%           double glrlLgre( II_BAND inband, int maxlength,
%              int noofruns );
%
%   DESCRIPTION
%          glrlSre
%                 Short Run emphasis
%
%          glrllre
%                 Long Run emphasis
%
%          glrlgln
%                 Grey Level Nonuniformity
%
%          glrlrln
%                 Run Length Nonuniformity
%
%          glrlrp Run Percentage
%
%          glrlsgre
%                 Low Grey Level Run emphasis
%
%          glrllgre
%                 High Grey Level Run emphasisglrlSre
%
%   RESTRICTIONS
%          inband must have pixel type integer.
%
%   RETURN VALUE
%           -1 => bad pixel value in input band
%
%   REFERENCE
%           A. Chu et al.
%           "Use of grey value distribution of run lengths for texture
%           analysis"
%           Pattern Recognition Letters - 11
%           June 1990 - 415-420.
%
%   SEE ALSO
%          glcmParameter(3), glRunLength(3)
%
%   AUTHOR
%          Yogesan, BLAB, Ifi, UiO
%
%   ID
%          $Id: glRunLength.c,v 1.30 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 GLRLPARAMETERS(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"glrlSgre"};
    double return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    II_BAND inband;
    int maxlength;
    int noofruns;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (II_BAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "maxlength" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        maxlength = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        maxlength = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "noofruns" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        noofruns = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        noofruns = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* Execute C routine. */
    return_value = glrlSgre(inband, maxlength, noofruns);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
    ((double *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
