
/*C*

________________________________________________________________

        xite_peanoScan.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/texture.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int peanoScan(IBAND inband, IBAND outband, int blockPow) */

/*M*
function [outband] = xite_peanoScan(inband, blockPow)
%xite_peanoScan Scan a band in "peano" order.
%   Calls the XITE C function peanoScan.
%   function [outband] = xite_peanoScan(inband, blockPow)
%
%   The C function has the following prototype:
%   int peanoScan(IBAND inband, IBAND outband, int blockPow)
%
%   and the following manual page:
%
%   NAME
%          peanoScan - scan a band in "peano" order
%
%   SYNTAX
%           #include <xite/texture.h>
%
%           int peanoScan( IBAND inband, IBAND outband,
%              int blockPow );
%
%   DESCRIPTION
%          This procedure scans an input band according to the Peano curve rules
%          and write the pixel values in horizontal lines in an output band, one
%          dimensional representation of the input band.
%
%          If power is set to zero, only the largest possible square block of size
%          some power of two is transformed.
%           EX:
%             128x128 : The whole band is transformed
%             127x1024 : Only the upper, left 64x64 is transformed
%
%          If power is set to n, the image is split up in disjunct, square blocks
%          of size (2**n)x(2**n). Each complete block (entirely inside the band)
%          is transformed separately.
%           EX blockPow = 3:
%             128x128 : | The whole band is transformed in 16x16
%                        | separate blocks of size 8x8
%             127x1024 : | The upper, left field of size 120x1024
%                        | is transformed in 15x128 separate blocks
%                        | of size 8x8
%
%   RESTRICTIONS
%          Accepts only usigned byte pixels.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad input pixel type
%           2 => Bad output pixel type
%           3 => Bad power argument
%           4 => Warning: Output band to small
%
%   REFERENCE
%           1) Joel Quinqueton and Marc Berthod
%              "A Locally Adaptive Peano Scanning Algorithm"
%              IEEE Transaction on Pattern Ana. and Machine Inte.
%              Vol. PAMI-3, No. 4, July 1981
%           2) P. T. Nguyen and J. Quinqueton
%              "Space Filling Curves and Texture Analysis"
%              IEEE Transaction on Pattern Ana. and Machine Inte.
%              Vol. 2, 1982.
%
%   AUTHOR
%          Yogesan and Lønnestad, BLAB, ifi, UiO
%
%   ID
%          $Id: peanoScan.c,v 1.25 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 PEANOSCAN(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"peanoScan"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int blockPow;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "blockPow" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        blockPow = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        blockPow = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = peanoScan(inband, outband, blockPow);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C peanoScan.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
