
/*C*

________________________________________________________________

        xite_thn_lc.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thin.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thn_lc(int verbose, IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_thn_lc(verbose, inband)
%xite_thn_lc Lee and Chen’s thinning method.
%   Calls the XITE C function thn_lc.
%   function [outband] = xite_thn_lc(verbose, inband)
%
%   The C function has the following prototype:
%   int thn_lc(int verbose, IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          thn_lc - Lee and Chen’s thinning method.
%
%   SYNTAX
%           #include <xite/thin.h>
%
%           int thn_lc( int verbose, IBAND inband,
%              IBAND outband );
%
%   DESCRIPTION
%          Thin a binary image with Lee and Chen’s method. Black (0) is fore-
%          ground, and white (255) is background.
%
%          This method is simply an extension of Zhang and Suen’s method.  First,
%          Zhang and Suen’s method is used. The skeleton thus obtained is not
%          truly 8-connected, since some non-junction pixels have more than two
%          neighbors, making the skeleton useless for algorithms that require this
%          constraint.  Therefore, some pixels have to be removed. The skeleton is
%          inspected, and each pixel is tested using a lookup table. The result is
%          a true 8-connected skeleton where only junction pixels have more than
%          two 8-neighbors.
%
%          If verbose > 0, some messages are printed during execution.
%
%   RESTRICTIONS
%          inband must have pixel type unsigned byte.  inband is assumed to be
%          binary valued, with foreground = 0, background = 255.
%
%   AUTHOR
%          Øivind Due Trier, late one night at Michigan State University.
%
%   SEE ALSO
%          thn_lc(1), thn_har(3), thn_zs(3)
%
%   REFERENCE
%          H..--J.. Lee and B.. Chen,,
%                 "Recognition of handwritten chinese characters via short line
%                 segments,"  Pattern Recognition, vol.  25, no. 5, pp. 543-552,
%                 1992.
%
%                                     27 Feb 2014 THN_LC(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thn_lc"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    int verbose;
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "verbose" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        verbose = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        verbose = ((int *) mxGetData(prhs[0]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = thn_lc(verbose, inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thn_lc.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
