
/*C*

________________________________________________________________

        xite_histoCentroid.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thresMl.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int histoCentroid(histogram hin, histogram hout, int m, int method) */

/*M*
function [hout] = xite_histoCentroid(hin, m, method)
%xite_histoCentroid Move histogram mass to local centroid iteratively.
%   Calls the XITE C function histoCentroid.
%   function [hout] = xite_histoCentroid(hin, m, method)
%
%   The C function has the following prototype:
%   int histoCentroid(histogram hin, histogram hout, int m, int method)
%
%   and the following manual page:
%
%   NAME
%          histoCentroid - move histogram mass to local centroid iteratively
%
%   SYNTAX
%           #include <xite/thresMl.h>
%
%           int histoCentroid( histogram hin,
%              histogram hout, int m, int method );
%
%   DESCRIPTION
%          Iteratively, histogram mass is moved to local centroid.  The centroid
%          is the MEAN inside a histogram window if method=0, the MEDIAN inside
%          the window if method=1.  The size of the histogram window is (2*m)+1.
%          This procedure is repeated until no changes, or at most 100 iterations.
%          The "clustered" histogram result is returned in hout.
%
%   RESTRICTIONS
%           1 <= m <= 127
%           method = 0 OR method = 1
%
%   SEE ALSO
%          thresMl(1), thresMlAppScale(3), thresMlAppMean(3), thresMlAppMedian(3),
%          thresMlApply(3), thresMlCentroid(3), thresMlCentroid(1), mkComCur(3),
%          thresMlComCur(3), thresMlComCur(1), thresMlReddi(3), thresMlReddi(1),
%          thresMlWaHa(3), thresMlWaHa(1)
%
%   RETURN VALUE
%           0 => ok
%           1 => Warning: The procedure didn’t converge in
%                         100 iterations
%           2 => Error:   Bad m argument
%           3 => Error:   Bad method argument
%
%   REFERENCE
%          R.. Wilson
%                 "A Class of local centroid algorithms for classification and
%                 quantization in spaces of arbitrary dimension"Linkoping Univ,
%                 Internal Report LiTH-ISY-I-0610 (1983).
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresMlCentroid.c,v 1.27 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 HISTOCENTROID(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"histoCentroid"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram hin;
    histogram hout;
    int m;
    int method;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr hin_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 3) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 3) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    hin_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(hin, hin_ptr, sizeof(int) * 256);
    
    /* If scalar double is passed in for "m" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        m = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        m = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "method" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        method = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        method = ((int *) mxGetData(prhs[2]))[0];
    }

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = histoCentroid(hin, hout, m, method);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C histoCentroid.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(256, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), hout, 256 * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
