
/*C*

________________________________________________________________

        xite_mkComCur.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thresMl.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int mkComCur(IBAND band, histogram cc) */

/*M*
function [cc] = xite_mkComCur(band)
%xite_mkComCur Make complexity curve for a band.
%   Calls the XITE C function mkComCur.
%   function [cc] = xite_mkComCur(band)
%
%   The C function has the following prototype:
%   int mkComCur(IBAND band, histogram cc)
%
%   and the following manual page:
%
%   NAME
%          mkComCur - make complexity curve for a band
%
%   SYNTAX
%           #include <xite/thresMl.h>
%
%           int mkComCur( IBAND band, histogram cc );
%
%   DESCRIPTION
%          mkComCur produces the complexity curve of an N-bit image band by calcu-
%          lating the lengths of black and white boundaries for every possible
%          threshold.
%
%          cc is the returned complexity curve, indexable on 0..255.
%
%   RESTRICTIONS
%          band must have pixel type unsigned byte.
%
%   RETURN VALUE
%           0 => ok
%           2 => Bad pixel type
%
%   REFERENCE
%          Sei--ichiro Kamata,, Richard O.. Eason and Eiji Kawaguchi
%                 "Complexity Curves Versus Histograms and Their Application to
%                 Image Segmentation", 1070-1077
%
%   SEE ALSO
%          thresMl(1), thresMlAppScale(3), thresMlAppMean(3), thresMlAppMedian(3),
%          thresMlApply(3), histoCentroid(3), thresMlCentroid(3), thresMlCen-
%          troid(3), thresMlComCur(3), thresMlComCur(1), thresMlReddi(3),
%          thresMlReddi(1), thresMlWaHa(3), thresMlWaHa(1)
%
%   AUTHOR
%          Hung Buu Huynh, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresMlComCur.c,v 1.18 1998/02/26 10:57:52 svein Exp $
%
%                                     27 Feb 2014 MKCOMCUR(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"mkComCur"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    histogram cc;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));

    /* Prepare C result variable. */
        
    /* Execute C routine. */
    return_value = mkComCur(band, cc);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C mkComCur.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(256, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), cc, 256 * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
