
/*C*

________________________________________________________________

        xite_thresMlAppScale.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thresMl.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresMlAppScale(IBAND inband, IBAND outband, int *thresholds, int nt, double scale) */

/*M*
function [outband] = xite_thresMlAppScale(inband, thresholds, nt, scale)
%xite_thresMlAppScale Routines that applies ml-thresholds.
%   Calls the XITE C function thresMlAppScale.
%   function [outband] = xite_thresMlAppScale(inband, thresholds, nt, scale)
%
%   The C function has the following prototype:
%   int thresMlAppScale(IBAND inband, IBAND outband, int *thresholds, int nt, double scale)
%
%   and the following manual page:
%
%   NAME
%          thresMlApply, thresMlAppScale, thresMlAppMean, thresMlAppMedian - rou-
%          tines that applies ml-thresholds.
%
%   SYNTAX
%           #include <xite/thresMl.h>
%
%           int thresMlAppScale( IBAND inband,
%              IBAND outband, int* thresholds, int nt,
%              double scale );
%
%           int thresMlAppMean( IBAND inband,
%              IBAND outband, int* thresholds, int nt,
%              histogram histo );
%
%           int thresMlAppMedian( IBAND inband,
%              IBAND outband, int* thresholds, int nt,
%              histogram histo );
%
%   DESCRIPTION
%          The thresMlApply routines apply a vector of thresholds (Multi-Level
%          threshold)  to a band inband giving outband.  inband and outband may be
%          identical.
%
%          thresMlAppScale gives all pixels below the first threshold value 0, all
%          pixels between the first and the second: scale, then 2* scale,...  If
%          scale is too large ( scale >255* nt) it is adjusted down to fit.
%
%          thresMlAppMean sets all pixels between two thresholds to the mean of
%          all these pixels in the input band.
%
%          thresMlAppMedian sets all pixels between two thresholds to the median
%          of all these pixels in the input band.
%
%          nt is the number of thresholds.  thresholds is the vector of thresh-
%          olds, indexable on 0..nt-1.  scale is the scale factor.  histo is the
%          histogram of inband.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   SEE ALSO
%          thresMl(1), histoCentroid(3), thresMlCentroid(3), thresMlCentroid(1),
%          mkComCur(3), thresMlComCur(3), thresMlComCur(1), thresMlReddi(3),
%          thresMlReddi(1), thresMlWaHa(3), thresMlWaHa(1)
%
%   RETURN VALUE
%           0 : Ok
%           1 : Bad input pixel type
%           2 : Bad output pixel type
%           3 : nt is smaller than 1
%           4 : scale is negative
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresMlApply.c,v 1.19 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 THRESMLAPPLY(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresMlAppScale"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int *thresholds;
    int nt;
    double scale;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    thresholds = ((int **) mxGetData(prhs[1]))[0];
    
    /* If scalar double is passed in for "nt" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        nt = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        nt = ((int *) mxGetData(prhs[2]))[0];
    }
    scale = ((double *) mxGetData(prhs[3]))[0];

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = thresMlAppScale(inband, outband, thresholds, nt, scale);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thresMlAppScale.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
