
/*C*

________________________________________________________________

        xite_thresMlReddi.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thresMl.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresMlReddi(histogram h, int *t, int nt) */

/*M*
function [t] = xite_thresMlReddi(h, nt)
%xite_thresMlReddi Multi-level thresholding by Reddi et al.
%   Calls the XITE C function thresMlReddi.
%   function [t] = xite_thresMlReddi(h, nt)
%
%   The C function has the following prototype:
%   int thresMlReddi(histogram h, int *t, int nt)
%
%   and the following manual page:
%
%   NAME
%          thresMlReddi - multi-level thresholding by Reddi et al.
%
%   SYNTAX
%           #include <xite/thresMl.h>
%
%           int thresMlReddi( histogram h, int* t, int nt );
%
%   DESCRIPTION
%          thresMlReddi performs multi-level thresholding according to the method
%          by Reddi et al.  h is input histogram, t is output threshold array,
%          indexable on 0..  nt -1.
%
%   REFERENCE
%          S.. S.. Reddi,, S.. F.. Rudin and H.. R.. Keshavan
%                 "An Optimal Multiple Threshold Scheme for Image Segmentation"
%                 IEEE Transactions on Systems, Man and Cybernetics, Vol SMC-14,
%                 pp. 661-665, 1984.
%
%   SEE ALSO
%          thresMl(1), thresMlAppScale(3), thresMlAppMean(3), thresMlAppMedian(3),
%          thresMlApply(3), histoCentroid(3), thresMlCentroid(3), thresMlCen-
%          troid(3), mkComCur(3), thresMlComCur(3), thresMlComCur(1),
%          thresMlReddi(1), thresMlWaHa(3), thresMlWaHa(1)
%
%   RETURN VALUE
%           0 => ok
%           1 => nt less than 1
%
%   AUTHOR
%          Martin Torpe Lie, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresMlReddi.c,v 1.23 1998/02/26 10:57:53 svein Exp $
%
%                                     27 Feb 2014 THRESMLREDDI(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresMlReddi"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram h;
    int *t;
    int nt;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr h_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    h_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(h, h_ptr, sizeof(int) * 256);
    
    /* If scalar double is passed in for "nt" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        nt = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        nt = ((int *) mxGetData(prhs[1]))[0];
    }

    /* Prepare C result variable. */
    t = (int *) mxCalloc(nt, sizeof(int));
    
    /* Execute C routine. */
    return_value = thresMlReddi(h, t, nt);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thresMlReddi.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    tmp_plhs[0] =    mxCreateNumericMatrix(nt, 1, mxINT32_CLASS, mxREAL);
    memcpy(mxGetData(tmp_plhs[0]), t, (nt) * sizeof(int));
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
