
/*C*

________________________________________________________________

        xite_thresMlWaHa.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/thresMl.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresMlWaHa(IBAND gray, IBAND tri, IBAND edge, int thVect[], int edgeTh, int numTh, int population, int n, int *totTh) */

/*M*
function [tri, edge, thVect, totTh] = xite_thresMlWaHa(gray, edgeTh, numTh, population, n)
%xite_thresMlWaHa Finds multiple thresholds by Wang & Haralick method.
%   Calls the XITE C function thresMlWaHa.
%   function [tri, edge, thVect, totTh] = xite_thresMlWaHa(gray, edgeTh, numTh, population, n)
%
%   The C function has the following prototype:
%   int thresMlWaHa(IBAND gray, IBAND tri, IBAND edge, int thVect[], int edgeTh, int numTh, int population, int n, int *totTh)
%
%   and the following manual page:
%
%   NAME
%          thresMlWaHa - Finds multiple thresholds by Wang & Haralick method.
%
%   SYNTAX
%           #include <xite/thresMl.h>
%
%           int thresMlWaHa( IBAND gray, IBAND tri,
%              IBAND edge, int thVect[], int edgeTh,
%              int numTh, int population, int n,
%              int* totTh );
%
%   DESCRIPTION
%          The pixels in gray are first classified as either edge pixels or
%          nonedge pixels. Edge pixels are then classified, on the basis of their
%          neighborhoods, as being relatively dark, relatively bright or gray. A
%          histogram of the graytone intensities is obtained for those pixels
%          which are edge pixels and relatively dark, and another histogram is
%          obtained for those which are edge pixels and relatively bright.  Two
%          candidate thresholds are found at the intensity values corresponding to
%          peaks in the two histograms. The threshold giving the highest probabil-
%          ity of getting pure dark or bright regions is selected. If the selected
%          threshold corresponds to the peak in the dark edge histogram, the pro-
%          cess is repeated for the graytone range above the threshold, if the
%          threshold corresponds to the peak in the bright edge histogram, the
%          process is repeated for the graytone range below the threshold.
%
%          You can specify the number of thresholds to be found, or a minimum pop-
%          ulation of the two histograms at which to stop.  The population of the
%          two histograms decreases rapidly, and might reach zero before the spec-
%          ified number of thresholds is found. There is no peak in a histogram
%          with only zero entries and the process stops.
%
%   PARAMETERS
%          gray The inband to be thresholded.
%
%          tri A band to hold the tri-tone edge band. Can be NULL.
%
%          edge A band to hold the sobel magnitude edge band. Can be NULL.
%
%          thVect A 256 element int vector returning the threshold values in
%                 increasing order. The number of thresholds found are returned
%                 through the argument totTh.
%
%          edgeTh The threshold-value at wich to threshold the edge band.
%
%          numTh The number of thresholds wanted.
%
%          pop The minimum population allowed for the dark/bright edge his-
%                 tograms. One and only one of numTh and pop must be specified,
%                 the other one should be zero.
%
%          n An integer used to adjust the number of edge pixels classified
%                 as either dark or bright. The number of pixels classified as
%                 either dark or bright increases with increasing n.  Must be pos-
%                 itive and non-zero, usually n = 3.
%
%          totTh The number of thresholds found.
%
%   REFERENCE
%          S.. Wang and R.. M.. Haralick
%                 "Automatic Multithreshold Selection".  Computer Vision, Graph-
%                 ics, and Image Processing, 25, 46-67 (1984).
%
%   RESTRICTIONS
%          gray, tri and edge must have pixel type unsigned byte.
%
%   SEE ALSO
%          thresMl(1), thresMlAppScale(3), thresMlAppMean(3), thresMlAppMedian(3),
%          thresMlApply(3), histoCentroid(3), thresMlCentroid(3), thresMlCen-
%          troid(3), mkComCur(3), thresMlComCur(3), thresMlComCur(1),
%          thresMlReddi(3), thresMlReddi(3), thresMlWaHa(1), sobel(3)
%
%   RETURN VALUE
%           Status value:
%              0 => ok
%              1 => None or both "numTh" and "pop" specified.
%              2 => Wrong pixeltype in "gray".
%              3 => Wrong pixeltype in "tri".
%              4 => Wrong pixeltype in "edge".
%              5 => Malloc error. Not able to create temporary image.
%              6 => "tri" size different from "gray" size.
%              7 => "edge" size different from "gray" size.
%              8 => "n" < 1.
%              9 => No thresholds found.
%
%   AUTHOR
%          Jørn Tilley Jacobsen, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresMlWaHa.c,v 1.24 2014/01/31 12:53:49 svein Exp $
%
%                                     27 Feb 2014 THRESMLWAHA(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresMlWaHa"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND gray;
    IBAND tri;
    IBAND edge;
    int *thVect;
    int edgeTh;
    int numTh;
    int population;
    int n;
    int totTh;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE gray_img;
    IMAGE tri_img;
    IMAGE edge_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 5) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 5) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 4) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    gray_img = Init_image(1, "");
    
    convert_status = matlab2biff(gray_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    gray = (IBAND) (gray_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) gray));
    output_ysize = MAX(output_ysize, Iysize((IBAND) gray));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) gray));
    
    /* If scalar double is passed in for "edgeTh" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        edgeTh = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        edgeTh = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "numTh" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        numTh = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        numTh = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "population" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        population = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        population = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "n" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        n = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        n = ((int *) mxGetData(prhs[4]))[0];
    }

    /* Prepare C result variable. */
    tri_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    tri = (IBAND) (tri_img[1]);
    edge_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    edge = (IBAND) (edge_img[1]);
    thVect = (int *) mxCalloc(256, sizeof(int));
        
    /* Execute C routine. */
    return_value = thresMlWaHa(gray, tri, edge, thVect, edgeTh, numTh, population, n, &totTh);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thresMlWaHa.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(tri_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }
    if (nlhs > 1) {
        convert_status =
    	biff2matlab(edge_img, NULL, 1, &tmp_plhs[1], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 1) plhs[1] = tmp_plhs[1];
    }
    if (nlhs > 2) {
        tmp_plhs[2] =    mxCreateNumericMatrix(256, 1, mxINT32_CLASS, mxREAL);
        memcpy(mxGetData(tmp_plhs[2]), thVect, (256) * sizeof(int));
        plhs[2] = tmp_plhs[2];
    }
    if (nlhs > 3) {
        tmp_plhs[3] =
            mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
        ((int *) mxGetData(tmp_plhs[3]))[0] = totTh;
        plhs[3] = tmp_plhs[3];
    }

    PopMessage();

    return;
}
