
/*C*

________________________________________________________________

        xite_thresBernsen.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresBernsen(IBAND inband, IBAND outband, int cmin, int dx, int dy, int lpv, int hpv, int low) */

/*M*
function [outband] = xite_thresBernsen(inband, cmin, dx, dy, lpv, hpv, low)
%xite_thresBernsen Local treshold by the method of Bernsen.
%   Calls the XITE C function thresBernsen.
%   function [outband] = xite_thresBernsen(inband, cmin, dx, dy, lpv, hpv, low)
%
%   The C function has the following prototype:
%   int thresBernsen(IBAND inband, IBAND outband, int cmin, int dx, int dy, int lpv, int hpv, int low)
%
%   and the following manual page:
%
%   NAME
%          thresBernsen - local treshold by the method of Bernsen.
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int thresBernsen( IBAND inband, IBAND outband,
%              int cmin, int dx, int dy, int lpv, int hpv,
%              int low );
%
%   DESCRIPTION
%          If the difference between the largest and the smallest pixel value
%          within the dx * dy window is greater than or equal to cmin (local con-
%          trast threshold), the average of the two values is used as threshold.
%          Pixels below this threshold are set to lpv, and pixels above are set to
%          hpv.
%
%          If low is TRUE, pixels in homogenous areas (difference below cmin) are
%          assumed to be below the threshold and are set to lpv, and if low is
%          FALSE these pixels are set to hpv.
%
%   RESTRICTIONS
%          inband and outband must have pixel type unsigned byte.
%
%   REFERENCE
%          Bernsen, J:  "Dynamic thresholding of grey-level images" Proc. of the
%          8th ICPR, Paris, Oct 1986, 1251-1255.
%
%   RETURN VALUE
%           0 => ok
%           1 => bad pixel type inband
%           2 => bad pixel type outband
%           3 => bad cmin value
%           4 => bad dx or dy value
%           5 => bad lpv or hpv value
%           6 => not enough memory
%
%   AUTHOR
%          Olav Borgli and Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresBernsen.c,v 1.24 2014/01/31 12:53:50 svein Exp $
%
%                                     27 Feb 2014 THRESBERNSEN(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresBernsen"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND inband;
    IBAND outband;
    int cmin;
    int dx;
    int dy;
    int lpv;
    int hpv;
    int low;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 7) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 7) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));
    
    /* If scalar double is passed in for "cmin" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        cmin = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        cmin = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "dx" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        dx = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        dx = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "dy" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        dy = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        dy = ((int *) mxGetData(prhs[3]))[0];
    }
    
    /* If scalar double is passed in for "lpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[4]) && mxGetM(prhs[4]) && mxGetN(prhs[4])) {
        lpv = (int) (((double *) mxGetData(prhs[4]))[0]);
    } else {
        lpv = ((int *) mxGetData(prhs[4]))[0];
    }
    
    /* If scalar double is passed in for "hpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[5]) && mxGetM(prhs[5]) && mxGetN(prhs[5])) {
        hpv = (int) (((double *) mxGetData(prhs[5]))[0]);
    } else {
        hpv = ((int *) mxGetData(prhs[5]))[0];
    }
    
    /* If scalar double is passed in for "low" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[6]) && mxGetM(prhs[6]) && mxGetN(prhs[6])) {
        low = (int) (((double *) mxGetData(prhs[6]))[0]);
    } else {
        low = ((int *) mxGetData(prhs[6]))[0];
    }

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = thresBernsen(inband, outband, cmin, dx, dy, lpv, hpv, low);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thresBernsen.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
