
/*C*

________________________________________________________________

        xite_thresMinErrEqVar.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresMinErrEqVar(histogram h, int tinit) */

/*M*
function [return_value] = xite_thresMinErrEqVar(h, tinit)
%xite_thresMinErrEqVar Minimum error thresholding with equal variances.
%   Calls the XITE C function thresMinErrEqVar.
%   function [return_value] = xite_thresMinErrEqVar(h, tinit)
%
%   The C function has the following prototype:
%   int thresMinErrEqVar(histogram h, int tinit)
%
%   and the following manual page:
%
%   NAME
%          thresMinErrEqVar - minimum error thresholding with equal variances.
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int thresMinErrEqVar( histogram h, int tinit );
%
%   DESCRIPTION
%          thresMinErrEqVar calculates the threshold value by using the minimum
%          error thresholding technique with equal variances as in Gonzalez and
%          Wintz.  Initially, the threshold value t is set according to tinit.
%
%           0 <= tinit <= 255, then t = tinit
%           tinit = -1, then t = the average of all the pixels
%           tinit = -2, then t = the result from thresRidCal(h)
%
%          Then, iteratively, the threshold is set to the average of the mean of
%          the pixels below the threshold, and the mean of the pixels above the
%          threshold , plus a compensation factor depending on the variance of the
%          distribution. This variance is the average of the variance below and
%          above the threshold. When the threshold value remains constant, itera-
%          tion terminates.
%
%          The actual thresholding is not performed, see the routine "threshold".
%
%   REFERENCE
%          Gonzales, Rafael C & Wintz, Paul: "Digital Image Processing" Addison-
%          Wesley, 1977.
%
%   RETURN VALUE
%          The threshold value. Pixel values up to, and including, the return
%          value, is below the threshold.
%
%   AUTHOR
%          Olav Borgli, Fritz Albregtsen, BLAB, ifi, UiO
%
%   ID
%          $Id: thresMinErr.c,v 1.28 2014/01/31 12:53:50 svein Exp $
%
%                                     27 Feb 2014 THRESMINERREQVAR(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresMinErrEqVar"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram h;
    int tinit;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr h_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    h_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(h, h_ptr, sizeof(int) * 256);
    
    /* If scalar double is passed in for "tinit" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        tinit = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        tinit = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* Execute C routine. */
    return_value = thresMinErrEqVar(h, tinit);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
