
/*C*

________________________________________________________________

        xite_thresOtsu.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresOtsu(histogram h) */

/*M*
function [return_value] = xite_thresOtsu(h)
%xite_thresOtsu Threshold by the method of Otsu.
%   Calls the XITE C function thresOtsu.
%   function [return_value] = xite_thresOtsu(h)
%
%   The C function has the following prototype:
%   int thresOtsu(histogram h)
%
%   and the following manual page:
%
%   NAME
%          thresOtsu - threshold by the method of Otsu
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int thresOtsu( histogram h );
%
%   DESCRIPTION
%          thresOtsu calculates the threshold value needed to threshold the image
%          that the histogram h is created from, according to the method of Otsu.
%          The actual thresholding is not performed, see the "threshold" routine.
%
%   REFERENCE
%          Otsu,, N..
%                 "A Threshold Selection Method from Gray-Level histograms"  IEEE
%                 Trans. Syst., Man and Cyb., Vol SMC-9, No.1, pp.62-66.
%
%   RETURN VALUE
%          The threshold value.
%
%   SEE ALSO
%          thresOtsu(1), threshold(3), thresholdSpOut(3), thresPtile(3), thresRid-
%          Cal(3), thresLloyd(3)
%
%   AUTHOR
%          Tor Lønnestad, BLAB, Ifi, UiO
%
%   ID
%          $Id: thresOtsu.c,v 1.22 2014/01/31 12:53:50 svein Exp $
%
%                                     27 Feb 2014 THRESOTSU(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresOtsu"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram h;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr h_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    h_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(h, h_ptr, sizeof(int) * 256);
    
    /* Execute C routine. */
    return_value = thresOtsu(h);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
