
/*C*

________________________________________________________________

        xite_thresSigma.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresSigma(histogram h, double x) */

/*M*
function [return_value] = xite_thresSigma(h, x)
%xite_thresSigma Threshold at fraction of standard deviation.
%   Calls the XITE C function thresSigma.
%   function [return_value] = xite_thresSigma(h, x)
%
%   The C function has the following prototype:
%   int thresSigma(histogram h, double x)
%
%   and the following manual page:
%
%   NAME
%          thresSigma - threshold at fraction of standard deviation
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int thresxsigma(h, x)
%           histogram h;
%           double x;
%
%           int thresSigma( histogram h, double x );
%
%   DESCRIPTION
%          thresSigma calculates the threshold value by finding the mean (mu) and
%          the standard deviation (sigma) of the image, and then finding the
%          threshold by the formula:
%
%          t = mu + (x * sigma)
%
%          The actual thresholding is not performed, see the routine "threshold".
%
%   RETURN VALUE
%          The threshold value. Pixel values up to, and including, the return
%          value, is below the threshold.
%
%   AUTHOR
%          Olav Borgli, BLAB, ifi, UiO
%
%   ID
%          $Id: thresSigma.c,v 1.22 2004/05/05 07:34:31 svein Exp $
%
%                                     27 Feb 2014 THRESSIGMA(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresSigma"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    histogram h;
    double x;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    typedef int * histogram_ptr;
    histogram_ptr h_ptr;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    /* Check data type of histogram. */
    
    if (mxGetClassID(prhs[0]) != mxINT32_CLASS) {
        errorMsgToMatlab(2, "Histogram must have data type int.\n");
    }    
    
    /* Check size of histogram. */
    if (!((mxGetM(prhs[0]) == 256 && mxGetN(prhs[0]) == 1) ||
          (mxGetN(prhs[0]) == 256 && mxGetM(prhs[0]) == 1))) {
        errorMsgToMatlab(2, "Histogram must have length 256.\n");
    }
    h_ptr = ((histogram_ptr) mxGetData(prhs[0]));
    memcpy(h, h_ptr, sizeof(int) * 256);
    x = ((double *) mxGetData(prhs[1]))[0];
    
    /* Execute C routine. */
    return_value = thresSigma(h, x);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
