
/*C*

________________________________________________________________

        xite_thresUniErr.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresUniErr(IBAND band) */

/*M*
function [return_value] = xite_thresUniErr(band)
%xite_thresUniErr Uniform Error Threshold.
%   Calls the XITE C function thresUniErr.
%   function [return_value] = xite_thresUniErr(band)
%
%   The C function has the following prototype:
%   int thresUniErr(IBAND band)
%
%   and the following manual page:
%
%   NAME
%          thresUniErr - Uniform Error Threshold
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int thresUniErr( IBAND band );
%
%   DESCRIPTION
%          thresUniErr calculates the threshold value needed to threshold the
%          image according to the uniform error thresholding metod as described in
%          Dunn & Harwood & Davis.
%
%          The actual thresholding is not performed, see the routine "threshold".
%
%   REFERENCE
%          S. M. Dunn & D. Harwood & L. S. Davis: "Local Estimation of the Uniform
%          Error Threshold" IEEE Trans. on PAMI, Vol PAMI-6, No 6, Nov 1984.
%
%   RETURN VALUE
%          The threshold value.  Pixel values up to, and including, the return
%          value, is below the threshold.  Returns -1 if bad pixel type, accepts
%          ony UNS_BYTE pixels.
%
%   BUGS
%          It only works well on images whith large objects.
%
%   AUTHOR
%          Olav Borgli, BLAB, ifi, UiO
%
%   ID
%          $Id: thresUniErr.c,v 1.21 1998/02/26 10:59:55 svein Exp $
%
%                                     27 Feb 2014 THRESUNIERR(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresUniErr"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND band;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE band_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 1) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 1) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    band_img = Init_image(1, "");
    
    convert_status = matlab2biff(band_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    band = (IBAND) (band_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) band));
    output_ysize = MAX(output_ysize, Iysize((IBAND) band));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) band));
    
    /* Execute C routine. */
    return_value = thresUniErr(band);
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));
    
    /* Pack return value. */
    tmp_plhs[0] =
        mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    ((int *) mxGetData(tmp_plhs[0]))[0] = return_value;
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    } else if (nlhs > 0) {
        plhs[0] = tmp_plhs[0];
    }

    PopMessage();

    return;
}
