
/*C*

________________________________________________________________

        xite_thresholdSpOut.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/threshold.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int thresholdSpOut(IBAND b1, IBAND b2, int t, int lpv, int hpv) */

/*M*
function [b2] = xite_thresholdSpOut(b1, t, lpv, hpv)
%xite_thresholdSpOut Threshold a band with specified threshold.
%   Calls the XITE C function thresholdSpOut.
%   function [b2] = xite_thresholdSpOut(b1, t, lpv, hpv)
%
%   The C function has the following prototype:
%   int thresholdSpOut(IBAND b1, IBAND b2, int t, int lpv, int hpv)
%
%   and the following manual page:
%
%   NAME
%          threshold, thresholdSpOut - threshold a band with specified threshold
%
%   SYNTAX
%           #include <xite/threshold.h>
%
%           int threshold( IBAND b1, IBAND b2, int t );
%
%           int thresholdSpOut( IBAND b1, IBAND b2, int t,
%              int lpv, int hpv );
%
%   DESCRIPTION
%          threshold and thresholdSpOut thresholds b1 into b2 with threshold t.
%
%           if (b1[x,y] LE t) b2[x,y] = lpv; else b2[x,y] = hpv;
%
%          threshold uses 0 for low pixel value ( lpv) and 255 for high pixel
%          value ( hpv), while thresholdSpOut uses the supplied arguments.   Only
%          largest common rectangle of b1 and b2 is thresholded. Accepts only
%          UNS_BYTE pixles.  Identical in- and out- parameters are legal.   <lpv>
%          and <hpv> should be in 1..255.
%
%   RETURN VALUE
%           0 => ok
%           1 => Bad input pixel type
%           2 => Bad output pixel type
%           3 => Bad lpv value (only thresholdSpOut)
%           4 => Bad hpv value (only thresholdSpOut)
%
%   AUTHOR
%          Tor L|nnestad, BLAB, IfI, UiO
%
%   ID
%          $Id: threshold.c,v 1.28 2014/01/31 12:53:50 svein Exp $
%
%                                     27 Feb 2014 THRESHOLD(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"thresholdSpOut"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    IBAND b1;
    IBAND b2;
    int t;
    int lpv;
    int hpv;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE b1_img;
    IMAGE b2_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 4) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 4) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    b1_img = Init_image(1, "");
    
    convert_status = matlab2biff(b1_img, NULL, 1,
                                 &prhs[0], NULL, NULL);
    b1 = (IBAND) (b1_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) b1));
    output_ysize = MAX(output_ysize, Iysize((IBAND) b1));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) b1));
    
    /* If scalar double is passed in for "t" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[1]) && mxGetM(prhs[1]) && mxGetN(prhs[1])) {
        t = (int) (((double *) mxGetData(prhs[1]))[0]);
    } else {
        t = ((int *) mxGetData(prhs[1]))[0];
    }
    
    /* If scalar double is passed in for "lpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[2]) && mxGetM(prhs[2]) && mxGetN(prhs[2])) {
        lpv = (int) (((double *) mxGetData(prhs[2]))[0]);
    } else {
        lpv = ((int *) mxGetData(prhs[2]))[0];
    }
    
    /* If scalar double is passed in for "hpv" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[3]) && mxGetM(prhs[3]) && mxGetN(prhs[3])) {
        hpv = (int) (((double *) mxGetData(prhs[3]))[0]);
    } else {
        hpv = ((int *) mxGetData(prhs[3]))[0];
    }

    /* Prepare C result variable. */
    b2_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    b2 = (IBAND) (b2_img[1]);
    
    /* Execute C routine. */
    return_value = thresholdSpOut(b1, b2, t, lpv, hpv);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C thresholdSpOut.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(b2_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
