
/*C*

________________________________________________________________

        xite_zer_con.c
        Copyright 1994-2003, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

#include <xite/includes.h>
#include <xite/matlab.h>
#include <xite/zernike.h>
#include <xite/message.h>
#include <string.h>
#include <mex.h>
#include <xite/mex.h>

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? a : b)
#endif
#ifndef MIN
#define MIN(a,b) ((a)<(b) ? a : b)
#endif

/* int zer_con(int n, IBAND inband, IBAND outband) */

/*M*
function [outband] = xite_zer_con(n, inband)
%xite_zer_con Zernike moments.
%   Calls the XITE C function zer_con.
%   function [outband] = xite_zer_con(n, inband)
%
%   The C function has the following prototype:
%   int zer_con(int n, IBAND inband, IBAND outband)
%
%   and the following manual page:
%
%   NAME
%          zer_mom, zer_pol, zer_rec, zer_con - Zernike moments
%
%   SYNTAX
%           #include <xite/zernike.h>
%
%           int zer_mom( IBAND inband, int n, int m,
%              DCOMPLEX *res );
%
%           int zer_pol( int n, int m, double x, double y,
%              DCOMPLEX *res );
%
%           int zer_con( int n, IBAND inband, IBAND outband );
%
%           int zer_rec( int order, IBAND inband, IBAND outband );
%
%   DESCRIPTION
%          The Zernike moment of order n and repetition m of an image f(x,y) is
%          defined as follows:
%                    n+1 *
%           A(n,m) = ---- Sum Sum f(x,y)[V(n,m, x,y)]
%                     pi x y
%
%           where x^2+y^2 <= 1
%
%          The image V(n,m, x,y) is the Zernike basis images of order n and repe-
%          tition m.   These basis images are complex and orthogonal. The Zernike
%          moments are essentially the projections of the input image onto these
%          basis images.
%
%          The original image can be reconstructed from the Zernike moments. The
%          N-th order approximation is given by
%            ^         N
%            f(x,y) = Sum Sum A(n,m) V(n,m, x,y)
%                     n=0 m
%
%          The contribution or information content of the n-th order moments is
%
%            I(x,y, n) = Sum A(n,m) V(n,m, x,y)
%                        m
%
%          zer_mom computes A(n,m).
%
%          zer_pol computes the zernike basis function V(n,m,x,y).
%
%          zer_con computes the absolute value of the contribution of the n-th
%          order moments, i.e. the absolute value of I(x,y, n).
%
%          zer_rec demonstrates how the input image can be reconstructed from the
%          Zernike basis functions and the Zernike moments.
%
%   RETURN VALUE
%           0 : OK.
%
%   AUTHOR
%          Øivind Due Trier, Dept. Informatics, Univ. Oslo
%
%                                     27 Feb 2014 ZER_MOM(3)
*/

void mexFunction(int nlhs, mxArray **plhs, int nrhs, const mxArray **prhs) {
    int convert_status = Iok;
    int argc = 1;
    char *argv[] = {"zer_con"};
    int return_value;      /* From C routine. */
    IPIXTYP output_pixtyp = Iu_byte_typ;
    long output_xsize = 0;
    long output_ysize = 0;
    
    /* Declaration of variables to pass to/from C routine. */
    int n;
    IBAND inband;
    IBAND outband;
    /* Declaration of array to temporarily hold output arguments and
       non-status function return value when MATLAB output variables are
       present. */
    mxArray **tmp_plhs;

    /* Auxiliary variables for format conversion. */
    IMAGE inband_img;
    IMAGE outband_img;

    /* Initialize messaging system - send XITE warnings/errors to MATLAB. */
    Iset_message(1);
    InitMessage(&argc, argv, NULL);
    PushMessage(warnMsgToMatlab, warnMsgToMatlab, errorMsgToMatlab, 0);
    /* Check argument count. */
    if (nrhs < 2) errorMsgToMatlab(2, "Missing input arguments.\n");
    if (nrhs > 2) errorMsgToMatlab(2, "Too many input arguments.\n");
    if (nlhs > 1) errorMsgToMatlab(2, "Too many output arguments.\n");

    /* Convert from MATLAB input array to C type variables. */
    
    /* If scalar double is passed in for "n" which expects */
    /* "int", be flexible and allow this mismatch for ease of use */
    /* in MATLAB. */
    
    if (mxIsDouble(prhs[0]) && mxGetM(prhs[0]) && mxGetN(prhs[0])) {
        n = (int) (((double *) mxGetData(prhs[0]))[0]);
    } else {
        n = ((int *) mxGetData(prhs[0]))[0];
    }
    inband_img = Init_image(1, "");
    
    convert_status = matlab2biff(inband_img, NULL, 1,
                                 &prhs[1], NULL, NULL);
    inband = (IBAND) (inband_img[1]);
    if (convert_status != Iok)
        errorMsgToMatlab(2, "Failed in converting MATLAB array to BIFF image.\n");
    output_xsize = MAX(output_xsize, Ixsize((IBAND) inband));
    output_ysize = MAX(output_ysize, Iysize((IBAND) inband));
    output_pixtyp = MAX(output_pixtyp, Ipixtyp((IBAND) inband));

    /* Prepare C result variable. */
    outband_img = 
        Imake_image(1, "", output_pixtyp, output_xsize, output_ysize);
    outband = (IBAND) (outband_img[1]);
    
    /* Execute C routine. */
    return_value = zer_con(n, inband, outband);
    if (return_value != Iok)
        errorMsgToMatlab(2, "Failed in C zer_con.\n");
    
    tmp_plhs = (mxArray **) mxCalloc((size_t) MAX(nlhs, 1), sizeof(mxArray *));

    /* Convert to MATLAB arrays. */
    if (nlhs == 0 || nlhs > 0) {
        convert_status =
    	biff2matlab(outband_img, NULL, 1, &tmp_plhs[0], NULL, NULL);
        if (convert_status != Iok)
            errorMsgToMatlab(2,
                "Failed in converting BIFF image to MATLAB array.\n");
        if (nlhs > 0) plhs[0] = tmp_plhs[0];
    }
    if (nlhs == 0) {
        /* Return in callers workspace as variable ans. */
        mexPutVariable("caller", "ans", tmp_plhs[0]);
    }

    PopMessage();

    return;
}
