
/*C*
________________________________________________________________

        biff2png
        $Id: biff2png.c,v 1.17 2014/01/31 12:53:48 svein Exp $
        Copyright 2004, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: ";

#include <xite/includes.h>
#include <xite/message.h>
#include <xite/readarg.h>
#include <xite/debug.h>
#include XITE_MALLOC_H
#include <xite/xite_png.h>

#ifndef MAIN

static png_text *create_png_text(IMAGE img, png_structp png_ptr,
				 int *num_text) {
  png_text *text_ptr;
  char *title       = Ititle(img);
  char *comment     = "Converted from XITE BIFF format by biff2png.";
  char *description;

  Iget_text(img, &description);

  *num_text = 3;
  text_ptr = (png_text *) png_malloc(png_ptr, (*num_text) * sizeof(png_text));

  text_ptr[0].key         = "Title";
  text_ptr[0].compression = PNG_TEXT_COMPRESSION_NONE;
  text_ptr[0].text        = title;
  text_ptr[0].text_length = strlen(title);

  text_ptr[1].key         = "Description";
  text_ptr[1].compression = PNG_TEXT_COMPRESSION_NONE;
  text_ptr[1].text        = description;
  text_ptr[1].text_length = strlen(description);

  text_ptr[2].key         = "Comment";
  text_ptr[2].compression = PNG_TEXT_COMPRESSION_NONE;
  text_ptr[2].text        = comment;
  text_ptr[2].text_length = strlen(comment);

  return(text_ptr);

} /* create_png_text */

/*F:biff2png8=biff2png*/
/*F:biff2png16=biff2png*/
/*F:biff2png8p=biff2png*/
/*F:biff2png8c=biff2png*/
/*F:biff2png16c=biff2png*/
/*F:biff2png*

________________________________________________________________

                biff2png
________________________________________________________________

Name:           biff2png, biff2png8, biff2png16, biff2png8p, biff2png8c,
                biff2png16c - Convert a BIFF image to PNG

Syntax:         | #include <xite/png.h>
                |
		| int biff2png( IMAGE img, png_structp png_ptr,
		|    png_infop info_ptr, int format, Color_tab coltab,
		|    int colLen, int compression_method,
		|    int interlace_method );
		|
		| int biff2png8( IBAND band, png_structp png_ptr,
		|    png_infop info_ptr, int compression_method,
		|    int interlace_method );
		|
		| int biff2png16( IUS_BAND band, png_structp png_ptr,
		|    png_infop info_ptr, int compression_method,
		|    int interlace_method );
		|
		| int biff2png8p( IBAND band, png_structp png_ptr,
		|    png_infop info_ptr, Color_tab coltab, int colLen,
		|    int compression_method, int interlace_method );
		|
		| int biff2png8c( IMAGE img, png_structp png_ptr,
		|    png_infop info_ptr, int compression_method,
		|    int interlace_method );
		|
		| int biff2png16c( IUS_IMAGE img, png_structp png_ptr,
		|    png_infop info_ptr, int compression_method,
		|    int interlace_method );

Description:    'biff2png' is a wrapper function that decides which one of the
                other routines to use, depending on BIFF image pixeltyp and the
		value of the parameter 'format' (XITE_PNG_RGB,
		XITE_PNG_PALETTE or XITE_PNG_GRAYSCALE). 'coltab' is the
		supplied XITE colortable with 'colLen' entries.
		'compression_method' is one of the legal libpng values for
		compression, 'interlace_method' is one of the legal libpng
		values for interlacing.

		Byte-ordering between little-endian and big-endian machines
		is handled automatically.

		The conversion is according to the second edition of the
		PNG specification and uses the libpng library which must exist
		on your system, it is not supplied with XITE.

		Grayscale conversion routines:

		'biff2png8' converts an unsigned byte BIFF band to a PNG
		grayscale (8-bit unsigned byte) image.

		'biff2png16' converts an unsigned short BIFF band to a PNG
		grayscale (16-bit unsigned short) image.

		Palette (pseudocolor) conversion routines:

		'biff2png8p' converts an unsigned byte BIFF band with the
		colortable 'coltab' to a PNG palette image.

		RGB conversion routines:

		'biff2png8c' converts the first three unsigned byte BIFF bands
		to a PNG 3*8-bit unsigned byte rgb image.

		'biff2png16c' converts the first three unsigned short BIFF
		bands to a PNG 3*16-bit unsigned byte rgb image.

Return value:   | 0 => ok
                | 3 => Error

See also:       'biff2png(1)', 'png2biff(1)', 'png2biff(3)'

Author:         Svein Bøe, BLAB, Ifi, UiO

Id:             $Id: biff2png.c,v 1.17 2014/01/31 12:53:48 svein Exp $

*/

int biff2png8(IBAND band, png_structp png_ptr, png_infop info_ptr, int compression_method, int interlace_method) {
  png_uint_32 width, height, x, y;
  int bit_depth, num, byte_depth, status;
  png_bytepp row_pointers;

  ENTER_FUNCTION_DEBUG("biff2png8");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png8.\n",
		 status));
  }

  width      = (png_uint_32) Ixsize(band);
  height     = (png_uint_32) Iysize(band);
  bit_depth  = (int) Ipixsize(Ipixtyp(band));
  byte_depth = (bit_depth + 7) / 8;

  FPRINTF5("width=%d, height=%d, bit_depth=%d byte_depth=%d\n",
	   width, height, bit_depth, byte_depth);

  png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth,
	       PNG_COLOR_TYPE_GRAY, interlace_method, compression_method,
	       PNG_FILTER_TYPE_DEFAULT);

  png_data_freer(png_ptr, info_ptr, PNG_DESTROY_WILL_FREE_DATA,
		 PNG_FREE_ROWS | PNG_FREE_TEXT);

  row_pointers = (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));
  for (num = 0; num < height; num++)
    row_pointers[num] =
      (png_bytep) png_malloc(png_ptr, width * byte_depth * sizeof(png_byte));

  png_set_rows(png_ptr, info_ptr, row_pointers);

  for (y = 0; y < height; y++) {
    for (x = 0; x < width; x++) {
      row_pointers[y][x] = band[y + 1][x + 1];
      if (y == 0) FPRINTF2("%hhd ", row_pointers[y][x]);
    }
    if (y == 0) FPRINTF1("\n");
  }

  LEAVE_FUNCTION_DEBUG("biff2png8");

  return(0);

} /* biff2png8 */

int biff2png16(IUS_BAND band, png_structp png_ptr, png_infop info_ptr, int compression_method, int interlace_method) {
  png_uint_32 width, height, x, y;
  int bit_depth, num, byte_depth, status;
  png_uint_16pp row_pointers;

  ENTER_FUNCTION_DEBUG("biff2png16");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png16.\n",
		 status));
  }

  width      = (png_uint_32) Ixsize((IBAND) band);
  height     = (png_uint_32) Iysize((IBAND) band);
  bit_depth  = (int) Ipixsize(Ipixtyp((IBAND) band));
  byte_depth = (bit_depth + 7) / 8;

  FPRINTF5("width=%d, height=%d, bit_depth=%d byte_depth=%d\n",
	   width, height, bit_depth, byte_depth);

  png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth,
	       PNG_COLOR_TYPE_GRAY, interlace_method, compression_method,
	       PNG_FILTER_TYPE_DEFAULT);

  png_data_freer(png_ptr, info_ptr, PNG_DESTROY_WILL_FREE_DATA,
		 PNG_FREE_ROWS | PNG_FREE_TEXT);

  row_pointers =
    (png_uint_16pp) png_malloc(png_ptr, height * sizeof(png_uint_16p));
  for (num = 0; num < height; num++)
    row_pointers[num] =
      (png_uint_16p) png_malloc(png_ptr,
				width * byte_depth * sizeof(png_byte));

  png_set_rows(png_ptr, info_ptr, (png_bytepp) row_pointers);

  for (y = 0; y < height; y++) {
    for (x = 0; x < width; x++) {
      row_pointers[y][x] = band[y + 1][x + 1];
      if (y == 0) FPRINTF2("%hd ", row_pointers[y][x]);
    }
    if (y == 0) FPRINTF1("\n");
  }

  LEAVE_FUNCTION_DEBUG("biff2png16");

  return(0);

} /* biff2png16 */

int biff2png8p(IBAND band, png_structp png_ptr, png_infop info_ptr, Color_tab coltab, int colLen, int compression_method, int interlace_method) {
  png_uint_32 width, height, x, y;
  int bit_depth, num, byte_depth, status;
  png_color *palette;
  png_bytepp row_pointers;

  ENTER_FUNCTION_DEBUG("biff2png8p");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, lonjmp returned %d in biff2png8p.\n",
		 status));
  }

  if (colLen > XITE_PNG_PALETTE_SIZE)
    return(Error(3,
		 "BIFF colortable larger than palette supported by png.\n"));

  png_data_freer(png_ptr, info_ptr, PNG_DESTROY_WILL_FREE_DATA,
		 PNG_FREE_ROWS | PNG_FREE_TEXT | PNG_FREE_PLTE);

  palette = (png_colorp) png_malloc(png_ptr,
				    XITE_PNG_PALETTE_SIZE * sizeof(png_color));

  /* Initialize. */

  for (num = 0; num < XITE_PNG_PALETTE_SIZE; num++) {
      palette[num].red   = 0;
      palette[num].green = 0;
      palette[num].blue  = 0;
  }

  FPRINTF1("Palette initialized.\n");

  for (num = 0; num < colLen; num++) {
    int index = coltab[num].pixel & 0xff; /* Single-channel pixel-value. */

    /* BIFF colortable is 16-bit, png palette is 8-bit. */
    palette[index].red   = (coltab[index].red >> 8);
    palette[index].green = (coltab[index].green >> 8);
    palette[index].blue  = (coltab[index].blue >> 8);

    FPRINTF5("index=%d, red=%x, green=%x, blue=%x\n", index,
	     coltab[index].red, coltab[index].green, coltab[index].blue);

    FPRINTF5("index=%d, red=%x, green=%x, blue=%x\n", index,
	     palette[index].red, palette[index].green, palette[index].blue);
  }

  width      = (png_uint_32) Ixsize(band);
  height     = (png_uint_32) Iysize(band);
  bit_depth  = (int) Ipixsize(Ipixtyp(band));
  byte_depth = (bit_depth + 7) / 8;

  FPRINTF5("width=%d, height=%d, bit_depth=%d byte_depth=%d\n",
	   width, height, bit_depth, byte_depth);

  png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth,
	       PNG_COLOR_TYPE_PALETTE, interlace_method, compression_method,
	       PNG_FILTER_TYPE_DEFAULT);

  png_set_PLTE(png_ptr, info_ptr, palette, colLen);

  row_pointers = (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));
  for (num = 0; num < height; num++)
    row_pointers[num] =
      (png_bytep) png_malloc(png_ptr, width * byte_depth * sizeof(png_byte));

  png_set_rows(png_ptr, info_ptr, row_pointers);

  for (y = 0; y < height; y++) {
    for (x = 0; x < width; x++) {
      row_pointers[y][x] = band[y + 1][x + 1];
      if (y == 0) FPRINTF2("%hhd ", row_pointers[y][x]);
    }
    if (y == 0) FPRINTF1("\n");
  }

  LEAVE_FUNCTION_DEBUG("biff2png8p");

  return(0);

} /* biff2png8p */

int biff2png8c(IMAGE img, png_structp png_ptr, png_infop info_ptr, int compression_method, int interlace_method) {
  png_uint_32 width, height, x, y;
  IBAND rband, gband, bband;
  int bit_depth, num, byte_depth, num_channels, row_bytes, status;
  png_bytepp row_pointers;
  png_bytep rp;

  ENTER_FUNCTION_DEBUG("biff2png8c");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png8c.\n",
		 status));
  }

  if (Inbands((IMAGE) img) < 3)
    return(Error(3, "BIFF has less than three bands.\n"));

  rband        = (IBAND) img[1];
  gband        = (IBAND) img[2];
  bband        = (IBAND) img[3];
  width        = (png_uint_32) Ixsize(rband);
  height       = (png_uint_32) Iysize(rband);
  bit_depth    = (int) Ipixsize(Ipixtyp(rband));
  byte_depth   = (bit_depth + 7) / 8;
  num_channels = 3;
  row_bytes    = num_channels * width * byte_depth;

  FPRINTF6("width=%d, height=%d, bit_depth=%d, byte_depth=%d, row_bytes=%d\n",
	   width, height, bit_depth, byte_depth, row_bytes);

  png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth,
	       PNG_COLOR_TYPE_RGB, interlace_method, compression_method,
	       PNG_FILTER_TYPE_DEFAULT);

  png_data_freer(png_ptr, info_ptr, PNG_DESTROY_WILL_FREE_DATA,
		 PNG_FREE_ROWS | PNG_FREE_TEXT);

  row_pointers = (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));
  for (num = 0; num < height; num++)
    row_pointers[num] =
      (png_bytep) png_malloc(png_ptr, row_bytes * sizeof(png_byte));

  png_set_rows(png_ptr, info_ptr, row_pointers);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++) {
      *rp++ = rband[y + 1][x + 1];
      *rp++ = gband[y + 1][x + 1];
      *rp++ = bband[y + 1][x + 1];
    }
  }

  LEAVE_FUNCTION_DEBUG("biff2png8c");

  return(0);
  
} /* biff2png8c */

int biff2png16c(IUS_IMAGE img, png_structp png_ptr, png_infop info_ptr, int compression_method, int interlace_method) {
  png_uint_32 width, height, x, y;
  IUS_BAND rband, gband, bband;
  int bit_depth, num, byte_depth, num_channels, row_bytes, status;
  png_uint_16pp row_pointers;
  png_uint_16p rp;

  ENTER_FUNCTION_DEBUG("biff2png16c");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png16c.\n",
		 status));
  }

  if (Inbands((IMAGE) img) < 3)
    return(Error(3, "BIFF image has less than three bands.\n"));

  rband        = (IUS_BAND) img[1];
  gband        = (IUS_BAND) img[2];
  bband        = (IUS_BAND) img[3];
  width        = (png_uint_32) Ixsize((IBAND) rband);
  height       = (png_uint_32) Iysize((IBAND) rband);
  bit_depth    = (int) Ipixsize(Ipixtyp((IBAND) rband));
  byte_depth   = (bit_depth + 7) / 8;
  num_channels = 3;
  row_bytes    = num_channels * width * byte_depth;

  FPRINTF6("width=%d, height=%d, bit_depth=%d byte_depth=%d, row_bytes=%d\n",
	   width, height, bit_depth, byte_depth, row_bytes);

  png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth,
	       PNG_COLOR_TYPE_RGB, interlace_method, compression_method,
	       PNG_FILTER_TYPE_DEFAULT);

  png_data_freer(png_ptr, info_ptr, PNG_DESTROY_WILL_FREE_DATA,
		 PNG_FREE_ROWS | PNG_FREE_TEXT);

  row_pointers =
    (png_uint_16pp) png_malloc(png_ptr, height * sizeof(png_uint_16p));
  for (num = 0; num < height; num++)
    row_pointers[num] =
      (png_uint_16p) png_malloc(png_ptr, row_bytes * sizeof(png_byte));

  png_set_rows(png_ptr, info_ptr, (png_bytepp) row_pointers);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++) {
      *rp++ = rband[y + 1][x + 1];
      *rp++ = gband[y + 1][x + 1];
      *rp++ = bband[y + 1][x + 1];
    }
  }

  LEAVE_FUNCTION_DEBUG("biff2png16c");

  return(0);
  
} /* biff2png16c */

int biff2png(IMAGE img, png_structp png_ptr, png_infop info_ptr, int format, Color_tab coltab, int colLen, int compression_method, int interlace_method) {
  int num_text, status;
  IBAND band;
  long pixeltype;
  png_text *text_ptr;

  ENTER_FUNCTION_DEBUG("biff2png");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png.\n",
		 status));
  }
  if (!img) return(Error(3, "Image is null.\n"));

  band = img[1];

  if (format == XITE_PNG_GRAYSCALE) {
    pixeltype = Ipixtyp(band);
    
    switch(pixeltype) {
    case Iu_byte_typ:
      status = biff2png8(band, png_ptr, info_ptr, compression_method,
			 interlace_method);
      break;
    case Iu_short_typ:
      status = biff2png16((IUS_BAND) band, png_ptr, info_ptr,
			  compression_method, interlace_method);
      break;
    default: status = 1;
      break;
    }
    if (status)
      return(Error(3, "%s %s %s\n",
		   "biff2png: Cannot convert pixel type ",
		   Ipixname(pixeltype), "to png grayscale image."));
  } /* GRAYSCALE */
  else if (format == XITE_PNG_PALETTE) {
    pixeltype = Ipixtyp(band);
    
    switch(pixeltype) {
    case Iu_byte_typ:
      status = biff2png8p(band, png_ptr, info_ptr, coltab, colLen,
			  compression_method, interlace_method);
      break;
    default: status = 1;
      break;
    }
    if (status)
      return(Error(3, "%s %s %s\n",
		   "biff2png: Cannot convert pixel type ",
		   Ipixname(pixeltype), "to png palette image."));
  } /* PALETTE */
  else if (format == XITE_PNG_RGB) {
    pixeltype = Ipixtyp(band);

    switch(pixeltype) {
    case Iu_byte_typ:
      status = biff2png8c(img, png_ptr, info_ptr, compression_method,
			  interlace_method);
      break;
    case Iu_short_typ:
      status = biff2png16c((IUS_IMAGE) img, png_ptr, info_ptr,
			   compression_method, interlace_method);
      break;
    default: status = 1;
      break;
    }
    if (status)
      return(Error(3, "%s %s %s\n",
		   "biff2png: Cannot convert pixel type ",
		   Ipixname(pixeltype), "to png palette image."));
  } /* RGB */

  text_ptr = create_png_text(img, png_ptr, &num_text);
  png_set_text(png_ptr, info_ptr, text_ptr, num_text);

  LEAVE_FUNCTION_DEBUG("biff2png");

  return(0);

} /* biff2png */

#endif /* not MAIN */

/*P:biff2png*

________________________________________________________________

                biff2png
________________________________________________________________

Name:           biff2png - Convert an image from BIFF to PNG format

Syntax:         biff2png [-g] <BIFF-image> [<colortable filename>] <png-image>

Description:    Convert a BIFF image file to a PNG image file.

                With two filename arguments, the default is to convert a
		three-band BIFF image to PNG rgb and the first band of other
		BIFF images to grayscale PNG. With three filename arguments,
		convert the first BIFF band and the colortable to a paletted
		PNG.

Options:        &-g
                Convert the first band of the BIFF image to PNG graylevel,
		even if the BIFF image has exactly three bands. Only two
		filename arguments are allowed.

		&-i interlace
		Specify interlacing scheme, where 'interlace' may be one of

		    &&null
		    No interlacing.

		    &&adam7
		    Adam7 breaks down the image into seven smaller images of
		    varying size.

Restrictions:   For grayscale and rgb conversion, only pixeltypes unsigned byte
                and unsigned short are handled. For palette conversion,
		only pixeltype unsigned byte is handled.

See also:       'biff2png(3)', 'png2biff(1)', 'png2biff(3)'

Author:         Svein Bøe, BLAB, Ifi, UiO

Examples:       biff2png mona.img mona.png

Id:             $Id: biff2png.c,v 1.17 2014/01/31 12:53:48 svein Exp $
________________________________________________________________

*/

#ifdef MAIN
int main(int argc, char *argv[])
{
  IMAGE img = NULL;
  FILE *fp;
  png_structp png_ptr;
  png_infop info_ptr;
  char *png_filename, *biff_filename, *colfileName, *interlace;
  int status, gray, format, colLen = 0, transform = 0;
  int interlace_method = PNG_INTERLACE_NONE,
    compression_method = PNG_COMPRESSION_TYPE_DEFAULT;
  Color_tab coltab;

  ENTER_FUNCTION_DEBUG("main");

  Iset_message(TRUE);
  Iset_abort(TRUE);
  InitMessage(&argc, argv, xite_app_std_usage_text(
   "Usage: %s [-g] [-i <interlace >] <BIFF-image> [<colortable filename>] <png-image>\n\
      where \n\
        -g       : Convert the first band to PNG graylevel.\n\
      and <interlace> is one of \n\
        null     : Single image, i.e. one pass. The default. \n\
        adam7    : Seven images, i.e. seven passes\n\
      and <colortable filename> is needed only when '-p' is used.\n"));

  if (argc == 1) Usage(1, NULL);

  gray        = read_bswitch(&argc, argv, "-g");
  interlace   = read_switch(&argc, argv, "-i", 1, "null");

  if (argc > 4) Usage(2, "Too many command-line arguments.\n");
  if (argc < 3) Usage(2, "Missing command-line arguments.\n");

  biff_filename = argv[1];

  if      (argc == 4) format = XITE_PNG_PALETTE;
  else if (argc == 3) {
    if (gray) format = XITE_PNG_GRAYSCALE;
    else {
      img = Iread_image(biff_filename);
      if (!img) return(Error(3, "Failed in reading BIFF image from file %s.\n",
			     biff_filename));
      if (Inbands(img) == 3) format = XITE_PNG_RGB;
      else format = XITE_PNG_GRAYSCALE;
    }
  }
  else Usage(2, "Illegal command line.\n");

  if      (!strcmp(interlace, "null"))
    interlace_method = PNG_INTERLACE_NONE;
  else if (!strcmp(interlace, "adam7"))
    interlace_method = PNG_INTERLACE_ADAM7;
  else return(Usage(2, "Unknown interlace scheme %s.\n", interlace));

  FPRINTF1("About to create png_struct.\n");

  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!png_ptr)
    return(Error(3, "Could not create png image.\n"));

  FPRINTF1("png_struct created.\n");

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
    return(Error(3, "Could not create png info.\n"));
  }

  if (format == XITE_PNG_PALETTE) {
    IMAGE colImg;
    char *colName;

    colfileName  = argv[2];
    png_filename = argv[3];
    if ((colImg = Iread_image(colfileName)) == NULL)
      return(Error(3, "Failed in reading colortable file %s.\n", colfileName));

    if (Iysize(colImg[1]) > XITE_PNG_PALETTE_SIZE)
      return(Error(3,
		   "BIFF colortable in file %s too large for png palette.\n",
		   colfileName));

    if (color_convertfromimg(colImg, coltab, &colName, &colLen) != Iok)
      return(Error(3, "Failed in reading colortable file %s.\n", colfileName));
    if (colLen != Iysize(colImg[1]))
      return(Error(3, "Error in colortable file %s.\n", colfileName));
  } else {
    png_filename = argv[2];
  }

  if (((status = setjmp(png_jmpbuf(png_ptr))))) {
    png_destroy_write_struct(&png_ptr, &info_ptr);
    return(Error(3, "Error in libpng, longjmp returned %d in biff2png.\n",
		 status));
  }

  if (!img) img = Iread_image(biff_filename);
  if (!img) return(Error(3, "Failed in reading BIFF image from file %s.\n",
			 biff_filename));

  if ((status = biff2png(img, png_ptr, info_ptr, format, coltab, colLen,
			 compression_method, interlace_method)))
    return(Error(3, "Error in biff2png conversion.\n"));

  if (img) Idel_image(&img);
  img = NULL;

  if (strcmp(png_filename, "-") && strcmp(png_filename, "-1"))
    fp = fopen(png_filename, "wb");
  else fp = fdopen(1, "wb");

  if (!fp) return(Error(3, "Could not open file %s.\n", png_filename));

  png_init_io(png_ptr, fp);

  if (host_byte_order() == XITE_LITTLE_ENDIAN)
    transform |= PNG_TRANSFORM_SWAP_ENDIAN;

  png_write_png(png_ptr, info_ptr, transform, NULL);

  png_destroy_write_struct(&png_ptr, &info_ptr);
  fclose(fp);

  LEAVE_FUNCTION_DEBUG("main");

  return(0);
}


#endif /* MAIN */
