
/*C*
________________________________________________________________

        png2biff
        $Id: png2biff.c,v 1.12 2014/01/31 12:53:48 svein Exp $
        Copyright 2004, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/

static char *Id = "$Id: ";

#include <xite/includes.h>
#include <xite/message.h>
#include <xite/debug.h>
#include XITE_MALLOC_H
#include <xite/color.h>
#include <xite/xite_png.h>

#ifndef MAIN

/*F:png2biff8=png2biff*/
/*F:png2biff16=png2biff*/
/*F:png2biff8p=png2biff*/
/*F:png2biff8c=png2biff*/
/*F:png2biff16c=png2biff*/
/*F:png2biff*

________________________________________________________________

                png2biff
________________________________________________________________

Name:           png2biff, png2biff8, png2biff16, png2biff8p, png2biff8c,
                png2biff16c - Convert a PNG image to a BIFF image

Syntax:         | #include <xite/png.h>
                |
                | int png2biff( png_structp png_ptr, png_infop info_ptr,
		|    IMAGE *img, Color_tab coltab, int *colLen );
		|
                | int png2biff8( png_structp png_ptr, png_infop info_ptr,
		|    IBAND *band );
                |
		| int png2biff16( png_structp png_ptr, png_infop info_ptr,
		|    IUS_BAND *band );
		|
		| int png2biff8p( png_structp png_ptr, png_infop info_ptr,
		|    IBAND *band, Color_tab coltab, int *colLen );
		|
		| int png2biff8c( png_structp png_ptr, png_infop info_ptr,
		|    IMAGE *img );
		|
		| int png2biff16c( png_structp png_ptr, png_infop info_ptr,
		|    IMAGE *img );


Description:    'png2biff' is a wrapper function that decides which one of the
                other routines to use, depending on what kind the PNG image is
		(graylevel (8- or 16-bit unsigned), paletted (8-bit unsigned)
		or rgb (3 x 8-bit unsigned)). In the case of a paletted image,
		the parameters 'coltab' and 'colLen' reflect the palette.
		Any alpha channel is stripped.

		Byte-ordering between little-endian and big-endian machines
                is handled automatically.

		The conversion is according to the second edition of the
                PNG specification and uses the libpng library which must exist
                on your system, it is not supplied with XITE.

		Grayscale conversion routines:

		'png2biff8' converts a graylevel 8-bit unsigned PNG image to
		a BIFF unsigned byte band.

		'png2biff16' converts a graylevel 16-bit unsigned PNG image to
		a BIFF unsigned short band.

		Palette (pseudocolor) conversion routines:

		'png2biff8p' converts a paletted 8-bit unsigned PNG image to
		a BIFF unsigned byte band with the colortable/palette
		returned in 'coltab' with length given by 'colLen'.

		RGB conversion routines:

		'png2biff8c' converts a 3 x 8-bit unsigned PNG image or a
		4 x 8-bit unsigned PNG image to a three-band unsigned byte BIFF
		image. Alpha channel is stripped.

		'png2biff16c' converts a 3 x 16-bit unsigned PNG image or a
		4 x 16-bit unsigned PNG image to a three-band unsigned short
		BIFF image. Alpha channel is stripped.

Return value:   | 0 => ok
                | 3 => Error

See also:       'png2biff(1)', 'biff2png(1)', 'biff2png(3)'

Author:         Svein Bøe, BLAB, Ifi, UiO

Id:             $Id: png2biff.c,v 1.12 2014/01/31 12:53:48 svein Exp $

*/

int png2biff8(png_structp png_ptr, png_infop info_ptr, IBAND *band) {
  int i, pixel_size = 1, status;
  png_bytepp row_pointers;
  png_uint_32 width, height, x, y;
  IBAND bnd;
  png_bytep rp;

  ENTER_FUNCTION_DEBUG("png2biff8");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
    return(Error(3, "Error in libpng, lonjmp returned %d in png2biff8.\n",
		 status));
  }

  png_read_update_info(png_ptr, info_ptr);

  width  = png_get_image_width(png_ptr, info_ptr);
  height = png_get_image_height(png_ptr, info_ptr);

  FPRINTF3("width=%d, height=%d\n", width, height);

  row_pointers =
    (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));

  for (i = 0; i < height; i++)
    row_pointers[i] = png_malloc(png_ptr, width * pixel_size);

  png_read_image(png_ptr, row_pointers);

  bnd = Imake_band(Iu_byte_typ, width, height);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++)
      bnd[y + 1][x + 1] = *rp++;
  }

  *band = bnd;

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    png_free(png_ptr, rp);
  }

  png_free(png_ptr, row_pointers);

  LEAVE_FUNCTION_DEBUG("png2biff8");

  return(0);

} /* png2biff8 */

int png2biff16(png_structp png_ptr, png_infop info_ptr, IUS_BAND *band) {
  int i, pixel_size = 2;
  png_uint_16pp row_pointers;
  png_uint_32 width, height, x, y;
  IUS_BAND bnd;
  png_uint_16p rp;

  if (host_byte_order() == XITE_LITTLE_ENDIAN)
    png_set_swap(png_ptr);

  png_read_update_info(png_ptr, info_ptr);

  width  = png_get_image_width(png_ptr, info_ptr);
  height = png_get_image_height(png_ptr, info_ptr);

  FPRINTF3("width=%d, height=%d\n", width, height);

  row_pointers =
    (png_uint_16pp) png_malloc(png_ptr, height * sizeof(png_uint_16p));

  for (i = 0; i < height; i++)
    row_pointers[i] = png_malloc(png_ptr, width * pixel_size);

  png_read_image(png_ptr, (png_bytepp) row_pointers);

  bnd = (IUS_BAND) Imake_band(Iu_short_typ, width, height);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++)
      bnd[y + 1][x + 1] = *rp++;
  }

  *band = bnd;

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    png_free(png_ptr, rp);
  }

  png_free(png_ptr, row_pointers);

  return(0);

} /* png2biff16 */

int png2biff8p(png_structp png_ptr, png_infop info_ptr, IBAND *band,
	       Color_tab coltab, int *colLen) {
  int i, pixel_size = 1, status;
  png_bytepp row_pointers;
  png_uint_32 width, height, x, y;
  IBAND bnd;
  png_colorp palette;
  png_bytep rp;

  ENTER_FUNCTION_DEBUG("png2biff8p");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
    return(Error(3, "Error in libpng, longjmp returned %d in png2biff8p.\n",
		 status));
  }

  png_read_update_info(png_ptr, info_ptr);

  width  = png_get_image_width(png_ptr, info_ptr);
  height = png_get_image_height(png_ptr, info_ptr);

  FPRINTF3("width=%d, height=%d\n", width, height);

  row_pointers =
    (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));

  for (i = 0; i < height; i++)
    row_pointers[i] = png_malloc(png_ptr, width * pixel_size);

  png_read_image(png_ptr, row_pointers);

  bnd = Imake_band(Iu_byte_typ, width, height);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++)
      bnd[y + 1][x + 1] = *rp++;
  }

  *band = bnd;

  png_get_PLTE(png_ptr, info_ptr, &palette, colLen);

  if (*colLen > XITE_PNG_PALETTE_SIZE)
    return(Error(3, "png palette larger than BIFF colortable.\n"));

  for (i = 0; i < (*colLen); i++) {
    coltab[i].pixel = ShiftToRed(i) | ShiftToGreen(i) | ShiftToBlue(i);
    coltab[i].red   = (palette[i].red   << 8);
    coltab[i].green = (palette[i].green << 8);
    coltab[i].blue  = (palette[i].blue  << 8);
  }

  for (i = (*colLen); i < XITE_PNG_PALETTE_SIZE; i++) {
    coltab[i].pixel = ShiftToRed(i) | ShiftToGreen(i) | ShiftToBlue(i);
    coltab[i].red   = 0;
    coltab[i].green = 0;
    coltab[i].blue  = 0;
  }

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    png_free(png_ptr, rp);
  }

  png_free(png_ptr, row_pointers);

  LEAVE_FUNCTION_DEBUG("png2biff8p");

  return(0);

} /* png2biff8p */

int png2biff8c(png_structp png_ptr, png_infop info_ptr, IMAGE *img) {
  int i, pixel_size = 3, status, color_type;
  png_bytepp row_pointers;
  png_uint_32 width, height, x, y;
  IBAND rband, gband, bband;
  png_bytep rp;

  ENTER_FUNCTION_DEBUG("png2biff8c");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
    return(Error(3, "Error in libpng, longjmp returned %d in png2biff8c.\n",
		 status));
  }

  color_type = png_get_color_type(png_ptr, info_ptr);

  if (color_type & PNG_COLOR_MASK_ALPHA) {
    png_set_strip_alpha(png_ptr);
    Warning(2, "Neglecting alpha channel.\n");
  }

  png_read_update_info(png_ptr, info_ptr);

  width  = png_get_image_width(png_ptr, info_ptr);
  height = png_get_image_height(png_ptr, info_ptr);

  FPRINTF3("width=%d, height=%d\n", width, height);

  row_pointers =
    (png_bytepp) png_malloc(png_ptr, height * sizeof(png_bytep));

  for (i = 0; i < height; i++)
    row_pointers[i] = png_malloc(png_ptr, width * pixel_size);

  png_read_image(png_ptr, row_pointers);

  rband = Imake_band(Iu_byte_typ, width, height);
  gband = Imake_band(Iu_byte_typ, width, height);
  bband = Imake_band(Iu_byte_typ, width, height);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++) {
      rband[y + 1][x + 1] = *rp++;
      gband[y + 1][x + 1] = *rp++;
      bband[y + 1][x + 1] = *rp++;
    }
  }

  *img = Init_image(3, "");
  (*img)[1] = rband;
  (*img)[2] = gband;
  (*img)[3] = bband;

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    png_free(png_ptr, rp);
  }

  png_free(png_ptr, row_pointers);

  LEAVE_FUNCTION_DEBUG("png2biff8c");

  return(0);

} /* png2biff8c */

int png2biff16c(png_structp png_ptr, png_infop info_ptr, IMAGE *img) {
  int i, pixel_size = 6, status, color_type;
  png_uint_16pp row_pointers;
  png_uint_32 width, height, x, y;
  png_uint_16p rp;
  IUS_BAND rband, gband, bband;

  ENTER_FUNCTION_DEBUG("png2biff16c");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
    return(Error(3, "Error in libpng, longjmp returned %d in png2biff16c.\n",
		 status));
  }

  color_type = png_get_color_type(png_ptr, info_ptr);

  if (color_type & PNG_COLOR_MASK_ALPHA) {
    png_set_strip_alpha(png_ptr);
    Warning(2, "Neglecting alpha channel.\n");
  }

  if (host_byte_order() == XITE_LITTLE_ENDIAN)
    png_set_swap(png_ptr);

  png_read_update_info(png_ptr, info_ptr);

  width  = png_get_image_width(png_ptr, info_ptr);
  height = png_get_image_height(png_ptr, info_ptr);

  FPRINTF3("width=%d, height=%d\n", width, height);

  row_pointers =
    (png_uint_16pp) png_malloc(png_ptr, height * sizeof(png_uint_16p));

  for (i = 0; i < height; i++)
    row_pointers[i] = png_malloc(png_ptr, width * pixel_size);

  png_read_image(png_ptr, (png_bytepp) row_pointers);

  rband = (IUS_BAND) Imake_band(Iu_short_typ, width, height);
  gband = (IUS_BAND) Imake_band(Iu_short_typ, width, height);
  bband = (IUS_BAND) Imake_band(Iu_short_typ, width, height);

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    for (x = 0; x < width; x++) {
      rband[y + 1][x + 1] = *rp++;
      gband[y + 1][x + 1] = *rp++;
      bband[y + 1][x + 1] = *rp++;
    }
  }

  *img = Init_image(3, "");
  (*img)[1] = (IBAND) rband;
  (*img)[2] = (IBAND) gband;
  (*img)[3] = (IBAND) bband;

  for (y = 0; y < height; y++) {
    rp = row_pointers[y];
    png_free(png_ptr, rp);
  }

  png_free(png_ptr, row_pointers);

  LEAVE_FUNCTION_DEBUG("png2biff16c");

  return(0);

} /* png2biff16c */

int png2biff(png_structp png_ptr, png_infop info_ptr, IMAGE *img,
	     Color_tab coltab, int *colLen) {
  png_text *text_ptr;
  png_color_8p sig_bit;
  int bit_depth, color_type, num_comments, status = 0, i, free_title = 0;
  IBAND band;
  char *title = NULL;
  IMAGE tmp_img;

  ENTER_FUNCTION_DEBUG("png2biff");

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
    return(Error(3, "Error in libpng, longjmp returned %d in png2biff.\n",
		 status));
  }

  bit_depth  = png_get_bit_depth(png_ptr, info_ptr);
  color_type = png_get_color_type(png_ptr, info_ptr);

  FPRINTF3("bit_depth=%d, color_type=%d\n", bit_depth, color_type);

  if (bit_depth < 8)
    png_set_packing(png_ptr);

  if (png_get_sBIT(png_ptr, info_ptr, &sig_bit))
    png_set_shift(png_ptr, sig_bit);

  num_comments = png_get_text(png_ptr, info_ptr, &text_ptr, NULL);
  for (i = 0; i < num_comments; i++) {
    if (!strcmp(text_ptr[i].key, "Title")) title = text_ptr[i].text;
  }

  *colLen = 0;
  if (title == NULL) {
    title      = (char *) malloc(1);
    title[0]   = '\0';
    free_title = 1;
  }

  if (color_type == PNG_COLOR_TYPE_GRAY) {
    switch(bit_depth) {
    case 1:
    case 2:
    case 4:
    case 8:
      status = png2biff8(png_ptr, info_ptr, &band);
      break;
    case 16:
      status = png2biff16(png_ptr, info_ptr, (IUS_BAND *) &band);
      break;
    default: status = 1;
      break;
    }
    if (status)
      return(Error(3, "%s %ld %s\n",
		   "png2biff: Cannot convert grayscale bit-depth ",
		   bit_depth, "to BIFF band."));

    tmp_img    = Init_image(1, title);
    tmp_img[1] = band;
    *img = tmp_img;
  } else if (color_type == PNG_COLOR_TYPE_PALETTE) {
    status = png2biff8p(png_ptr, info_ptr, &band, coltab, colLen);
    tmp_img    = Init_image(1, title);
    tmp_img[1] = band;
    *img = tmp_img;
  } else if (color_type == PNG_COLOR_TYPE_RGB) {
    switch(bit_depth) {
    case 8:
      status = png2biff8c(png_ptr, info_ptr, img);
      break;
    case 16:
      status = png2biff16c(png_ptr, info_ptr, img);
      break;
    default: status = 1;
      break;
    }
    if (status)
      return(Error(3, "%s %ld %s\n",
		   "png2biff: Cannot convert rgb bit-depth ",
		   bit_depth, "to BIFF image."));
  }

  if (title && free_title) free(title);

  LEAVE_FUNCTION_DEBUG("png2biff");

  return(0);

} /* png2biff */

#endif /* not MAIN */

#ifdef MAIN

/*P:png2biff*

________________________________________________________________

                png2biff
________________________________________________________________

Name:           png2biff - Convert an image from PNG format to BIFF format

Syntax:         png2biff <png-image> <BIFF-image> [<colortable filename>]

Description:    Convert a PNG image file to a BIFF image file.

                With two filename arguments, the PNG image must be either
		graylevel or rgb. With three filename arguments, the PNG
		image must be paletted.

Options:        

Restrictions:   If an alpha channel is present in the PNG image, it will
                be stripped, and the remaining rgb channels converted.

Exit value:     | 0 => ok
                | 3 => Error

See also:       'png2biff(3)', 'biff2png(1)', 'biff2png(3)'

Author:         Svein Bøe, BLAB, Ifi, UiO

Examples:       png2biff mona.png mona.img

Id:             $Id: png2biff.c,v 1.12 2014/01/31 12:53:48 svein Exp $
________________________________________________________________

*/

int main(int argc, char *argv[]) {
  IMAGE img;
  FILE *fp;
  png_structp png_ptr;
  png_infop info_ptr, end_info;
  char *png_filename, *biff_filename, *colfileName;
  int status, is_png = 0, n_sig_bytes = 8, colLen;
  Color_tab coltab;
  png_bytep header;

  ENTER_FUNCTION_DEBUG("main");

  Iset_message(TRUE);
  Iset_abort(TRUE);
  InitMessage(&argc, argv, xite_app_std_usage_text(
   "Usage: %s <png-image> <BIFF-image> [<colortable filename>]\n\
      where <colortable filename> is required only for a palette image.\n"));

  if (argc == 1) Usage(1, NULL);
  if (argc > 4)  Usage(2, "Too many arguments.\n");
  if (argc < 3)  Usage(2, "Missing arguments.\n");

  png_filename  = argv[1];
  biff_filename = argv[2];

  FPRINTF3("png_filename=%s, biff_filename=%s\n", png_filename, biff_filename);

  if (strcmp(png_filename, "-") && strcmp(png_filename, "-0"))
    fp = fopen(png_filename, "rb");
  else
    fp = fdopen(0, "rb");

  if (!fp) {
    return(Error(3, "Could not open file %s.\n", png_filename));
  }

  FPRINTF1("File open\n");

  header = (png_bytep) malloc(8 * sizeof(png_byte));
  if (fread(header, 1, n_sig_bytes, fp) < n_sig_bytes) {
    return(Error(3, "Could not read the PNG file signature properly.\n"));
  }
  is_png = !png_sig_cmp(header, 0, n_sig_bytes);
  free(header);
  if (!is_png) {
    fclose(fp);
    return(Error(3, "The file %s is not a png file.\n", png_filename));
  }

  FPRINTF1("Signature checked.\n");

  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!png_ptr) {
    fclose(fp);
    return(Error(3, "Could not create png image.\n"));
  }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    png_destroy_read_struct(&png_ptr, (png_infopp) NULL, (png_infopp) NULL);
    fclose(fp);
    return(Error(3, "Could not create png info.\n"));
  }

  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
    fclose(fp);
    return(Error(3, "Could not create png end_info.\n"));
  }

  if ( (status = setjmp(png_jmpbuf(png_ptr))) ) {
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return(Error(3, "Error in libpng, longjmp returned %d in png2biff.\n",
		 status));
  }

  FPRINTF1("Ready to initialize io.\n");

  png_init_io(png_ptr, fp);

  FPRINTF1("Ready to advance passed signature bytes.\n");
  png_set_sig_bytes(png_ptr, n_sig_bytes);
  FPRINTF1("Ready to read PNG info.\n");
  png_read_info(png_ptr, info_ptr);

  FPRINTF1("Ready to convert.\n");

  if ((status = png2biff(png_ptr, info_ptr, &img, coltab, &colLen))) {
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return(Error(3, "Error in png2biff conversion, status=%d.\n", status));
  }

  png_read_end(png_ptr, end_info);

  Iwrite_image(img, biff_filename);

  FPRINTF1("biff-file written\n");

  Idel_image(&img);

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  fclose(fp);

  if (colLen > 0) {
    if (argc < 4) Usage(2, "Missing arguments.\n");

    colfileName = argv[3];
    FPRINTF2("colfileName=%s\n", colfileName);
    if ((status = color_write(colfileName, coltab, Ititle(img), colLen)))
      return(Error(3, "Error in writing colortable file %s.\n", colfileName));

    FPRINTF1("colortable-file written\n");
  } else if (argc > 3) Usage(2, "Too many arguments.\n");

  return(0);
}

#endif /* MAIN */
