
/*C*

________________________________________________________________

        regionAnalyse
        $Id: regionAnalyse.c,v 1.48 2014/01/31 12:53:49 svein Exp $
        Copyright 1990, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@ifi.uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

static char *Id = "$Id: regionAnalyse.c,v 1.48 2014/01/31 12:53:49 svein Exp $, Blab, UiO";

#include <xite/includes.h>
#include <xite/biff.h>
#include <stdlib.h>
#include XITE_STDIO_H
#include <xite/blab.h>
#include <xite/color.h>
#include <xite/compu4.h>
#include <xite/draw.h>
#include <xite/histo.h>
#include <xite/message.h>
#include <xite/region.h>
#include <xite/readarg.h>
#include <xite/statistics.h>
#include XITE_STRING_H
#include XITE_MALLOC_H

#define NEWc(n, t)	((t *) calloc(n, sizeof(t)))

#ifndef MAIN

#define TESTHSTAT { if (NOT hstat)					\
       { hstat = TRUE;							\
         if (band) regionHisto(band, reg, TRUE, histo);			\
	 else regionHisto(region_band, reg, TRUE, histo);		\
         hstatistics(histo, &minpix, &maxpix, &median, &meanpix, &stdev); }}

/*F:regionAnalyse*

________________________________________________________________

		regionAnalyse
________________________________________________________________

Name:		regionAnalyse - split BIFF image band into regions and return
                description of regions

Syntax:		| #include <xite/region.h>
                |
                | int regionAnalyse( IBAND region_band, IBAND band,
		|    region_behaviour *beh, region_properties *prop_in,
		|    region_properties ***prop_out, int *size,
		|    IBAND *outband );

Description:    'regionAnalyse' splits the band 'region_band' into disjunct
                homogenous regions and calculates a number of properties for
                every region or every set of regions (object). Which properties
		to calculate are given by non-zero member values of 'prop_in'.
		The result is returned by the array of structs 'prop_out'.
		'prop_out' is allocated by the function and is indexable from
		zero to 'size'. Element zero will always be NULL.

		The struct 'beh' modifes the "behavior" of the program.

		A number of properties (minpix, maxpix, xmean, ymean, moment,
		etc.) allow a second band 'band' to be specified. If
		specified, it is used for the calculation of these properties,
		while still using 'region_band' to define the extent of the
		regions. If 'band' is missing, 'region_band' is used to
		define the regions as well as to supply pixel values for these
		measurements.

Behaviour specification:
                The items below refer to members of the region_behaviour
		struct (defined in region.h). Some are in effect mere booleans,
		others supply values.

		&header
		If nonzero, print a header showing the contents of the table.

		&nc
		Neighbour connectedness: 4 or 8 (default 4).
		'nc' applies to objects (pixel value > 0).
		The opposite applies to the background (=0).

                &sort
		If nonzero, sort regions with respect to object number, store
		in an object array.

		&split
		If nonzero, split objects (except the background, the object
		with object number 0) so that every region forms an independent
		object.

		&cursorx
		If greater than zero, the horizontal coordinate of a cursor
		point ('x', 'y').

		&cursory
		If greater than zero, the vertical coordinate of a cursor point
		('x', 'y').
		
		&ifline
		If non-NULL, use only regions which make the expression
		'ifline' TRUE. 'ifline' is a logical expression. See also
		item 'outfile'.

		&&Variables available for use in 'expr'
		| N - Object number.
		| R - Region number.
		| A - Region area.
		| W - Region width.
		| H - Region height.
		| M - Region pixel mean.
		| CR - Region number of cursor point
		| CN - Object number of cursor point

		&&Operators available in 'expr'
		| + - / *
		| < <= == >= > != LT LE EQ GE GT NE
		| AND OR NOT
		
		&&Examples:
		| 'A > 50'
		| 'A * 0.75 > W * H AND W > H'
		| 'CR EQ R' (when cursorx and cursory are also given)

		-&&

		&outfile
		If non-NULL, used in combination with 'ifline' to make a new
		band 'outband' containing only the regions of 'region_band'
		which makes the 'ifline' TRUE. 'outband' will be NULL if
		not requested.

		&bg
		Set the background pixel value in 'outband' to 'bg'.
              
Property specification:
                The items below refer to members of the region_properties
		struct (defined in region.h). For the input parameter
		'prop_in', these are in effect mere booleans, telling which
		properties to calculate. For the output parameter 'prop_out',
		they return the property values.

		&objectnr
		Include object number.

		&regionnr
		Include region number.

		&area
		Measure and include the area.

		&perimeter
		Measure and include the perimeter.

		&xmin
		Measure and include xmin.

		&ymin
		Measure and include ymin.

		&xmax
		Measure and include xmax.

		&ymax
		Measure and include ymax.

		&height
		Measure and include the height of the region.

		&width
		Measure and include the width of the region.

		&xpoint
		Include the x coordinate of a point in the region (the
		leftmost point on the upper line).

		&ypoint
		Include the y coordinate of a point in the region (the
		leftmost point on the upper line).

		-&
		Property options acting on 'band' if present,
		otherwise on 'region_band':

		&minpix
		Measure and include minimum pixel value within region.

		&maxpix
		Measure and include maximum pixel value within region.

		&xmean
		Measure and include xmean, using 'band' as weights.

		&ymean
		Measure and include ymean, using 'band' as weights.

		&moment
		Measure and include the moment of order 'p'+'q', using 'band'
		if present. This option may be used repreatedly with different
		order arguments.

		&cmoment
		Measure and include the central moment of order 'p'+'q', using
		'band' if present. This option may be used repreatedly
		with different order arguments.

		&imoment
		7 invariant moments.

		&volume
		Measure and include the volume under 'band' if present.

		&mean
		Measure and include the mean of the pixel values within the
		region(s).

		&median
		Measure and include the median of the pixel values.

		&stdev
		Measure and include the standard deviation of the pixel values.

		-&
		Note: If using 'sort' together with 'regionnr', only the first
		region number in each object will be written.

Files:	        region.h
See also:       regionAnalyse(1), regionArea(3),
		regionConvexHull(3), regionCreate(3), regionDraw(3),
		regionFree(3), regionHistoUpdate(3), regionHisto(3),
		regionImoment(3), regionInsertYline(3), regionDeleteYline(3),
		regionEndInsert(3), regionXmean(3), regionYmean(3),
		regionMean(3), regionXmin(3), regionXmax(3), regionYmin(3),
		regionYmax(3), regionMinMax(3), regionCMoment(3),
		regionPMoment(3), regionMoment(3), regionPerform(3),
		regionPerimeter(3), regionIsPoint(3), regionPoint(3),
		regionSearch(3), regionSort(3), regionCopy(3), regionUnion(3),
		regionOverlap(3), regionNeighbor(3), regionVolume(3),
		regionYline(3)

Restrictions:   Input band must have pixel type unsigned byte.

Author:         Svein Bøe, based on previous main.

Id: 		$Id: regionAnalyse.c,v 1.48 2014/01/31 12:53:49 svein Exp $
________________________________________________________________

*/

int regionAnalyse (IBAND region_band, IBAND band, region_behaviour *beh,
		   region_properties *prop_in, region_properties ***prop_out,
		   int *size, IBAND *outband) {
  region **regs, **preg;
  region* reg;
  int nc, n;
  histogram histo;
  int minpix, maxpix, median, hstat;
  int cursorr, cursoro;
  double meanpix, stdev;
  int area, xmin, xmax, ymin, ymax;
  region_properties *prop, **prop_arr = NULL;

  if (region_band == NULL)
    return(Error(2, "Region-image band is NULL.\n"));

  /* behaviour */
  nc = beh->nc;
  if ((nc NE 4) AND (nc NE 8)) nc = 4;

  /* split into homogenous regions */
  regionYline(region_band, &regs, nc, beh->sort, beh->split, size);

  if (beh->cursorx > 0 && beh->cursory > 0) {
    for (n=0; n<=(*size); n++) {
      reg = regs[n];

      while (reg) {
	if (regionIsPoint(reg, 0, beh->cursorx, beh->cursory)) {
	  cursoro = reg->objectnr;
	  cursorr = reg->regionnr;
	}
	reg = reg->next;
      }
    }
  }

  if (beh->ifline) {
    for (n=0; n<=(*size); n++) {
      preg = &regs[n];
      reg = *preg;
      while (reg) {
	hstat=0;
	regionArea(reg, 0, &area);
	regionMinMax(reg, 0, &xmin, &ymin, &xmax, &ymax);
	TESTHSTAT;
	compute_line("N = %d", reg->objectnr);
	compute_line("R = %d", reg->regionnr);
	compute_line("A = %d", area);
	compute_line("W = %d", xmax - xmin + 1);
	compute_line("H = %d", ymax - ymin + 1);
	compute_line("CR = %d", cursorr);
	compute_line("CN = %d", cursoro);
	compute_line("M = %f", meanpix);
	hstat = compute_line(beh->ifline);
	if (hstat == 0) *preg = reg->next; else preg = &reg->next;
	reg = *preg;
      }
    }
  }

  /* Measure properties */

  /* Allocate array for property pointers (indexable from zero to size). */
  prop_arr = NEWc((*size + 1), region_properties *);

  for (n=0; n LE (*size); n++) {
    prop_arr[n] = NULL;

    if ((reg = regs[n])) {
      prop_arr[n] = NEWc(1, region_properties);
      prop = prop_arr[n];
      hstat=0;

      if (prop_in->objectnr) prop->objectnr = reg->objectnr;
      if (prop_in->regionnr) prop->regionnr = reg->regionnr;
      if (prop_in->area)     regionArea(reg, TRUE, &(prop->area));
      if (prop_in->perimeter)
	regionPerimeter(reg, nc, TRUE, &(prop->perimeter));
      if (prop_in->volume)   {
	if (band != NULL) regionVolume(band, reg, 0, &(prop->volume));
	else {
	  int area;
	  TESTHSTAT;
	  regionArea(reg, TRUE, &area);
	  prop->volume = (int) (meanpix * area + 0.5);
	}
      }

      if (prop_in->xmean && prop_in->ymean)
	regionMean(band, reg, TRUE, &(prop->xmean), &(prop->ymean));
      else if (prop_in->xmean) regionXmean(band, reg, TRUE, &(prop->xmean));
      else if (prop_in->ymean) regionYmean(band, reg, TRUE, &(prop->ymean));

      if (prop_in->xmin && prop_in->ymin && prop_in->xmax && prop_in->ymax)
	regionMinMax(reg, TRUE, &(prop->xmin), &(prop->ymin),
		     &(prop->xmax), &(prop->ymax));
      else if (prop_in->xmin)   regionXmin(reg, TRUE, &(prop->xmin));
      else if (prop_in->ymin)   prop->ymin = reg->ymin;
      else if (prop_in->xmax)   regionXmax(reg, TRUE, &(prop->xmax));
      else if (prop_in->ymax)   prop->ymax = reg->ymax;

      if (prop_in->height)   prop->height = reg->ymax - reg->ymin + 1;
      if (prop_in->width)    {
	int xmin, ymin, xmax, ymax;
	regionMinMax(reg, TRUE, &xmin, &ymin, &xmax, &ymax);
	prop->width = xmax - xmin + 1;
      }
      if (prop_in->minpix)   {
	TESTHSTAT;
	prop->minpix = minpix;
      }
      if (prop_in->maxpix)   {
	TESTHSTAT;
	prop->maxpix = maxpix;
      }
      if (prop_in->xpoint) prop->xpoint = reg->ylines[reg->ymin]->xmin;
      if (prop_in->ypoint) prop->ypoint = reg->ymin;
      if (prop_in->mean)     {
	TESTHSTAT;
	prop->mean = meanpix;
      }
      if (prop_in->median)   {
	TESTHSTAT;
	prop->median = median;
      }
      if (prop_in->stdev)    {
	TESTHSTAT;
	prop->stdev = stdev;
      }
      if (prop_in->moment)   {
	momentData *md_in = prop_in->moment;
	momentData *md_out_prev = NULL;
	while (md_in) {
	  momentData *md_out = NEWc(1, momentData);

	  md_out->p = md_in->p;
	  md_out->q = md_in->q;

	  regionMoment(band, reg, md_in->p, md_in->q, TRUE, &(md_out->moment));

	  prop->moment = md_out;
	  if (md_out_prev) md_out_prev->next = md_out;

	  md_in = md_in->next;
	  md_out_prev = md_out;
	}
      }
      if (prop_in->cmoment)  {
	momentData *md_in = prop_in->cmoment;
	momentData *md_out_prev = NULL;
	while (md_in) {
	  momentData *md_out = NEWc(1, momentData);

	  md_out->p = md_in->p;
	  md_out->q = md_in->q;

	  regionCMoment(band, reg, md_in->p, md_in->q, TRUE,
			&(md_out->moment));

	  prop->cmoment = md_out;
	  if (md_out_prev) md_out_prev->next = md_out;

	  md_in = md_in->next;
	  md_out_prev = md_out;
	}
      }
      if (prop_in->imoment)  {
	if (prop->imoment == NULL) prop->imoment = NEWc(8, double);

	TESTHSTAT;
	regionImoment(band, reg, TRUE, prop->imoment);
      }
    } /* if reg */
  } /* for n */

  if (beh->outfile != NULL) {
    *outband = Imake_band(Ipixtyp(region_band), Ixsize(region_band),
			  Iysize(region_band));

    drawBand(*outband, (double) beh->bg, 0.0);

    for (n=0; n LE (*size); n++) {
      if ((reg = regs[n])) {
	regionDraw(*outband, reg, 0, reg->objectnr);
      }
    }
  } else {
    *outband = NULL;
  }

  *prop_out = prop_arr;

  return(Iok);

} /* regionAnalyse() */

#endif /* !MAIN */

/*P:regionAnalyse*

________________________________________________________________

		regionAnalyse
________________________________________________________________

Name:		regionAnalyse - split BIFF image into regions and describe
                regions

Syntax:		regionAnalyse [<option>...] <region_image> [<inimage>]

Description:    'regionAnalyse' splits the image 'region_image' into disjunct
                homogenous regions and calculates a number of properties for
                every region or every set of regions (object). The result is
                written to stdout in ascii format. Some options modify the
                "behavior" of the program, others tell the system to measure
                and write a specific property for every region or object.
                Properties are written in the order in which they appear in
                the options list.

		A number of properties (minpix, maxpix, xmean, ymean, moment,
		etc.) allow a second image 'inimage' to be specified. If
		specified, it is used for the calculation of these properties,
		while still using 'region_image' to define the extent of the
		regions. If 'inimage' is missing, 'region_image' is used to
		define the regions as well as to supply pixel values for these
		measurements.

Behaviour options:
		&-header
		Print a header showing the contents of the table.

		&-nc nc
		Neighbour connectedness: 4 or 8 (default 4).
		'nc' applies to objects (pixel value > 0).
		The opposite applies to the background (=0).

                &-sort
		Sort regions with respect to object number, store in an
		object-array.

		&-split
		Split objects (except the background, the object with object
		number 0) so that every region forms an independent object.

		&-cursor x y
		Define a cursor point ('x', 'y').
		
		&-if expr
		Use only regions which make the expression 'expr' TRUE.
		'expr' is a logical expression. See also option '-o'.

		&&Variables available for use in 'expr'
		| N - Object number.
		| R - Region number.
		| A - Region area.
		| W - Region width.
		| H - Region height.
		| M - Region pixel mean.
		| CR - Region number of cursor point
		| CN - Object number of cursor point

		&&Operators available in 'expr'
		| + - / *
		| < <= == >= > != LT LE EQ GE GT NE
		| AND OR NOT
		
		&&Examples:
		| -if 'A > 50'
		| -if 'A * 0.75 > W * H AND W > H'
		| -cursor 45 70 -if 'CR EQ R'

		-&&

		&-o outimg
		Used in combination with '-if' to make a new region-image
		containing only the regions which makes the if-expr TRUE.

		&-bg val
		Set the background pixel value in 'outimg' to 'val'. 
              
Property options:
		&-objnr
		Write object number.

		&-regnr
		Write region number.

		&-area
		Measure and write the area.

		&-perimeter
		Measure and write the perimeter.

		&-xmin
		Measure and write xmin.

		&-ymin
		Measure and write ymin.

		&-xmax
		Measure and write xmax.

		&-ymax
		Measure and write ymax.

		&-minmax
		Measure and write xmin, ymin, xmax and ymax.

		&-height
		Measure and write the height of the region.

		&-width
		Measure and write the width of the region.

		&-point
		Write the x and y coordinates of a point in the region (the
		leftmost point on the upper line).

		-&
		Property options acting on 'inimage' if present,
		otherwise on 'region_image':

		&-minpix
		Measure and write minimum pixel value within region.

		&-maxpix
		Measure and write maximum pixel value within region.

		&-minmaxpix
		Measure and write minimum and maximum pixel values within
		region.

		&-xmean
		Measure and write xmean, using 'inimage' as weights.

		&-ymean
		Measure and write ymean, using 'inimage' as weights.

		&-xymean
		Measure and write xmean and ymean, using 'inimage' as weights.

		&-moment p q
		Measure and write the moment of order 'p'+'q', using 'inimage'
		if present. This option may be used repreatedly with different
		order arguments.

		&-cmoment p q
		Measure and write the central moment of order 'p'+'q', using
		'inimage' if present. This option may be used repreatedly
		with different order arguments.

		&-imoment
		7 invariant moments.

		&-volume
		Measure and write the volume under 'inimage' if present.

		&-mean
		Measure and write the mean of the pixel values within the
		region(s).

		&-median
		Measure and write the median of the pixel values.

		&-stdev
		Measure and write the standard deviation of the pixel values.

		-&
		Note: If using '-sort' together with '-regnr', only the first
		region number in each object will be written.

Files:	        region.h
See also:       regionAnalyse(3), regionArea(3),
		regionConvexHull(3), regionCreate(3), regionDraw(3),
		regionFree(3), regionHistoUpdate(3), regionHisto(3),
		regionImoment(3), regionInsertYline(3), regionDeleteYline(3),
		regionEndInsert(3), regionXmean(3), regionYmean(3),
		regionMean(3), regionXmin(3), regionXmax(3), regionYmin(3),
		regionYmax(3), regionMinMax(3), regionCMoment(3),
		regionPMoment(3), regionMoment(3), regionPerform(3),
		regionPerimeter(3), regionIsPoint(3), regionPoint(3),
		regionSearch(3), regionSort(3), regionCopy(3), regionUnion(3),
		regionOverlap(3), regionNeighbor(3), regionVolume(3),
		regionYline(3)

Restrictions:   Input image must have bands with pixel type unsigned byte.

Author:		Tor Lønnestad, BLAB, Ifi, UiO.
                Some additional features: Otto Milvang

Examples:       | regionAnalyse -header -if 'A > 50' -regnr -area \\
                |     -width monat.img > mona.txt

Id: 		$Id: regionAnalyse.c,v 1.48 2014/01/31 12:53:49 svein Exp $
________________________________________________________________

*/

#ifdef MAIN

#define MAXSWITCHES	100

#define	OBJNR		1
#define	REGNR		2
#define	AREA		3
#define	PERIMETER	4
#define	VOLUME		5
#define XMEAN		6
#define YMEAN		7
#define	XYMEAN		8
#define	MOMENT		9
#define	CMOMENT		10
#define XMIN		11
#define YMIN		12
#define XMAX		13
#define YMAX		14
#define	MINMAX		15
#define HEIGHT		16
#define WIDTH		17
#define MINPIX		18
#define MAXPIX		19
#define	MINMAXPIX	20
#define	POINT		21
#define	MEAN		22
#define	MEDIAN		23
#define	STDEV		24
#define	IMOMENT		25

#define nilc		((char*)0)
 
typedef struct {
  char* swPos;
  int swType;
} Switch;

#ifndef FUNCPROTO
static momentData* momData(s)
char* s;
#else /* FUNCPROTO */
static momentData* momData(char *s)
#endif /* FUNCPROTO */
{
  momentData* md;

  md = NEWc(1, momentData);
  if (!md) fprintf(stderr, "Error!\n");

  if (sscanf(s, "%d%d", &(md->p), &(md->q)) != 2) {
    printf("Error: bad arguments to moment\n"); return(0);
  }
  return(md);

} /* momData */

#ifndef FUNCPROTO
static void wrInteger(width, number)
int width, number;
#else /* FUNCPROTO */
static void wrInteger(int width, int number)
#endif /* FUNCPROTO */
{
  char format[20];
  sprintf(format, " %%%dd ", width);
  printf(format, number);

} /* wrInteger */

#ifndef FUNCPROTO
static void wrDouble(width, precision, number)
int width, precision;
double number;
#else /* FUNCPROTO */
static void wrDouble(int width, int precision, double number)
#endif /* FUNCPROTO */
{
  char format[20];
  sprintf(format, " %%%d.%df ", width, precision);
  printf(format, number);

} /* wrDouble */

#ifndef FUNCPROTO
static void wrIquad(width, n1, n2, n3, n4)
int width, n1, n2, n3, n4;
#else /* FUNCPROTO */
static void wrIquad(int width, int n1, int n2, int n3, int n4)
#endif /* FUNCPROTO */
{
  char format[20];
  sprintf(format, " %%%dd  %%%dd  %%%dd  %%%dd ",
	  width, width, width, width);
  printf(format, n1, n2, n3, n4);

} /* wrIquad */

#ifndef FUNCPROTO
static void wrIpair(width, n1, n2)
int width, n1, n2;
#else /* FUNCPROTO */
static void wrIpair(int width, int n1, int n2)
#endif /* FUNCPROTO */
{
  char format[20];
  sprintf(format, " %%%dd  %%%dd ", width, width);
  printf(format, n1, n2);

} /* wrIpair */

#ifndef FUNCPROTO
static void wrDpair(width, precision, n1, n2)
int width, precision;
double n1, n2;
#else /* FUNCPROTO */
static void wrDpair(int width, int precision, double n1, double n2)
#endif /* FUNCPROTO */
{
  char format[20];
  sprintf(format, " %%%d.%df  %%%d.%df ", width, precision, width, precision);
  printf(format, n1, n2);

} /* wrDpair */

#ifndef FUNCPROTO
static void wrObjnr(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrObjnr(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->objectnr);
}

#ifndef FUNCPROTO
static void wrRegnr(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrRegnr(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->regionnr);
}

#ifndef FUNCPROTO
static void wrArea(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrArea(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(7, prop->area);
}

#ifndef FUNCPROTO
static void wrPerim(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrPerim(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->perimeter);
}

#ifndef FUNCPROTO
static void wrVolume(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrVolume(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(10, prop->volume);
}

#ifndef FUNCPROTO
static void wrXmean(band, prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrXmean(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->xmean);
}

#ifndef FUNCPROTO
static void wrYmean(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrYmean(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->ymean);
}

#ifndef FUNCPROTO
static void wrXYmean(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrXYmean(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDpair(10, 3, prop->xmean, prop->ymean);
}

#ifndef FUNCPROTO
static void wrMoment(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMoment(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->moment->moment);
}

#ifndef FUNCPROTO
static void wrCMoment(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrCMoment(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->cmoment->moment);
}

#ifndef FUNCPROTO
static void wrImoment(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrImoment(region_properties *prop)
#endif /* FUNCPROTO */
{
  int i;

  for(i = 1; i <= 7; i++)
    printf(prop->imoment[i] * prop->imoment[i] < 0.000001 ?
	   " %9.3g " : " %9.6f ", prop->imoment[i]);
}

#ifndef FUNCPROTO
static void wrXmin(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrXmin(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->xmin);
}

#ifndef FUNCPROTO
static void wrYmin(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrYmin(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->ymin);
}

#ifndef FUNCPROTO
static void wrXmax(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrXmax(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->xmax);
}

#ifndef FUNCPROTO
static void wrYmax(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrYmax(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->ymax);
}

#ifndef FUNCPROTO
static void wrMinMax(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMinMax(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrIquad(5, prop->xmin, prop->ymin, prop->xmax, prop->ymax);

}

#ifndef FUNCPROTO
static void wrHeight(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrHeight(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->height);
}

#ifndef FUNCPROTO
static void wrWidth(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrWidth(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->width);
}

#ifndef FUNCPROTO
static void wrMinPix(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMinPix(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->minpix);
}

#ifndef FUNCPROTO
static void wrMaxPix(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMaxPix(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->maxpix);
}

#ifndef FUNCPROTO
static void wrMinMaxPix(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMinMaxPix(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrIpair(5, prop->minpix, prop->maxpix);
}

#ifndef FUNCPROTO
static void wrPoint(prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrPoint(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrIpair(5, prop->xpoint, prop->ypoint);
}

#ifndef FUNCPROTO
static void wrMean(prop)
double meanpix;
#else /* FUNCPROTO */
static void wrMean(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->mean);
}

#ifndef FUNCPROTO
static void wrMedian(region_properties *prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrMedian(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrInteger(5, prop->median);
}

#ifndef FUNCPROTO
static void wrStdev(region_properties *prop)
region_properties *prop;
#else /* FUNCPROTO */
static void wrStdev(region_properties *prop)
#endif /* FUNCPROTO */
{
  wrDouble(10, 3, prop->stdev);
}

static char *headers[] = {
  "",
  " OBJNR ",
  " REGNR ",
  "  AREA   ",
  " PERIM ",
  "   VOLUME   ",
  "   XMEAN    ",
  "   YMEAN    ",
  "   XMEAN       YMEAN    ",
  "   MOMENT   ",
  "  CMOMENT   ",
  " XMIN  ",
  " YMIN  ",
  " XMAX  ",
  " YMAX  ",
  " XMIN   YMIN   XMAX   YMAX  ",
  " HEIGHT",
  " WIDTH ",
  " MINPIX",
  " MAXPIX",
  " MINPIX MAXPIX",
  " XPOINT YPOINT",
  "    MEAN    ",
  " MEDIAN",
  "    STDEV   ",
  "   IMOM 1     IMOM 2     IMOM 3     IMOM 4     IMOM 5     IMOM 6     IMOM 7  ",
};

#ifndef FUNCPROTO
static void wrHeader(sws, perm, nsw)
Switch* sws;
int *perm, nsw;
#else /* FUNCPROTO */
static void wrHeader(Switch *sws, int *perm, int nsw)
#endif /* FUNCPROTO */
{
  int sw, hlen=0;

  printf("#");
  FOR (sw=0; sw LT nsw; sw++)
    printf("%s", headers[sws[perm[sw]].swType]);
    hlen += strlen(headers[sws[perm[sw]].swType]);
  ENDFOR;
  printf("\n#");
  for (sw=0; sw LT hlen; sw++) printf("-");
  printf("\n");
}

#ifndef FUNCPROTO
static void permutate(sws, perm, nsw)
Switch* sws;
int *perm, nsw;
#else /* FUNCPROTO */
static void permutate(Switch *sws, int *perm, int nsw)
#endif /* FUNCPROTO */
{
  int a, b, min;
  for (a=0; a LT nsw; a++) perm[a] = a;
  FOR (a=0; a LT nsw-1; a++)
    min = a;
    for (b=a+1; b LT nsw; b++)
      if (sws[perm[b]].swPos LT sws[perm[min]].swPos) min = b;
    b = perm[min];
    perm[min] = perm[a];
    perm[a] = b;
  ENDFOR;
}

static void free_region_properties(region_properties *prop) {
  if (!prop) return;

  if (prop->moment) {
    free(prop->moment);
    prop->moment = NULL;
  }
  if (prop->cmoment) {
    free(prop->cmoment);
    prop->cmoment = NULL;
  }
  if (prop->imoment) {
    free(prop->imoment);
    prop->imoment = NULL;
  }
  free(prop);

  return;
}

static void free_region_properties_arr(region_properties **prop, int size) {
  int i;

  for (i = 0; i <= size; i++) {
    free_region_properties(prop[i]);
    prop[i] = NULL;
  }

  free(prop);
}

#ifndef FUNCPROTO
int main(argc, argv)
int argc;
char *argv[];
#else /* FUNCPROTO */
int main(int argc, char **argv)
#endif /* FUNCPROTO */
{
  IMAGE region_img, img;
  IBAND region_band, band, outband;
  Switch switches[MAXSWITCHES];
  int perm[MAXSWITCHES]; /* array for permutation of switches */
  int size, swNo = 0, status;
  char *args, *cursorline;
  region_behaviour *beh   = NEWc(1, region_behaviour);
  region_properties *prop = NEWc(1, region_properties);
  region_properties **prop_arr;

  InitMessage(&argc, argv, xite_app_std_usage_text(
    "Usage: %s [<option>...] <region_image> [<inimage>]\n\
       See manual for the numerous options.\n"));

  if (argc == 1) Usage(1, NULL);
  args = argvOptions(argc, argv);

  /* behaviour options */
  beh->header  = read_bswitch(&argc, argv, "-header");
  beh->nc      = read_iswitch(&argc, argv, "-nc", 4);
  if ((beh->nc NE 4) AND (beh->nc NE 8)) beh->nc = 4;
  beh->sort    = read_bswitch(&argc, argv, "-sort");
  beh->split   = read_bswitch(&argc, argv, "-split");
  beh->ifline  = read_switch(&argc, argv, "-if", 1, NULL);
  beh->outfile = read_switch(&argc, argv, "-o", 1, NULL);
  cursorline   = read_switch(&argc, argv, "-cursor", 2, "0 0");
  sscanf(cursorline, "%d%d", &(beh->cursorx), &(beh->cursory));
  beh->bg      = read_iswitch(&argc, argv, "-bg", 0);

  /* property switches */
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-objnr", FALSE, nilc)))
    { prop->objectnr = 1; switches[swNo++].swType = OBJNR; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-regnr", FALSE, nilc)))
    { prop->regionnr = 1; switches[swNo++].swType = REGNR; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-area", FALSE, nilc)))
    { prop->area = 1; switches[swNo++].swType = AREA; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-perimeter", FALSE, nilc)))
    { prop->perimeter = 1; switches[swNo++].swType = PERIMETER; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-volume", FALSE, nilc)))
    { prop->volume = 1; switches[swNo++].swType = VOLUME; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-xmean", FALSE, nilc)))
    { prop->xmean = 1; switches[swNo++].swType = XMEAN; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-ymean", FALSE, nilc)))
    { prop->ymean = 1; switches[swNo++].swType = YMEAN; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-xymean", FALSE, nilc)))
    { prop->xmean = 1; prop->ymean = 1; switches[swNo++].swType = XYMEAN; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-xmin", FALSE, nilc)))
    { prop->xmin = 1; switches[swNo++].swType = XMIN; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-ymin", FALSE, nilc)))
    { prop->ymin = 1; switches[swNo++].swType = YMIN; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-xmax", FALSE, nilc)))
    { prop->xmax = 1; switches[swNo++].swType = XMAX; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-ymax", FALSE, nilc)))
    { prop->ymax = 1; switches[swNo++].swType = YMAX; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-minmax", FALSE, nilc))) {
    prop->xmin = 1; prop->ymin = 1; prop->xmax = 1; prop->ymax = 1;
    switches[swNo++].swType = MINMAX;
  }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-height", FALSE, nilc)))
    { prop->height = 1; switches[swNo++].swType = HEIGHT; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-width", FALSE, nilc)))
    { prop->width = 1; switches[swNo++].swType = WIDTH; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-minpix", FALSE, nilc)))
    { prop->minpix = 1; switches[swNo++].swType = MINPIX; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-maxpix", FALSE, nilc)))
    { prop->maxpix = 1; switches[swNo++].swType = MAXPIX; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-minmaxpix", FALSE, nilc))) {
    prop->minpix = 1; prop->maxpix = 1; switches[swNo++].swType = MINMAXPIX;
  }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-point", FALSE, nilc)))
    { prop->xpoint = 1; prop->ypoint = 1; switches[swNo++].swType = POINT; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-mean", FALSE, nilc)))
    { prop->mean = 1; switches[swNo++].swType = MEAN; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-median", FALSE, nilc)))
    { prop->median = 1; switches[swNo++].swType = MEDIAN; }
  if ((switches[swNo].swPos = read_switch(&argc, argv, "-stdev", FALSE, nilc)))
    { prop->stdev = 1; switches[swNo++].swType = STDEV; }
  if ((switches[swNo].swPos =
       read_switch(&argc, argv, "-imoment", FALSE, nilc)))
    { prop->imoment = NEWc(1, double); switches[swNo++].swType = IMOMENT; }
  while ((switches[swNo].swPos =
	  read_switch(&argc, argv, "-moment", 2, nilc))) {
    momentData *md = momData(switches[swNo].swPos);

    switches[swNo].swType = MOMENT;

    if (prop->moment == NULL) prop->moment = md;
    else {
      momentData *mdp = prop->moment;

      while (mdp->next) {
	mdp = mdp->next;
      }
      mdp->next = md;
    }
    swNo++;
  }
  while ((switches[swNo].swPos =
	  read_switch(&argc, argv, "-cmoment", 2, nilc))) {
    momentData *md = momData(switches[swNo].swPos);

    switches[swNo].swType = CMOMENT;

    if (prop->cmoment == NULL) prop->cmoment = md;
    else {
      momentData *mdp = prop->cmoment;

      while (mdp->next) {
	mdp = mdp->next;
      }
      mdp->next = md;
    }
    swNo++;
  }

  if ((argc LT 2) OR (argc GT 3))
    Usage(2, "Illegal number of arguments.\n");

  if (swNo EQ 0 && (beh->ifline EQ NULL OR beh->outfile EQ NULL))
    exit(Error(3, "Really? No property switches? - OK.\n"));

  /* sort acording to switch position - swPos */
  if (swNo > 0) permutate(switches, perm, swNo);

  /* read images */
  region_img = Iread_image(argv[1]);
  if (!region_img) exit(Error(4,"Can't read file %s\n", argv[1]));
  region_band = region_img[1];
  if (argc EQ 3) {
    img = Iread_image(argv[2]);
    if (!img) exit(Error(5, "Can't read file %s\n", argv[2]));
    band = img[1];
  } else {
    band = NULL;
  }

  status = regionAnalyse(region_band, band, beh, prop, &prop_arr, &size,
			 &outband);
  free_region_properties(prop);

  if (status) {
    return(Error(2, "Error in regionAnalyse().\n"));
  }

  if (beh->header) wrHeader(switches, perm, swNo);

  /* Write properties */
  if (swNo > 0) {
    int swn, n;
    for (n=0; n LE size; n++) {
      if ((prop = prop_arr[n])) {
	for (swn = 0; swn LT swNo; swn++) {
	  switch (switches[perm[swn]].swType) {
	  case OBJNR:	  wrObjnr(prop);     break;
	  case REGNR:	  wrRegnr(prop);     break;
	  case AREA:	  wrArea(prop);	     break;
	  case PERIMETER: wrPerim(prop);     break;
	  case VOLUME:    wrVolume(prop);    break;
	  case XMEAN:	  wrXmean(prop);     break;
	  case YMEAN:	  wrYmean(prop);     break;
	  case XYMEAN:    wrXYmean(prop);    break;
	  case MOMENT:    wrMoment(prop);    break;
	  case CMOMENT:   wrCMoment(prop);    break;
	  case XMIN:	  wrXmin(prop);      break;
	  case YMIN:	  wrYmin(prop);      break;
	  case XMAX:	  wrXmax(prop);      break;
	  case YMAX:	  wrYmax(prop);      break;
	  case MINMAX:    wrMinMax(prop);    break;
	  case HEIGHT:    wrHeight(prop);    break;
	  case WIDTH:     wrWidth(prop);     break;
	  case MINPIX:    wrMinPix(prop);    break;
	  case MAXPIX:    wrMaxPix(prop);    break;
	  case MINMAXPIX: wrMinMaxPix(prop); break;
	  case POINT:     wrPoint(prop);     break;
	  case MEAN:	  wrMean(prop);      break;
	  case MEDIAN:    wrMedian(prop);    break;
	  case STDEV:     wrStdev(prop);     break;
	  case IMOMENT:   wrImoment(prop);   break;
	  default: Warning(6, "Error: bad switch type: %d\n",
			   switches[perm[swn]].swType);
	  break;
	  }
	}
	printf("\n");
      }
    }
  }

  if (beh->outfile) {
    region_img[1] = outband;
    Ihistory(region_img, "regionAnalyse", beh->ifline);
    Iwrite_image(region_img, beh->outfile);
  }

  free(beh);
  free_region_properties_arr(prop_arr, size);

  return(0);
}

#endif /* MAIN */
